/*
wxPopList.cpp

wxWidgets ports of dialog windows
*/

#include <stdio.h>
#include <wx/wxprec.h>

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif

#include <wx/utils.h>

#include "xppaut.h"
#include "xppRunWindow_wx.h"
#include "xppPlotCanvas_wx.h"

#include "wxPopList.h"

#define MAX_LEN_SBOX 25

extern xppaut *xppautPtr;

int do_string_box(int n, int rows, int cols, char *title, char **names, char values[][MAX_LEN_SBOX], int maxchar)
{
  wxString tmp;
  tmp.Printf(_T("%s"), title);
  wxDialog *dlg = new wxDialog(NULL, -1, tmp, wxDefaultPosition,
                  wxDefaultSize);
  wxBoxSizer *topSizer = new wxBoxSizer(wxVERTICAL);
  wxBoxSizer *vertSizers[2*cols];
  wxBoxSizer *horizSizers[rows*cols];
  wxBoxSizer *buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  wxBoxSizer *yetAnotherSizer = new wxBoxSizer(wxHORIZONTAL);
  wxStaticText *labels[n];
  wxTextCtrl *inputs[n];
  wxString myValues[n];
  int i, j, k;

  wxButton *okButton = new wxButton(dlg, wxID_OK, _T("OK"));
  wxButton *cancelButton = new wxButton(dlg, wxID_CANCEL, _T("Cancel"));

  for(i=0; i<n; i++)
    myValues[i].Printf(_T("%s"), values[i]);

  for(i=0; i<cols; i++)
  {
    vertSizers[i] = new wxBoxSizer(wxVERTICAL);
    for(j=0; j<rows; j++)
    {
      k = i*rows+j;
      if(k == n)
        break;
      horizSizers[k] = new wxBoxSizer(wxHORIZONTAL);
      tmp.Printf(_T("%s:"), names[k]);
      labels[k] = new wxStaticText(dlg, -1, tmp,
                        wxDefaultPosition, wxDefaultSize);
      inputs[k] = new wxTextCtrl(dlg, -1, _T(""), wxDefaultPosition, wxSize(60,20),
                  0, wxTextValidator(wxFILTER_NONE, &myValues[k]));
      inputs[k]->SetMaxLength(MAX_LEN_SBOX);
      horizSizers[k]->Add(labels[k], 1, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
      horizSizers[k]->Add(inputs[k], 0);
      vertSizers[i]->Add(horizSizers[k], 1, wxALIGN_RIGHT);
    }
    yetAnotherSizer->Add(vertSizers[i]);
  }

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  topSizer->Add(yetAnotherSizer, 1);
  topSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL);

  dlg->SetSizer(topSizer);

  topSizer->SetSizeHints(dlg);

  if(dlg->ShowModal() == wxID_OK)
  {
    for(i=0; i<n; i++)
    {
      strcpy(values[i], myValues[i].mb_str());
    }

    dlg->Destroy();
    return 1;
  }
  else
  {
    dlg->Destroy();
    return 0;
  }
}

int new_string(char *name, char *value)
{
  int status;
  char values[1][MAX_LEN_SBOX];

  strcpy(values[0],value);
  status = do_string_box(1, 1, 1, "XPPAUT", &name, values, MAX_LEN_SBOX);

  if(status)
  {
    strcpy(value, values[0]);
    return 1;
  }
  else
    return 0;
}

int get_mouse_xy(int *x, int *y)
{
  xppautPtr->runWindow->GetCurrentCanvas()->UserClicked(x, y);
  return 1;
}

int rubber(int *x1, int *y1, int *x2, int *y2)
{
  xppautPtr->runWindow->GetCurrentCanvas()->UserBox(x1, y1, x2, y2);
  return 1;
}

void err_msg(char *string)
{
  wxString s;
  s.Printf(_T("%s"), string);
  wxMessageDialog dlg(NULL, s, _T("Error"),
    wxOK, wxDefaultPosition);

  dlg.ShowModal();
}

char TwoChoice (char *c1, char *c2, char *q, char *key)
{
  wxString choice1, choice2, question;
  choice1.Printf(_T("%s"), c1);
  choice2.Printf(_T("%s"), c2);
  question.Printf(_T("%s"), q);

  wxString title;
  title.Printf(_T("%s"), _T("???"));

  wxDialog *dlg = new wxDialog(NULL, -1, title);

  wxStaticText *qStaticText = new wxStaticText(dlg, -1, question, wxDefaultPosition);

  wxButton *choice1Btn = new wxButton(dlg, wxID_OK, choice1);
  wxButton *choice2Btn = new wxButton(dlg, wxID_CANCEL, choice2);

  wxBoxSizer *dlgSizer = new wxBoxSizer(wxVERTICAL);
  wxBoxSizer *buttonSizer = new wxBoxSizer(wxHORIZONTAL);

  buttonSizer->Add(choice1Btn, 0, wxALL, 3);
  buttonSizer->Add(choice2Btn, 0, wxALL, 3);

  dlgSizer->Add(qStaticText, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER | wxALL, 3);
  dlgSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  dlg->SetSizer(dlgSizer);
  dlgSizer->SetSizeHints(dlg);

  if(dlg->ShowModal() == wxID_OK)
  {
    dlg->Destroy();
    return key[0];
  }
  else
  {
    dlg->Destroy();
    return key[1];
  }
}
