/*
xppAutoCanvas_wx.cpp

Creates a canvas for plotting auto diagrams
*/

#include <wx/wxprec.h>

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif

#include "Ccode/autow.h"

#include "interfaceCPP_wx.h"
#include "xppAutoWindow_wx.h"
#include "xppAutoCanvas_wx.h"

//wxClientDC *autoDC;
//wxPen *autoPen;

extern AUTOWIN *AutoW;

IMPLEMENT_DYNAMIC_CLASS(xppAutoCanvas, wxPanel)

BEGIN_EVENT_TABLE(xppAutoCanvas, wxPanel)
  EVT_PAINT(  xppAutoCanvas::OnPaint)
  EVT_MOUSE_EVENTS( xppAutoCanvas::OnMouse)
  EVT_SIZE( xppAutoCanvas::OnSize)
END_EVENT_TABLE()

xppAutoCanvas::xppAutoCanvas(wxWindow *parent, xppAutoWindow *w)
  : wxPanel(parent, -1, wxDefaultPosition)
{
  autoWindow = w;
  bigFont = new wxFont(-1, wxMODERN, wxNORMAL, wxNORMAL);
  smallFont = new wxFont(-1, wxMODERN, wxNORMAL, wxNORMAL);
//  autoCanvasDC = new wxClientDC(this);
//  autoDC = autoCanvasDC;
//  autoPen = wxWHITE_PEN;
}

void xppAutoCanvas::OnPaint(wxPaintEvent &event)
{
  wxPaintDC dc(this);

  DrawPlot(dc);
}

void xppAutoCanvas::DrawBitmap(wxBitmap *bitmap)
{
  wxMemoryDC memoryDC;
  memoryDC.SelectObject(*bitmap);

  memoryDC.SetBackground(*wxWHITE_BRUSH);
  memoryDC.Clear();

  DrawPlot(memoryDC);
}

void xppAutoCanvas::DrawPlot(wxDC &dc)
{
  int i;
  wxPen paintPen;

  paintPen.SetWidth(AutoW->autoDrawnLines[0].w);
  paintPen.SetColour(*wxBLACK);
  paintPen.SetStyle(wxSOLID);
  paintPen.SetCap(wxCAP_BUTT);
  paintPen.SetJoin(wxJOIN_BEVEL);

  dc.SetPen(paintPen);

  dc.SetBackground(*wxWHITE_BRUSH);
  dc.Clear();

  dc.SetFont(*smallFont);
  dc.SetTextBackground(*wxWHITE);
  dc.SetTextForeground(*wxBLACK);

  setSmallFontParams(&dc);

  for(i=0; i<AutoW->numLines; i++)
  {
    paintPen.SetWidth(AutoW->autoDrawnLines[i].w);
    dc.SetPen(paintPen);
    dc.DrawLine(AutoW->autoDrawnLines[i].x1,
                AutoW->autoDrawnLines[i].y1,
                AutoW->autoDrawnLines[i].x2,
                AutoW->autoDrawnLines[i].y2);
  }

  paintPen.SetWidth(1);

  wxString tmp;
  for(i=0; i<AutoW->numTexts; i++)
  {
    tmp.Printf(_T("%s"), AutoW->autoDrawnTexts[i].string);
    dc.DrawText(tmp,
                AutoW->autoDrawnTexts[i].x,
                AutoW->autoDrawnTexts[i].y);
  }

  for(i=0; i<AutoW->numCircles; i++)
  {
    dc.DrawCircle(AutoW->autoDrawnCircles[i].x,
                  AutoW->autoDrawnCircles[i].y,
                  AutoW->autoDrawnCircles[i].r);
  }

  for(i=0; i<AutoW->numFCircles; i++)
  {
    dc.DrawCircle(AutoW->autoDrawnFCircles[i].x,
                  AutoW->autoDrawnFCircles[i].y,
                  AutoW->autoDrawnFCircles[i].r);
  }
}

void xppAutoCanvas::OnSize(wxSizeEvent &event)
{
/*
  wxPaintDC dc(this);
  dc.SetFont(*smallFont);
  setCurrentGraph(this);
  setSmallFontParams(&dc);
  resized_graph(this);
  Refresh();
  // The xppAutoCanvas windows can resize in any order,
  // making MyGraph impossible to track, so we reset it to
  // the currently selected tab
  runWindow->SetMyGraph();
*/
}

void xppAutoCanvas::OnMouse(wxMouseEvent &event)
{
/*
  wxString msg;
  float x, y;

  if(event.LeftDown())
  {
  }
  else if(event.LeftUp())
  {
  }
  else if(event.Dragging())
  {
  }
  else
  {
    getRealCoords(event.m_x, event.m_y, &x, &y);
    msg.Printf("(%f, %f)", x , y);
  }
  autoWindow->SetStatusText(msg);
*/
}

wxFont* xppAutoCanvas::GetBigFont()
{
  return bigFont;
}

wxFont* xppAutoCanvas::GetSmallFont()
{
  return smallFont;
}

void xppAutoCanvas::SetBigFont(wxFont font)
{
  delete bigFont;
  bigFont = new wxFont(font.GetPointSize(), font.GetFamily(), font.GetStyle(),
                font.GetWeight(), font.GetUnderlined(), font.GetFaceName());
}

void xppAutoCanvas::SetSmallFont(wxFont font)
{
  delete smallFont;
  smallFont = new wxFont(font.GetPointSize(), font.GetFamily(), font.GetStyle(),
                  font.GetWeight(), font.GetUnderlined(), font.GetFaceName());
}
