/*
xppDialogs_wx.cpp

All dialogs for the xppAutoWindow
*/
#include <wx/wxprec.h>

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif

#include <stdio.h>

#include "xppAutoWindow_wx.h"
#include "xppDialogs_wx.h"

#include "xppAutoDialogs_wx.h"

#define MAX_AUT_PER 10
typedef struct
{
  int exist;
  int ntst, nmx, npr;
  double ds, dsmax, dsmin, rl0, rl1, a0, a1;
  double xmin, xmax, ymin, ymax;
  double lastx, lasty;
  int wid, hgt, x0, y0, st_wid;
  int nfpar, nbc;
  int ips, irs, ilp, isp, isw, itp;
  int plot, var;
  int icp1, icp2, icp3, icp4, icp5;
  int nper;
  char hinttxt[256];
  double period[MAX_AUT_PER];
  int uzrpar[MAX_AUT_PER];
  double epsl, epsu, epss;
  int ncol;
} BIFUR;

extern BIFUR Auto;

/* Axes Dialog */

BEGIN_EVENT_TABLE(AutoAxesDlg, wxDialog)
  EVT_CHOICE( AAXES_CHOICE, AutoAxesDlg::OnAxesChoice)
END_EVENT_TABLE()

AutoAxesDlg::AutoAxesDlg(xppAutoWindow *p)
  : wxDialog(p, -1, _T("AUTO Axes"), wxDefaultPosition)
{
  parent = p;
  int i;
  wxString tmp;

  wxString labelStrings[8] = 
{_T("Type"),
 _T("Variable"),
 _T("Main Param"),
 _T("Second Param"),
 _T("Xmin"),
 _T("Xmax"),
 _T("Ymin"),
 _T("Ymax")};

  wxString plotStrings[7] = 
{_T("Hi"),
 _T("Norm"),
 _T("Hi-Lo"),
 _T("Period"),
 _T("Two Param"),
 _T("Frequency"),
 _T("Average")};

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  labels = new wxStaticText * [8];
  inputs = new wxTextCtrl * [4];
  choices = new wxChoice * [4];

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  for(i=0; i<8; i++)
  {
    labels[i] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);
  }

  choices[0] = new wxChoice(panel, AAXES_CHOICE, wxDefaultPosition, wxDefaultSize, 0, NULL);
  choices[1] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
  choices[2] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
  choices[3] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);

  for(i=0; i<7; i++)
  {
    choices[0]->Append(plotStrings[i]);
  }

  populateUvarChoice(choices[1]);

  for(i=0; i<5; i++)
  {
    tmp.Printf(_T("%s"), upar_names[AutoPar[i]]);
    choices[2]->Append(tmp);
  }

  for(i=0; i<5; i++)
  {
    tmp.Printf(_T("%s"), upar_names[AutoPar[i]]);
    choices[3]->Append(tmp);
  }

  for(i=0; i<4; i++)
  {
    inputs[i] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  }

  // Load up all of the values into the wxTextCtrl input boxes 
  UpdateDisplay();
/*
  if()
    Enable3D();
  else
    Disable3D();
*/

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(2);

  gridSizer->Add(labels[0], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(choices[0]);
  gridSizer->Add(labels[1], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(choices[1]);
  gridSizer->Add(labels[2], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(choices[2]);
  gridSizer->Add(labels[3], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(choices[3]);
  
  gridSizer->Add(labels[4], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[0]);
  gridSizer->Add(labels[5], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[1]);
  gridSizer->Add(labels[6], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[2]);
  gridSizer->Add(labels[7], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[3]);

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void AutoAxesDlg::OnAxesChoice(wxCommandEvent &WXUNUSED(event))
{
/*
  if(radioBox->GetSelection())
    Enable3D();
  else
    Disable3D();
*/
}

/*
void AutoAxesDlg::Disable3D()
{
  axisChoices[2]->Disable();
  inputs[0]->Disable();
  inputs[1]->Disable();
  inputs[2]->Disable();
  inputs[3]->Disable();
  inputs[4]->Disable();
  inputs[5]->Disable();
  inputs[8]->Disable();
  inputs[11]->Disable();
  inputs[14]->Disable();
  inputs[17]->Disable();
}

void AutoAxesDlg::Enable3D()
{
  axisChoices[2]->Enable();
  inputs[0]->Enable();
  inputs[1]->Enable();
  inputs[2]->Enable();
  inputs[3]->Enable();
  inputs[4]->Enable();
  inputs[5]->Enable();
  inputs[8]->Enable();
  inputs[11]->Enable();
  inputs[14]->Enable();
  inputs[17]->Enable();
}
*/

void AutoAxesDlg::UpdateValues()
{
  int plot;
  wxString tmpInput;

  tmpInput = inputs[0]->GetValue();
  if( stringToDouble(this, tmpInput, labels[4]->GetLabel() ))
    tmpInput.ToDouble(&(Auto.xmin));
  
  tmpInput = inputs[1]->GetValue();
  if( stringToDouble(this, tmpInput, labels[5]->GetLabel() ))
    tmpInput.ToDouble(&(Auto.xmax));

  tmpInput = inputs[2]->GetValue();
  if( stringToDouble(this, tmpInput, labels[6]->GetLabel() ))
    tmpInput.ToDouble(&(Auto.ymin));

  tmpInput = inputs[3]->GetValue();
  if( stringToDouble(this, tmpInput, labels[7]->GetLabel() ))
    tmpInput.ToDouble(&(Auto.ymax));

  Auto.var  = choices[1]->GetSelection();
  Auto.icp1 = choices[2]->GetSelection();
  Auto.icp2 = choices[3]->GetSelection();

  plot = choices[0]->GetSelection();

  if(plot == 0  // Hi
  || plot == 1  // Norm
  || plot == 2  // Hi-Lo
  || plot == 3  // Period
  || plot == 4) // Two Param
  {
    Auto.plot = plot;
  }

  if(plot == 5) // Frequency
  {
    Auto.plot = 9;
  }

  if(plot == 6) // Average
  {
    Auto.plot = 10;
  }
}

void AutoAxesDlg::UpdateDisplay()
{
  wxString tmp;

  tmp.Printf(_T("%.16g"), Auto.xmin);
  inputs[0]->SetValue(tmp);

  tmp.Printf(_T("%.16g"), Auto.xmax);
  inputs[1]->SetValue(tmp);

  tmp.Printf(_T("%.16g"), Auto.ymin);
  inputs[2]->SetValue(tmp);

  tmp.Printf(_T("%.16g"), Auto.ymax);
  inputs[3]->SetValue(tmp);

  choices[0]->SetSelection(0);
  choices[1]->SetSelection(Auto.var + 1);
  choices[2]->SetSelection(Auto.icp1);
  choices[3]->SetSelection(Auto.icp2);
}

/* Auto Numerics Dialog */

AutoNumericsDlg::AutoNumericsDlg(xppAutoWindow *p)
  : wxDialog(p, -1, _T("AUTO Numerics"), wxDefaultPosition)
{
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  int i;

  wxString labelStrings[14] =
{_T("Ntst"),
 _T("Dsmax"),
 _T("Nmax"),
 _T("Par Min"),
 _T("NPr"),
 _T("Par Max"),
 _T("Ds"),
 _T("Norm Min"),
 _T("Dsmin"),
 _T("Norm Max"),
 _T("Ncol"),
 _T("EPSU"),
 _T("EPSL"),
 _T("EPSS")};
  
  labels = new wxStaticText * [14];
  inputs = new wxTextCtrl * [14];
  for(i=0; i<14; i++)
  {
    labels[i] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);

    inputs[i] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
      wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  }

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  UpdateDisplay();

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(4);  

  for(i=0; i<14; i++)
  {
    gridSizer->Add(labels[i], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(inputs[i]);
  }

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void AutoNumericsDlg::UpdateValues()
{
  wxString tmpInput;
  long tmplong;

  tmpInput = inputs[0]->GetValue();
  if( stringToInt(this, tmpInput, labels[0]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    Auto.ntst = (int) tmplong;
  }

  tmpInput = inputs[1]->GetValue();
  if( stringToDouble(this, tmpInput, labels[1]->GetLabel() ))
    tmpInput.ToDouble(&(Auto.dsmax));

  tmpInput = inputs[2]->GetValue();
  if( stringToInt(this, tmpInput, labels[2]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    Auto.nmx = (int) tmplong;
  }

  tmpInput = inputs[3]->GetValue();
  if( stringToDouble(this, tmpInput, labels[3]->GetLabel() ))
    tmpInput.ToDouble(&(Auto.rl0));

  tmpInput = inputs[4]->GetValue();
  if( stringToInt(this, tmpInput, labels[4]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    Auto.npr = (int) tmplong;
  }

  tmpInput = inputs[5]->GetValue();
  if( stringToDouble(this, tmpInput, labels[5]->GetLabel() ))
    tmpInput.ToDouble(&(Auto.rl1));

  tmpInput = inputs[6]->GetValue();
  if( stringToDouble(this, tmpInput, labels[6]->GetLabel() ))
    tmpInput.ToDouble(&(Auto.ds));

  tmpInput = inputs[7]->GetValue();
  if( stringToDouble(this, tmpInput, labels[7]->GetLabel() ))
    tmpInput.ToDouble(&(Auto.a0));

  tmpInput = inputs[8]->GetValue();
  if( stringToDouble(this, tmpInput, labels[8]->GetLabel() ))
    tmpInput.ToDouble(&(Auto.dsmin));

  tmpInput = inputs[9]->GetValue();
  if( stringToDouble(this, tmpInput, labels[9]->GetLabel() ))
    tmpInput.ToDouble(&(Auto.a1));

  tmpInput = inputs[10]->GetValue();
  if( stringToInt(this, tmpInput, labels[10]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    Auto.ncol = (int) tmplong;
  }

  tmpInput = inputs[11]->GetValue();
  if( stringToDouble(this, tmpInput, labels[11]->GetLabel() ))
    tmpInput.ToDouble(&(Auto.epsu));

  tmpInput = inputs[12]->GetValue();
  if( stringToDouble(this, tmpInput, labels[12]->GetLabel() ))
    tmpInput.ToDouble(&(Auto.epsl));

  tmpInput = inputs[13]->GetValue();
  if( stringToDouble(this, tmpInput, labels[13]->GetLabel() ))
    tmpInput.ToDouble(&(Auto.epss));
}

void AutoNumericsDlg::UpdateDisplay()
{
  wxString tmp;

  tmp.Printf(_T("%i"), Auto.ntst);
  inputs[0]->SetValue(tmp);

  tmp.Printf(_T("%g"), Auto.dsmax);
  inputs[1]->SetValue(tmp);

  tmp.Printf(_T("%i"), Auto.nmx);
  inputs[2]->SetValue(tmp);

  tmp.Printf(_T("%g"), Auto.rl0);
  inputs[3]->SetValue(tmp);

  tmp.Printf(_T("%i"), Auto.npr);
  inputs[4]->SetValue(tmp);

  tmp.Printf(_T("%g"), Auto.rl1);
  inputs[5]->SetValue(tmp);

  tmp.Printf(_T("%g"), Auto.ds);
  inputs[6]->SetValue(tmp);

  tmp.Printf(_T("%g"), Auto.a0);
  inputs[7]->SetValue(tmp);

  tmp.Printf(_T("%g"), Auto.dsmin);
  inputs[8]->SetValue(tmp);

  tmp.Printf(_T("%g"), Auto.a1);
  inputs[9]->SetValue(tmp);

  tmp.Printf(_T("%i"), Auto.ncol);
  inputs[10]->SetValue(tmp);

  tmp.Printf(_T("%g"), Auto.epsu);
  inputs[11]->SetValue(tmp);

  tmp.Printf(_T("%g"), Auto.epsl);
  inputs[12]->SetValue(tmp);

  tmp.Printf(_T("%g"), Auto.epss);
  inputs[13]->SetValue(tmp);
}

/* Auto Parameters Dialog */

AutoParamDlg::AutoParamDlg(xppAutoWindow *p)
  : wxDialog(p, -1, _T("AUTO Axes"), wxDefaultPosition)
{
  parent = p;
  int i;

  wxString labelStrings[5] = 
{_T("Parameter 1"),
 _T("Parameter 2"),
 _T("Parameter 3"),
 _T("Parameter 4"),
 _T("Parameter 5")};

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  labels = new wxStaticText * [5];
  choices = new wxChoice * [5];

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  for(i=0; i<5; i++)
  {
    labels[i] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);

    choices[i] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);

    populateParamChoice(choices[i]);
  }

  UpdateDisplay();

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(2);

  gridSizer->Add(labels[0], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(choices[0]);
  gridSizer->Add(labels[1], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(choices[1]);
  gridSizer->Add(labels[2], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(choices[2]);
  gridSizer->Add(labels[3], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(choices[3]);
  gridSizer->Add(labels[4], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(choices[4]);

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void AutoParamDlg::UpdateValues()
{
  AutoPar[0] = choices[0]->GetSelection();
  AutoPar[1] = choices[1]->GetSelection();
  AutoPar[2] = choices[2]->GetSelection();
  AutoPar[3] = choices[3]->GetSelection();
  AutoPar[4] = choices[4]->GetSelection();
}

void AutoParamDlg::UpdateDisplay()
{
  choices[0]->SetSelection(AutoPar[0]);
  choices[1]->SetSelection(AutoPar[1]);
  choices[2]->SetSelection(AutoPar[2]);
  choices[3]->SetSelection(AutoPar[3]);
  choices[4]->SetSelection(AutoPar[4]);
}
