/*
xppAutoWindow_wx.h

Window that holds the AUTO diagram
*/

#ifndef __XPPAUTOWINH__
#define __XPPAUTOWINH__

class xppAutoWindow;
class xppAutoCanvas;
class xppRunWindow;

class xppAutoWindow : public wxFrame
{
  public:
    xppAutoWindow(xppRunWindow *w, wxPoint pos);

    // export submenu
    void OnExportImg(wxCommandEvent &event);
    void OnPostscript(wxCommandEvent &event);
    void OnWritePoints(wxCommandEvent &event);
    void OnAllInfo(wxCommandEvent &event);
    void OnInitData(wxCommandEvent &event);

    // file menu
    void OnImportOrbit(wxCommandEvent &event);
    void OnSaveDiagram(wxCommandEvent &event);
    void OnLoadDiagram(wxCommandEvent &event);
    void OnClose(wxCommandEvent &event);

    // edit menu
    void OnParam(wxCommandEvent &event);
    void OnNumerics(wxCommandEvent &event);
    void OnGrab(wxCommandEvent &event);
    void OnClearGrab(wxCommandEvent &event);
    void OnGrabPrev(wxCommandEvent &event);
    void OnGrabNext(wxCommandEvent &event);
    void OnGrabTab(wxCommandEvent &event);
    void OnGrabEnd(wxCommandEvent &event);
    void OnResetDiagram(wxCommandEvent &event);

    void OnEscapeKey(wxCommandEvent &event);

    // view menu
    void OnAxes(wxCommandEvent &event);
    void OnLast1Par(wxCommandEvent &event);
    void OnLast2Par(wxCommandEvent &event);
    void OnZoom(wxCommandEvent &event);
    void OnFit(wxCommandEvent &event);
    void OnClear(wxCommandEvent &event);
    void OnRedraw(wxCommandEvent &event);

    // run menu
    // start choices
    void OnStartSteadyState(wxCommandEvent &event);
    void OnStartPeriodic(wxCommandEvent &event);
    void OnStartBdryValue(wxCommandEvent &event);
    void OnStartHomoclinic(wxCommandEvent &event);
    void OnStartDiscrete(wxCommandEvent &event);
    // Hopf choices
    void OnHopfPeriodic(wxCommandEvent &event);
    void OnHopfExtend(wxCommandEvent &event);
    void OnHopfNewPt(wxCommandEvent &event);
    void OnHopfTwoPar(wxCommandEvent &event);
    // Period Doubling choices
    void OnPeriodDoubDoubling(wxCommandEvent &event);
    void OnPeriodDoubTwoParam(wxCommandEvent &event);
    void OnPeriodDoubFixedPeriod(wxCommandEvent &event);
    void OnPeriodDoubExtend(wxCommandEvent &event);
    // Torus choices
    void OnTorusTwoParam(wxCommandEvent &event);
    void OnTorusFixedPeriod(wxCommandEvent &event);
    void OnTorusExtend(wxCommandEvent &event);
    // Periodic choices
    void OnPeriodicExtend(wxCommandEvent &event);
    void OnPeriodicFixedPeriod(wxCommandEvent &event);
    // Various continuations
    void OnContLimitPt(wxCommandEvent &event);
    void OnContBranchPt(wxCommandEvent &event);
    void OnContSteadState(wxCommandEvent &event);
    void OnUnlabeledPt(wxCommandEvent &event);

    // Window close button
    void OnCloseWindow(wxCloseEvent &event);
  private:
    xppRunWindow *runWindow;
    xppAutoCanvas *autoCanvas;
    wxPanel *autoPanel;
    wxTextCtrl *autoInfo;
    wxBoxSizer *autoFrameSizer;
    wxBoxSizer *autoPanelSizer;
    wxBoxSizer *autoCanvasSizer;
    bool grabbing;
    bool runIsSet;
    char runType;

    void SetRunMenu();

  DECLARE_EVENT_TABLE()
};

enum
{
  // export submenu
  A_EXPORT = 9000,
  A_EXPORT_IMG,
  A_PS,

  // file menu
  A_IMPORT_ORBIT, 
  A_SAVE_DIAGRAM,
  A_LOAD_DIAGRAM,
  A_WRITE_PTS,
  A_ALL_INFO,
  A_INIT_DATA,
  A_CLOSE,

  // edit menu
  A_PARAM,
  A_NUMERICS,
  A_GRAB,
  A_CLEAR_GRAB,
  A_RESET_DIAGRAM,

  // More grabbing
  A_GRAB_PREV,
  A_GRAB_NEXT,
  A_GRAB_TAB,
  A_GRAB_END,

  // Escape key (grab and mouse)
  A_ESC,

  // view menu
  A_AXES,
  A_LAST_1_PAR,
  A_LAST_2_PAR,
  A_ZOOM,
  A_FIT,
  A_CLEAR,
  A_REDRAW,

  // run menu
  A_START,
  A_START_STEADY_STATE,
  A_START_PERIODIC,
  A_START_BDRY_VALUE,
  A_START_HOMOCLINIC,
  A_START_DISCRETE,

  A_HOPF,
  A_HOPF_PERIODIC,
  A_HOPF_EXTEND,
  A_HOPF_NEW_PT,
  A_HOPF_TWO_PAR,

  A_PERDOUB,
  A_PERDOUB_DOUB,
  A_PERDOUB_TWO_PAR,
  A_PERDOUB_FIX_PER,
  A_PERDOUB_EXTEND,

  A_TORUS,
  A_TORUS_TWO_PAR,
  A_TORUS_FIX_PER,
  A_TORUS_EXTEND,

  A_PERIODIC,
  A_PERIODIC_EXTEND,
  A_PERIODIC_FIX_PER,

  A_CONT_LIMIT_PT,
  A_CONT_BRANCH_PT,
  A_CONT_STEAD_STATE,
  A_UNLABELED_PT
};

extern "C" {
  void make_auto (int *w, int *h);
  void load_last_plot (int n);
  void auto_zoom (int i, int j, int k, int n);
  void auto_fit ();
  void clear_auto_plot ();

  void auto_new_ss ();
  void auto_start_diff_ss();
  void auto_new_per ();
  void auto_new_discrete();
  void auto_start_at_bvp();
  void auto_extend_ss();
  void auto_2p_hopf();
  void auto_period_double();
  void auto_twopar_double();
  void auto_2p_fixper();
  void auto_torus();
  void auto_2p_limit(int ips);
  void auto_switch_per();
  void auto_switch_ss();
  void auto_switch_bvp();
  void auto_extend_bvp();
  void set_up_homoclinic();

  void redraw_diagram ();
  void draw_bif_axes ();
  void auto_per_par ();
  void reset_auto ();
  void yes_reset_auto ();
  void keep_last_plot (int flag);

  /* File handling */
  void save_auto (char *filename);
  void load_auto (char *filename);
  void load_auto_orbit (char *filename);
  void post_auto (char *filename);
  void write_pts (char *filename);
  void write_info_out (char *filename);
  void write_init_data_file (char *filename);

  /* Point grabbing */
  void auto_grab ();
  void auto_grab_prev ();
  void auto_grab_next ();
  void auto_grab_tab ();
  void auto_grab_done ();

  //void auto_run (char wxCh);
  //void auto_plot_par (char ch);
  //void auto_num_par ();
  //void auto_params ();
  //void auto_file ();
}

#endif
