/*
xppCodePanel.cpp

Creates a basic text editor for editing .ode files
*/

#include <wx/wxprec.h>

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif

#include <wx/textctrl.h>
#include <wx/fdrepdlg.h>

#include "xppCodePanel_wx.h"

BEGIN_EVENT_TABLE(xppCodePanel, wxPanel)
#if wxUSE_FINDREPLDLG
  EVT_FIND             (-1, xppCodePanel::OnFind)
  EVT_FIND_NEXT        (-1, xppCodePanel::OnFindNext)
  EVT_FIND_REPLACE     (-1, xppCodePanel::OnFindReplace)
  EVT_FIND_REPLACE_ALL (-1, xppCodePanel::OnFindReplaceAll)
  EVT_FIND_CLOSE       (-1, xppCodePanel::OnFindClose)
#endif
END_EVENT_TABLE()

xppCodePanel::xppCodePanel(wxWindow* parent)
  : wxPanel(parent)
{
  code = new wxTextCtrl(this, -1, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE);
  codeSizer = new wxBoxSizer(wxVERTICAL);

  codeSizer->Add(code, 1, wxEXPAND);
  SetSizer(codeSizer);

#if wxUSE_FINDREPLDLG
  codeFindReplace = NULL;
#endif
}

bool xppCodePanel::LoadFile(const wxString &filename)
{
  return code->LoadFile(filename);
}

bool xppCodePanel::SaveFile(const wxString &filename)
{
  return code->SaveFile(filename);
}

bool xppCodePanel::IsModified()
{
  return code->IsModified();
}

void xppCodePanel::SelectAll()
{
  code->SetSelection(-1, -1);
}

#if wxUSE_FINDREPLDLG

void xppCodePanel::FindText()
{
  if(codeFind)
  {
    delete codeFind;
    codeFind = NULL;
  }
  else
  {
    codeFind = new wxFindReplaceDialog(code,&codeFindReplaceData,_T("Find"));
    codeFind->Show(TRUE);
  }
}

void xppCodePanel::FindReplaceText()
{
  if(codeFindReplace)
  {
    delete codeFindReplace;
    codeFindReplace = NULL;
  }
  else
  {
    codeFindReplace = new wxFindReplaceDialog(code,&codeFindReplaceData,_T("Find and Replace"),wxFR_REPLACEDIALOG);
    codeFindReplace->Show(TRUE);
  }
}

void xppCodePanel::OnFind(wxFindDialogEvent &event)
{

}

void xppCodePanel::OnFindNext(wxFindDialogEvent &event)
{

}

void xppCodePanel::OnFindReplace(wxFindDialogEvent &event)
{

}

void xppCodePanel::OnFindReplaceAll(wxFindDialogEvent &event)
{

}

void xppCodePanel::OnFindClose(wxFindDialogEvent &event)
{
  if(event.GetDialog() == codeFind)
    codeFind = NULL;
  else
    codeFindReplace = NULL;
}

#endif

wxTextCtrl* xppCodePanel::GetTextCtrl()
{
  return code;
}
