/*
xppColorDialog_wx.cpp

A dialog that handles editing the colormap
*/

#include <wx/wxprec.h>

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif

#include <wx/colordlg.h>
#include <wx/cmndata.h>
#include "xppRunWindow_wx.h"

#include "xppColorDialog_wx.h"

BEGIN_EVENT_TABLE(xppColorDialog, wxDialog)
  EVT_BUTTON (COLOR_0,    xppColorDialog::OnColorButton )
  EVT_BUTTON (COLOR_1,    xppColorDialog::OnColorButton )
  EVT_BUTTON (COLOR_2,    xppColorDialog::OnColorButton )
  EVT_BUTTON (COLOR_3,    xppColorDialog::OnColorButton )
  EVT_BUTTON (COLOR_4,    xppColorDialog::OnColorButton )
  EVT_BUTTON (COLOR_5,    xppColorDialog::OnColorButton )
  EVT_BUTTON (COLOR_6,    xppColorDialog::OnColorButton )
  EVT_BUTTON (COLOR_7,    xppColorDialog::OnColorButton )
  EVT_BUTTON (COLOR_8,    xppColorDialog::OnColorButton )
  EVT_BUTTON (COLOR_9,    xppColorDialog::OnColorButton )
  EVT_BUTTON (COLOR_10,   xppColorDialog::OnColorButton )
  EVT_BUTTON (COLOR_BACK, xppColorDialog::OnColorButton )
END_EVENT_TABLE()

xppColorDialog::xppColorDialog(xppRunWindow *w)
  : wxDialog(w, -1, _T("Edit Colormap"), wxDefaultPosition)
{
  runPtr = w;

//  wxStaticText *sTexts[12];
//  wxButton *colorButtons[12];

  wxButton *okButton = new wxButton(this, wxID_OK, _T("OK"));
  wxButton *cancelButton = new wxButton(this, wxID_CANCEL, _T("Cancel"));

  sTexts[0] = new wxStaticText(this, -1, _T("Color 0"), wxDefaultPosition);
  sTexts[1] = new wxStaticText(this, -1, _T("Color 1"), wxDefaultPosition);
  sTexts[2] = new wxStaticText(this, -1, _T("Color 2"), wxDefaultPosition);
  sTexts[3] = new wxStaticText(this, -1, _T("Color 3"), wxDefaultPosition);
  sTexts[4] = new wxStaticText(this, -1, _T("Color 4"), wxDefaultPosition);
  sTexts[5] = new wxStaticText(this, -1, _T("Color 5"), wxDefaultPosition);
  sTexts[6] = new wxStaticText(this, -1, _T("Color 6"), wxDefaultPosition);
  sTexts[7] = new wxStaticText(this, -1, _T("Color 7"), wxDefaultPosition);
  sTexts[8] = new wxStaticText(this, -1, _T("Color 8"), wxDefaultPosition);
  sTexts[9] = new wxStaticText(this, -1, _T("Color 9"), wxDefaultPosition);
  sTexts[10] = new wxStaticText(this, -1, _T("Color 10"), wxDefaultPosition);
  sTexts[11] = new wxStaticText(this, -1, _T("Background"), wxDefaultPosition);

  colorButtons[0] = new wxButton(this, COLOR_0, _T(""), wxDefaultPosition);
  colorButtons[1] = new wxButton(this, COLOR_1, _T(""), wxDefaultPosition);
  colorButtons[2] = new wxButton(this, COLOR_2, _T(""), wxDefaultPosition);
  colorButtons[3] = new wxButton(this, COLOR_3, _T(""), wxDefaultPosition);
  colorButtons[4] = new wxButton(this, COLOR_4, _T(""), wxDefaultPosition);
  colorButtons[5] = new wxButton(this, COLOR_5, _T(""), wxDefaultPosition);
  colorButtons[6] = new wxButton(this, COLOR_6, _T(""), wxDefaultPosition);
  colorButtons[7] = new wxButton(this, COLOR_7, _T(""), wxDefaultPosition);
  colorButtons[8] = new wxButton(this, COLOR_8, _T(""), wxDefaultPosition);
  colorButtons[9] = new wxButton(this, COLOR_9, _T(""), wxDefaultPosition);
  colorButtons[10] = new wxButton(this, COLOR_10, _T(""), wxDefaultPosition);
  colorButtons[11] = new wxButton(this, COLOR_BACK, _T(""), wxDefaultPosition);

  int i;
  for(i=0; i<12; i++)
    colorButtons[i]->SetBackgroundColour(runPtr->GetNormalColour(i));

  /* Sizer stuff */

  wxBoxSizer *dlgSizer = new wxBoxSizer(wxVERTICAL);
  wxBoxSizer *buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  wxGridSizer *gridSizer = new wxGridSizer(2);
  //wxBoxSizer *horizSizers[12];

  for(i=0; i<12; i++)
  {
    gridSizer->Add(sTexts[i], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(colorButtons[i]);
    //horizSizers[i] = new wxBoxSizer(wxHORIZONTAL);
    //horizSizers[i]->Add(sTexts[i], 1, wxEXPAND);
    //horizSizers[i]->Add(colorButtons[i], 1, wxEXPAND);
    //dlgSizer->Add(horizSizers[i], 1, wxEXPAND);
  }

  dlgSizer->Add(gridSizer, 1, wxEXPAND);

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  dlgSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);

  SetSizer(dlgSizer);
  dlgSizer->Fit(this);

  CenterOnParent();
}

void xppColorDialog::OnColorButton(wxCommandEvent &event)
{
  int colorNumber = event.GetId() - COLOR_0;

  wxColourData *cdata = new wxColourData();

  cdata->SetColour(runPtr->GetNormalColour(colorNumber));

  wxColourDialog *dlg = new wxColourDialog(this, cdata);

  dlg->CenterOnParent();

  if(dlg->ShowModal() == wxID_OK)
  {
    colorButtons[colorNumber]->SetBackgroundColour(dlg->GetColourData().GetColour());
    colorButtons[colorNumber]->Refresh();
  }
}

void xppColorDialog::SetColorMap()
{
  int i;
  for(i=0; i<12; i++)
    runPtr->SetNormalColour(i, colorButtons[i]->GetBackgroundColour());
}
