/*
xppDataWindow_wx.cpp

The window which holds the numerical run data
*/

#include <wx/wxprec.h>

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif

#include <wx/grid.h>

#include "Ccode/browse.h"
#include "xppRunWindow_wx.h"
#include "xppDialogs_wx.h"

#include "xppDataWindow_wx.h"

BEGIN_EVENT_TABLE(xppDataWindow, wxFrame)
  /* File menu */
  EVT_MENU (DATA_FILE_READ,  xppDataWindow::OnRead )
  EVT_MENU (DATA_FILE_WRITE, xppDataWindow::OnWrite)
  EVT_MENU (DATA_FILE_TABLE, xppDataWindow::OnTable)
  EVT_MENU (DATA_FILE_CLOSE, xppDataWindow::OnClose)

  /* Edit menu */
  EVT_MENU (DATA_EDIT_FIND,    xppDataWindow::OnFind   )
  EVT_MENU (DATA_EDIT_REPLACE, xppDataWindow::OnReplace)
  EVT_MENU (DATA_EDIT_UNDO,    xppDataWindow::OnUndo   )
  EVT_MENU (DATA_EDIT_ADD,     xppDataWindow::OnAdd    )
  EVT_MENU (DATA_EDIT_DEL,     xppDataWindow::OnDel    )
  EVT_MENU (DATA_EDIT_LOAD,    xppDataWindow::OnLoad   )

  /* View menu */
  EVT_MENU (DATA_VIEW_REFRESH, xppDataWindow::OnRefresh)
  EVT_MENU (DATA_VIEW_FIRST,   xppDataWindow::OnFirst  )
  EVT_MENU (DATA_VIEW_LAST,    xppDataWindow::OnLast   )

  EVT_CLOSE(  xppDataWindow::OnCloseWindow)
END_EVENT_TABLE()

xppDataWindow::xppDataWindow(xppRunWindow *w)
  : wxFrame(w, -1, _T("Data"), wxDefaultPosition)
{
  runWindow = w;
  dataGrid = (wxGrid*) NULL;

  dataFrameSizer = new wxBoxSizer(wxVERTICAL);
  dataGridSizer = new wxBoxSizer(wxVERTICAL);

  dataGrid = new wxGrid(this, -1);
  dataGrid->CreateGrid(my_browser.maxrow, my_browser.maxcol);
  dataGrid->SetDefaultCellAlignment(wxALIGN_LEFT, wxALIGN_CENTRE);
  dataGrid->SetEditable(FALSE);

  PopulateGrid();

  dataGridSizer->Add(dataGrid, 1, wxEXPAND);
  SetSizer(dataGridSizer);

#if wxUSE_MENUS
  // File menu
  wxMenu *dataFile = new wxMenu();
  dataFile->Append(DATA_FILE_READ, _T("Read data..."), _T("Load data from file into browser"));
  dataFile->Append(DATA_FILE_WRITE, _T("Write data..."), _T("Write data to ascii file"));
  dataFile->AppendSeparator();
  dataFile->Append(DATA_FILE_TABLE, _T("Table..."), _T("Write a column of data in tabular format"));
  dataFile->AppendSeparator();
  dataFile->Append(DATA_FILE_CLOSE, _T("Close"), _T("Close the data browser"));

  // Edit menu
  wxMenu *dataEdit = new wxMenu();
  dataEdit->Append(DATA_EDIT_FIND, _T("Find..."), _T("Find closest data point to given value"));
  dataEdit->AppendSeparator();
  dataEdit->Append(DATA_EDIT_REPLACE, _T("Replace..."), _T("Replace column by formula"));
  dataEdit->Append(DATA_EDIT_UNDO, _T("Undo"), _T("Undo last replacement"));
  dataEdit->AppendSeparator();
  dataEdit->Append(DATA_EDIT_ADD, _T("Add Col..."), _T("Add a new column to browser"));
  dataEdit->Append(DATA_EDIT_DEL, _T("Delete Col..."), _T("Delete a column from browser"));
  dataEdit->AppendSeparator();
  dataEdit->Append(DATA_EDIT_LOAD, _T("Load"), _T("Load first line of browser to initial data"));

  // View menu
  wxMenu *dataView = new wxMenu();
  dataView->Append(DATA_VIEW_REFRESH, _T("Refresh"), _T("Refresh browser with lastest data"));
  dataView->AppendSeparator();
  dataView->Append(DATA_VIEW_FIRST, _T("Mark First"), _T("Mark first point for plotting"));
  dataView->Append(DATA_VIEW_LAST, _T("Mark Last"), _T("Mark last point for plotting"));

  // Create the menu bar
  wxMenuBar *dataMenuBar = new wxMenuBar();
  dataMenuBar->Append(dataFile, _T("&File"));
  dataMenuBar->Append(dataEdit, _T("&Edit"));
  dataMenuBar->Append(dataView, _T("&View"));

  SetMenuBar(dataMenuBar);
#endif

#if wxUSE_TOOLBAR

#endif

#if wxUSE_STATUSBAR
  CreateStatusBar();
  SetStatusText(_T("Data"));
#endif

  dataFrameSizer->Add(dataGrid,1,wxEXPAND);
//  dataPanelSizer->Add(dataGrid,1,wxEXPAND);

//  dataPanel->SetSizer(dataFrameSizer);

  dataFrameSizer->SetMinSize(wxSize(640,480));
  dataFrameSizer->Fit(this);

  CenterOnParent();
}


void xppDataWindow::OnRead(wxCommandEvent &event)
{
  wxFileDialog dlg(this,_T("Open data file..."),_T(""),_T(""),_T("DAT (*.dat)|*.dat|All files (*.*)|*.*"), wxOPEN|wxFILE_MUST_EXIST);
  if (dlg.ShowModal() == wxID_OK)
  {
    wxString tmp;
    char filename[256];

    tmp = dlg.GetPath();
#ifdef wxUSE_UNICODE
    strcpy(filename, tmp.mb_str());
#else
    strcpy(filename, tmp.c_str());
#endif
    data_read(filename, &my_browser);
    PopulateGrid();
  }
}

void xppDataWindow::OnWrite(wxCommandEvent &event)
{
  wxFileDialog dlg(this,_T("Save data as..."),_T(""),_T(""),_T("DAT (*.dat)|*.dat"), wxSAVE | wxOVERWRITE_PROMPT); 
  if (dlg.ShowModal() == wxID_OK)
  {
    wxString tmp;
    char filename[256];

    tmp = dlg.GetPath();
#ifdef wxUSE_UNICODE
    strcpy(filename, tmp.mb_str());
#else
    strcpy(filename, tmp.c_str());
#endif

    data_write(filename, &my_browser);
  }
}

void xppDataWindow::OnTable(wxCommandEvent &event)
{
  DataTableDlg dlg(this);

  if(dlg.ShowModal() == wxID_OK)
  {
    int col;
    double xlo, xhi;
    char filename[256];
    wxString tmp;

    col = dlg.GetVar();
    xlo = dlg.GetXlo();
    xhi = dlg.GetXhi();
    tmp = dlg.GetFilename();
#ifdef wxUSE_UNICODE
    strcpy(filename, tmp.mb_str());
#else
    strcpy(filename, tmp.c_str());
#endif

    make_d_table (xlo, xhi, col, filename, my_browser);
  }
}

void xppDataWindow::OnClose(wxCommandEvent &event)
{
  Hide();
}

void xppDataWindow::OnCloseWindow(wxCloseEvent &event)
{
  Hide();
}

void xppDataWindow::OnFind(wxCommandEvent &event)
{
  FindDataValueDlg dlg(this);

  if(dlg.ShowModal() == wxID_OK)
  {
    int rowHeight, xunit, yunit;
    int varIndex;
    int rowIndex;
    double varValue;

    varIndex = dlg.GetVarIndex();
    varValue = dlg.GetValue();

    find_value(varIndex, varValue, &rowIndex, my_browser);

    if(rowIndex >= 0)
    {
      rowHeight = dataGrid->GetRowHeight(0);
      dataGrid->GetScrollPixelsPerUnit(&xunit, &yunit);
      dataGrid->SelectRow(rowIndex);
      dataGrid->Scroll(-1, rowIndex * rowHeight/yunit);
    }
  }
}

void xppDataWindow::OnReplace(wxCommandEvent &event)
{
  /* the info needed for replace requires
     the exact same info as for adding a col */
  AddDataColDlg dlg(this);

  if(dlg.ShowModal() == wxID_OK)
  {
    char name[12];
    char formula[12];

#ifdef wxUSE_UNICODE
    strcpy(name, dlg.GetName().mb_str());
    strcpy(formula, dlg.GetFormula().mb_str());
#else
    strcpy(name, dlg.GetName().c_str());
    strcpy(formula, dlg.GetFormula().c_str());
#endif

    replace_column(name, formula, my_browser.data, my_browser.maxrow);

    PopulateGrid();
  }
}

void xppDataWindow::OnUndo(wxCommandEvent &event)
{
  unreplace_column();
  PopulateGrid();
}

void xppDataWindow::OnAdd(wxCommandEvent &event)
{
  AddDataColDlg dlg(this);

  if(dlg.ShowModal() == wxID_OK)
  {
    char name[12];
    char formula[12];

#ifdef wxUSE_UNICODE
    strcpy(name, dlg.GetName().mb_str());
    strcpy(formula, dlg.GetFormula().mb_str());
#else
    strcpy(name, dlg.GetName().c_str());
    strcpy(formula, dlg.GetFormula().c_str());
#endif

    add_stor_col(name, formula, &my_browser);

    PopulateGrid();
  }
}

void xppDataWindow::OnDel(wxCommandEvent &event)
{
  wxString msg;
  msg.Printf(_T("Sorry - not working very well yet..."));
  wxMessageDialog dlg(this, msg, _T("D'oh"), wxOK, wxDefaultPosition);
  dlg.ShowModal();
/*
  DeleteColDlg dlg(this);

  if(dlg.ShowModal() == wxID_OK)
  {
    char name[12];

    strcpy(name, dlg.GetVar().c_str());

    del_stor_col(name, &my_browser);

    PopulateGrid();
  }
*/
}

void xppDataWindow::OnLoad(wxCommandEvent &event)
{
  data_get(&my_browser);
  runWindow->UpdateAllEditWins();

  wxString msg;
  msg.Printf(_T("Data loaded into ICs"));
  wxMessageDialog dlg(this, msg, _T("Load data"), wxOK, wxDefaultPosition);
  dlg.ShowModal();
}

void xppDataWindow::OnRefresh(wxCommandEvent &event)
{
  PopulateGrid();
}

void xppDataWindow::OnFirst(wxCommandEvent &event)
{
  data_first(&my_browser);
}

void xppDataWindow::OnLast(wxCommandEvent &event)
{
  data_last(&my_browser);
}

void xppDataWindow::PopulateGrid()
{
  int i, j, l;
  int n;
  wxString str;

  dataGrid->BeginBatch();

  // Fix the grid's dimensions
  n = dataGrid->GetNumberRows() - my_browser.maxrow;
  if(n > 0)
  {
    dataGrid->DeleteRows(0, n);
  }
  else if(n<0)
  {
    dataGrid->AppendRows(-n);
  }

  n = dataGrid->GetNumberCols() - my_browser.maxcol;
  if(n > 0)
  {
    dataGrid->DeleteCols(0, n);
  }
  else if(n<0)
  {
    dataGrid->AppendCols(-n);
  }

  // This traversal is taken straight from the C code
  dataGrid->SetColLabelValue(0, _T("Time"));
  for (i=0; i<my_browser.maxcol; i++)
  {
      str.Printf(_T("%s"), uvar_names[i]);
      dataGrid->SetColLabelValue(i+1, str);
  }

  for (i = my_browser.istart; i < my_browser.iend; i++)
  {
    if (N_plist > 0)
    {
      for (l = 0; l < N_plist; l++)
      {
        j = plotlist[l];
        str.Printf(_T("%g"), my_browser.data[j][i]);
        dataGrid->SetCellValue(i, l, str);
      } 
    }
    else
    {
      for (j = 0; j < my_browser.maxcol; j++)
      {
        str.Printf(_T("%g"), my_browser.data[j][i]);
        dataGrid->SetCellValue(i, j, str);
      }
    }
  }

  dataGrid->EndBatch();
}
