/*
xppDialogs_wx.cpp

All dialogs for the xppRunWindow and a few others

*/
#include <wx/wxprec.h>

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif

#include <stdio.h>

#include "Ccode/struct.h"
GRAPH *MyGraph;

#include "xppRunWindow_wx.h"
#include "xppDialogs_wx.h"

/* Struct for range and range2 integrate */
extern struct {
  char item[30],item2[30];
  int steps,steps2,reset,oldic,index,index2,cycle,type,type2,movie;
  double plow,phigh,plow2,phigh2;
  int rtype;
} range;

/* Struct for BVP range */
extern struct
{
  char item[30];
  int steps, side, cycle, movie;
  double plow, phigh;
} shoot_range;

/* Struct for Eq/Fixed Pt range */

extern struct {
  char item[30];
  int steps,shoot,col,movie;
  double plow,phigh;
} eq_range;

/* Struct for monte carlo equilib */

typedef struct
{
  int n;
  double tol;
  double xlo[MAXODE], xhi[MAXODE];
} FIXPTGUESS;

extern FIXPTGUESS fixptguess;


/* utitlity functions */

/* Fills a choicebox with dep var choices */
void populateUvarChoice(wxChoice *choiceBox)
{
  int i;
  wxString tmp;

  for(i=0; i<NEQ; i++)
  {
    tmp.Printf(_T("%s"), uvar_names[i]);
    choiceBox->Append(tmp);
  }
}

/* Fills a choicebox with parameter choices */
void populateParamChoice(wxChoice *choiceBox)
{
  int i;
  wxString tmp;

  for(i=0; i<NUPAR; i++)
  {
    tmp.Printf(_T("%s"), upar_names[i]);
    choiceBox->Append(tmp);
  }
}

/* Attempts to convert a wxString to a double.
   takes care of printing error messages if
   problems occur */

bool stringToDouble(wxWindow *parent, wxString input, wxString label)
{
  double testValue;
  bool convertFlag;
  wxString msg;

  convertFlag = input.ToDouble(&testValue);

  if(convertFlag)
  {
    return true;
  }
  else
  {
    msg = _T("Couldn't convert ") + label + _T("=") + input + _T(" to a double.");
    wxMessageDialog dlg(parent, msg, _T("Error"), wxOK, wxDefaultPosition);
    dlg.ShowModal();
    return false;
  }
}

/* Attempts to convert a wxString to an int.
   takes care of printing error messages if
   problems occur */

bool stringToInt(wxWindow *parent, wxString input, wxString label)
{
  long testValue;
  bool convertFlag;
  wxString msg;

  convertFlag = input.ToLong(&testValue);

  if(convertFlag)
  {
    return true;
  }
  else
  {
    msg = _T("Couldn't convert ") + label + _T("=") + input + _T(" to an integer.");
    wxMessageDialog dlg(parent, msg, _T("Error"), wxOK, wxDefaultPosition);
    dlg.ShowModal();
    return false;
  }
}


/* Integrate Range Dialog */

IntRangeDlg::IntRangeDlg(wxWindow *p)
  : wxDialog(p, -1, _T("Integrate Range"), wxDefaultPosition)
{
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  wxString labelStrings[7] = {_T("Vary"), _T("From"), _T("To"), _T("Number of steps"), _T("Use old ICs"), _T("Reset storage"), _T("Cycle color")};

  labels = new wxStaticText * [4];
  inputs = new wxTextCtrl * [3];
  checks = new wxCheckBox * [3];
  paramBox = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);

  populateUvarChoice(paramBox);
  populateParamChoice(paramBox);

  labels[0] = new wxStaticText(panel, -1, labelStrings[0], wxDefaultPosition);
  labels[1] = new wxStaticText(panel, -1, labelStrings[1], wxDefaultPosition);
  labels[2] = new wxStaticText(panel, -1, labelStrings[2], wxDefaultPosition);
  labels[3] = new wxStaticText(panel, -1, labelStrings[3], wxDefaultPosition);

  inputs[0] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  inputs[1] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  inputs[2] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));

  checks[0] = new wxCheckBox(panel, -1, labelStrings[4]);
  checks[1] = new wxCheckBox(panel, -1, labelStrings[5]);
  checks[2] = new wxCheckBox(panel, -1, labelStrings[6]);

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  UpdateDisplay();

  // Sizers 
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);

  gridSizer = new wxGridSizer(2);

  gridSizer->Add(labels[0], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(paramBox);
  gridSizer->Add(labels[1], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[0]);
  gridSizer->Add(labels[2], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[1]);

  gridSizer->Add(labels[3], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[2]);

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(checks[0]);
  panelSizer->Add(checks[1]);
  panelSizer->Add(checks[2]);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void IntRangeDlg::UpdateValues()
{
  long tmplong;
  wxString tmpInput;

  tmpInput = paramBox->GetStringSelection();
#ifdef wxUSE_UNICODE
  strcpy(range.item, tmpInput.mb_str());
#else
  strcpy(range.item, tmpInput.c_str());
#endif
  range_item();

  tmpInput = inputs[0]->GetValue();
  if( stringToDouble(this, tmpInput, labels[1]->GetLabel() ))
    tmpInput.ToDouble(&(range.plow));

  tmpInput = inputs[1]->GetValue();
  if( stringToDouble(this, tmpInput, labels[2]->GetLabel() ))
    tmpInput.ToDouble(&(range.phigh));

  tmpInput = inputs[2]->GetValue();
  if( stringToInt(this, tmpInput, labels[3]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    range.steps = (int) tmplong;
  }

  if(checks[0]->IsChecked())
    range.oldic = 1;
  else
    range.oldic = 0;

  if(checks[1]->IsChecked())
    range.reset = 1;
  else
    range.reset = 0;

  if(checks[2]->IsChecked())
    range.cycle = 1;
  else
    range.cycle = 0;
}

void IntRangeDlg::UpdateDisplay()
{
  wxString tmp;

  tmp.Printf(_T("%g"), range.plow);
  inputs[0]->SetValue(tmp);

  tmp.Printf(_T("%g"), range.phigh);
  inputs[1]->SetValue(tmp);

  tmp.Printf(_T("%i"), range.steps);
  inputs[2]->SetValue(tmp);

  checks[0]->SetValue(range.oldic);
  checks[1]->SetValue(range.reset);
  checks[2]->SetValue(range.cycle);
}

/* Integrate 2 Variable Range Dialog */

Int2RangeDlg::Int2RangeDlg(wxWindow *p)
  : wxDialog(p, -1, _T("Two Parameter Range"), wxDefaultPosition)
{
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  wxString labelStrings[9] = {_T("Vary 1"), _T("Vary 2"), _T("From"), _T("To"), _T("Steps 1"), _T("Use old ICs"), _T("Reset storage"), _T("Cycle color"), _T("Steps 2")};
  wxString radioChoices[2] = {_T("Array"), _T("Curve")};

  radioBox = new wxRadioBox(panel, -1, _T("Type"),
    wxDefaultPosition, wxDefaultSize, 2, radioChoices, 0, wxRA_SPECIFY_ROWS);

  labels = new wxStaticText * [8];
  inputs = new wxTextCtrl * [6];
  checks = new wxCheckBox * [3];
  paramBoxes = new wxChoice * [2];

  paramBoxes[0] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
  paramBoxes[1] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);

  populateUvarChoice(paramBoxes[0]);
  populateParamChoice(paramBoxes[0]);
  populateUvarChoice(paramBoxes[1]);
  populateParamChoice(paramBoxes[1]);

  labels[0] = new wxStaticText(panel, -1, labelStrings[0], wxDefaultPosition);
  labels[1] = new wxStaticText(panel, -1, labelStrings[1], wxDefaultPosition);
  labels[2] = new wxStaticText(panel, -1, labelStrings[2], wxDefaultPosition);
  labels[3] = new wxStaticText(panel, -1, labelStrings[2], wxDefaultPosition);
  labels[4] = new wxStaticText(panel, -1, labelStrings[3], wxDefaultPosition);
  labels[5] = new wxStaticText(panel, -1, labelStrings[3], wxDefaultPosition);
  labels[6] = new wxStaticText(panel, -1, labelStrings[4], wxDefaultPosition);
  labels[7] = new wxStaticText(panel, -1, labelStrings[8], wxDefaultPosition);

  inputs[0] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  inputs[1] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  inputs[2] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  inputs[3] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  inputs[4] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  inputs[5] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));

  checks[0] = new wxCheckBox(panel, -1, labelStrings[5]);
  checks[1] = new wxCheckBox(panel, -1, labelStrings[6]);
  checks[2] = new wxCheckBox(panel, -1, labelStrings[7]);

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  UpdateDisplay();

  // Sizers 
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(6);

  gridSizer->Add(labels[0], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(paramBoxes[0]);
  gridSizer->Add(labels[2], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[0]);
  gridSizer->Add(labels[4], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[1]);

  gridSizer->Add(labels[1], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(paramBoxes[1]);
  gridSizer->Add(labels[3], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[2]);
  gridSizer->Add(labels[5], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[3]);

  gridSizer->Add(labels[6], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[4]);

  gridSizer->Add(labels[7], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[5]);

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  panelSizer->Add(radioBox, 0, wxALIGN_CENTER);
  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(checks[0]);
  panelSizer->Add(checks[1]);
  panelSizer->Add(checks[2]);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();

}

void Int2RangeDlg::UpdateValues()
{
  long tmplong;
  wxString tmpInput;

  tmpInput = paramBoxes[0]->GetStringSelection();
#ifdef wxUSE_UNICODE
  strcpy(range.item, tmpInput.mb_str());
#else
  strcpy(range.item, tmpInput.c_str());
#endif
  range_item();

  tmpInput = paramBoxes[1]->GetStringSelection();
#ifdef wxUSE_UNICODE
  strcpy(range.item2, tmpInput.mb_str());
#else
  strcpy(range.item2, tmpInput.c_str());
#endif
  range_item2();

  tmpInput = inputs[0]->GetValue();
  if( stringToDouble(this, tmpInput, labels[1]->GetLabel() ))
    tmpInput.ToDouble(&(range.plow));

  tmpInput = inputs[1]->GetValue();
  if( stringToDouble(this, tmpInput, labels[2]->GetLabel() ))
    tmpInput.ToDouble(&(range.phigh));

  tmpInput = inputs[2]->GetValue();
  if( stringToDouble(this, tmpInput, labels[1]->GetLabel() ))
    tmpInput.ToDouble(&(range.plow2));

  tmpInput = inputs[3]->GetValue();
  if( stringToDouble(this, tmpInput, labels[2]->GetLabel() ))
    tmpInput.ToDouble(&(range.phigh2));

  tmpInput = inputs[4]->GetValue();
  if( stringToInt(this, tmpInput, labels[3]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    range.steps = (int) tmplong;
  }

  tmpInput = inputs[5]->GetValue();
  if( stringToInt(this, tmpInput, labels[7]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    range.steps2 = (int) tmplong;
  }

  if(checks[0]->IsChecked())
    range.oldic = 1;
  else
    range.oldic = 0;

  if(checks[1]->IsChecked())
    range.reset = 1;
  else
    range.reset = 0;

  if(checks[2]->IsChecked())
    range.cycle = 1;
  else
    range.cycle = 0;

  if(radioBox->GetSelection() == 0)
  {
    range.rtype = 2;
  }
  else
  {
    range.rtype = 1;
  }
}

void Int2RangeDlg::UpdateDisplay()
{
  wxString tmp;

  tmp.Printf(_T("%g"), range.plow);
  inputs[0]->SetValue(tmp);

  tmp.Printf(_T("%g"), range.phigh);
  inputs[1]->SetValue(tmp);

  tmp.Printf(_T("%g"), range.plow2);
  inputs[2]->SetValue(tmp);

  tmp.Printf(_T("%g"), range.phigh2);
  inputs[3]->SetValue(tmp);

  tmp.Printf(_T("%i"), range.steps);
  inputs[4]->SetValue(tmp);

  tmp.Printf(_T("%i"), range.steps2);
  inputs[5]->SetValue(tmp);

/*
  if(range.rtype == 2)
  {
*/
    /* Always set to Curve for now */
    radioBox->SetSelection(1);
/*
  }
  else
  {
    radioBox->SetSelection(0);
  }
*/

  checks[0]->SetValue(range.oldic);
  checks[1]->SetValue(range.reset);
  checks[2]->SetValue(range.cycle);
}

/* BVP Range Dialog */

BVPRangeDlg::BVPRangeDlg(wxWindow *p)
  : wxDialog(p, -1, _T("BVP Range"), wxDefaultPosition)
{
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  wxString labelStrings[5] = {_T("Vary"), _T("From"), _T("To"), _T("Number of steps"), _T("Cycle color")};
  wxString radioChoices[2] = {_T("Left"), _T("Right")};

  radioBox = new wxRadioBox(panel, -1, _T("Save side"),
    wxDefaultPosition, wxDefaultSize, 2, radioChoices);

  paramBox = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
  labels = new wxStaticText * [4];
  inputs = new wxTextCtrl * [3];

  check = new wxCheckBox(panel, -1, labelStrings[4]);

  populateParamChoice(paramBox);

  labels[0] = new wxStaticText(panel, -1, labelStrings[0], wxDefaultPosition);
  labels[1] = new wxStaticText(panel, -1, labelStrings[1], wxDefaultPosition);
  labels[2] = new wxStaticText(panel, -1, labelStrings[2], wxDefaultPosition);
  labels[3] = new wxStaticText(panel, -1, labelStrings[3], wxDefaultPosition);

  inputs[0] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  inputs[1] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  inputs[2] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  UpdateDisplay();

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(2);

  gridSizer->Add(labels[0], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(paramBox);
  gridSizer->Add(labels[1], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[0]);
  gridSizer->Add(labels[2], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[1]);

  gridSizer->Add(labels[3], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[2]);

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  panelSizer->Add(gridSizer);
  panelSizer->Add(radioBox);
  panelSizer->Add(check);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void BVPRangeDlg::UpdateValues()
{
  wxString tmpInput;
  long tmplong;

  strcpy(shoot_range.item, upar_names[paramBox->GetSelection()]);

/* Testing the shoot_range.item setting
  tmpInput.Printf(_T("%s"), upar_names[paramBox->GetSelection()]);
  wxMessageBox(tmpInput, _T(""), wxOK, this);
*/

  tmpInput = inputs[0]->GetValue();
  if( stringToDouble(this, tmpInput, labels[1]->GetLabel() ))
    tmpInput.ToDouble(&(shoot_range.plow));

  tmpInput = inputs[1]->GetValue();
  if( stringToDouble(this, tmpInput, labels[2]->GetLabel() ))
    tmpInput.ToDouble(&(shoot_range.phigh));

  tmpInput = inputs[2]->GetValue();
  if( stringToInt(this, tmpInput, labels[3]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    shoot_range.steps = (int) tmplong;
  }

  shoot_range.side = radioBox->GetSelection();

  if(check->IsChecked())
  {
    shoot_range.cycle = 1;
  }
  else
  {
    shoot_range.cycle = 0;
  }
}

void BVPRangeDlg::UpdateDisplay()
{
  wxString tmp;
  int i;

  /* don't know why this is - 2, but it is */
  i = get_param_index(shoot_range.item) - 2;
  if(i >= 0)
    paramBox->SetSelection(i); 
  else
    paramBox->SetSelection(0);

  tmp.Printf(_T("%g"), shoot_range.plow);
  inputs[0]->SetValue(tmp);

  tmp.Printf(_T("%g"), shoot_range.phigh);
  inputs[1]->SetValue(tmp);

  tmp.Printf(_T("%i"), shoot_range.steps);
  inputs[2]->SetValue(tmp);

  radioBox->SetSelection(shoot_range.side);

  check->SetValue(shoot_range.cycle);
}

/* Fixed Pt (Equilibrium) Dialog */

FixedPtRangeDlg::FixedPtRangeDlg(wxWindow *p)
  : wxDialog(p, -1, _T("Fixed Pt. Range"), wxDefaultPosition)
{
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  wxString labelStrings[6] = {_T("Vary"), _T("From"), _T("To"), _T("Number of steps"), _T("Stability Col."), _T("Shoot")};

  labels = new wxStaticText * [5];
  inputs = new wxTextCtrl * [3];
  check = new wxCheckBox(panel, -1, labelStrings[5]);
  paramBox = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
  populateParamChoice(paramBox);

  stabilityBox = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
  stabilityBox->Append(_T("None"));
  stabilityBox->Append(_T("Time"));
  populateUvarChoice(stabilityBox);

  labels[0] = new wxStaticText(panel, -1, labelStrings[0], wxDefaultPosition);
  labels[1] = new wxStaticText(panel, -1, labelStrings[1], wxDefaultPosition);
  labels[2] = new wxStaticText(panel, -1, labelStrings[2], wxDefaultPosition);
  labels[3] = new wxStaticText(panel, -1, labelStrings[3], wxDefaultPosition);
  labels[4] = new wxStaticText(panel, -1, labelStrings[4], wxDefaultPosition);

  inputs[0] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  inputs[1] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  inputs[2] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  UpdateDisplay();

  // Sizers 
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);

  gridSizer = new wxGridSizer(2);

  gridSizer->Add(labels[0], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(paramBox);
  gridSizer->Add(labels[1], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[0]);
  gridSizer->Add(labels[2], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[1]);
  gridSizer->Add(labels[3], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[2]);
  gridSizer->Add(labels[4], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(stabilityBox);

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(check);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void FixedPtRangeDlg::UpdateValues()
{
  int i;
  long tmplong;
  wxString tmpInput;

  tmpInput = paramBox->GetStringSelection();
#ifdef wxUSE_UNICODE
  strcpy(eq_range.item, tmpInput.mb_str());
#else
  strcpy(eq_range.item, tmpInput.c_str());
#endif

  tmpInput = inputs[0]->GetValue();
  if( stringToDouble(this, tmpInput, labels[1]->GetLabel() ))
    tmpInput.ToDouble(&(eq_range.plow));

  tmpInput = inputs[1]->GetValue();
  if( stringToDouble(this, tmpInput, labels[2]->GetLabel() ))
    tmpInput.ToDouble(&(eq_range.phigh));

  tmpInput = inputs[2]->GetValue();
  if( stringToInt(this, tmpInput, labels[3]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    eq_range.steps = (int) tmplong;
  }

  i = stabilityBox->GetSelection();
  if(i == 0)
    eq_range.col = -1;
  else
    eq_range.col = i;

  if(check->IsChecked())
    eq_range.shoot = 1;
  else
    eq_range.shoot = 0;
}

void FixedPtRangeDlg::UpdateDisplay()
{
  wxString tmp;
  int i;

  i = get_param_index(eq_range.item) - 2;
  if(i >= 0)
    paramBox->SetSelection(i); 
  else
    paramBox->SetSelection(0);

  tmp.Printf(_T("%g"), eq_range.plow);
  inputs[0]->SetValue(tmp);

  tmp.Printf(_T("%g"), eq_range.phigh);
  inputs[1]->SetValue(tmp);

  tmp.Printf(_T("%i"), eq_range.steps);
  inputs[2]->SetValue(tmp);

  check->SetValue(eq_range.shoot);

  if(eq_range.col >= 1 && eq_range.col<=NEQ)
  {
    stabilityBox->SetSelection(eq_range.col);
  }
  else
  {
    stabilityBox->SetSelection(0);
  }
}

/* View Dialog */

BEGIN_EVENT_TABLE(ViewDlg, wxDialog)
  EVT_RADIOBOX( VIEW_RADIO, ViewDlg::OnRadio)
END_EVENT_TABLE()

ViewDlg::ViewDlg(xppRunWindow *p)
  : wxDialog(p, -1, _T("View Parameters"), wxDefaultPosition)
{
  parent = p;
  int i;

  wxString labelStrings[21] = 
{_T("X-Axis"),
 _T("Y-Axis"),
 _T("Z-Axis"),
 _T("X-Min"),
 _T("Y-Min"),
 _T("Z-Min"),
 _T("X-Max"),
 _T("Y-Max"),
 _T("Z-Max"),
 _T("X-Lo"),
 _T("Y-Lo"),
 _T("Phi"),
 _T("X-Hi"),
 _T("Y-Hi"),
 _T("Theta"),
 _T("X-Label"),
 _T("Y-Label"),
 _T("Z-Label"),
 _T("X-Org"),
 _T("Y-Org"),
 _T("Z-Org")};

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  labels = new wxStaticText * [21];
  inputs = new wxTextCtrl * [18];
  axisChoices = new wxChoice * [3];

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  for(i=0; i<3; i++)
  {
    labels[i] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);
    axisChoices[i] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
    axisChoices[i]->Append(_T("Time"));
    populateUvarChoice(axisChoices[i]);
  }

  for(i=0; i<12; i++)
  {
    labels[i+3] = new wxStaticText(panel, -1, labelStrings[i+3], wxDefaultPosition);

    inputs[i] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  }

  for(i=12; i<15; i++)
  {
    labels[i+3] = new wxStaticText(panel, -1, labelStrings[i+3], wxDefaultPosition);

    inputs[i] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NONE));
  }

  for(i=15; i<18; i++)
  {
    labels[i+3] = new wxStaticText(panel, -1, labelStrings[i+3], wxDefaultPosition);

    inputs[i] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  }

  wxString radioChoices[2] = {_T("2D"), _T("3D")};

  radioBox = new wxRadioBox(panel, VIEW_RADIO, _T("View"),
    wxDefaultPosition, wxDefaultSize, 2, radioChoices, 0, wxRA_SPECIFY_ROWS);

  fitCheck = new wxCheckBox(panel, -1, _T("Auto fit"));

  // Load up all of the values into the wxTextCtrl input boxes 
  UpdateDisplay();
  if(radioBox->GetSelection())
    Enable3D();
  else
    Disable3D();

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  topHorizSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(6);

  topHorizSizer->Add(radioBox);
  topHorizSizer->Add(fitCheck);

  gridSizer->Add(labels[0], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(axisChoices[0]);
  gridSizer->Add(labels[1], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(axisChoices[1]);
  gridSizer->Add(labels[2], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(axisChoices[2]);
  
  gridSizer->Add(labels[3], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[0]);
  gridSizer->Add(labels[4], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[1]);
  gridSizer->Add(labels[5], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[2]);

  gridSizer->Add(labels[6], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[3]);
  gridSizer->Add(labels[7], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[4]);
  gridSizer->Add(labels[8], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[5]);

  gridSizer->Add(labels[9], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[6]);
  gridSizer->Add(labels[10], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[7]);
  gridSizer->Add(labels[11], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[8]);

  gridSizer->Add(labels[12], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[9]);
  gridSizer->Add(labels[13], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[10]);
  gridSizer->Add(labels[14], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[11]);

  gridSizer->Add(labels[15], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[12]);
  gridSizer->Add(labels[16], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[13]);
  gridSizer->Add(labels[17], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[14]);

  gridSizer->Add(labels[18], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[15]);
  gridSizer->Add(labels[19], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[16]);
  gridSizer->Add(labels[20], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(inputs[17]);

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panelSizer->Add(topHorizSizer, 0, wxALIGN_CENTER);
  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void ViewDlg::OnRadio(wxCommandEvent &WXUNUSED(event))
{
  if(radioBox->GetSelection())
    Enable3D();
  else
    Disable3D();
}

void ViewDlg::Disable3D()
{
  axisChoices[2]->Disable();
  inputs[0]->Disable();
  inputs[1]->Disable();
  inputs[2]->Disable();
  inputs[3]->Disable();
  inputs[4]->Disable();
  inputs[5]->Disable();
  inputs[8]->Disable();
  inputs[11]->Disable();
  inputs[14]->Disable();
  inputs[17]->Disable();
}

void ViewDlg::Enable3D()
{
  axisChoices[2]->Enable();
  inputs[0]->Enable();
  inputs[1]->Enable();
  inputs[2]->Enable();
  inputs[3]->Enable();
  inputs[4]->Enable();
  inputs[5]->Enable();
  inputs[8]->Enable();
  inputs[11]->Enable();
  inputs[14]->Enable();
  inputs[17]->Enable();
}

void ViewDlg::UpdateValues()
{
  long tmplong;
  wxString tmpInput;

  MyGraph->ThreeDFlag = radioBox->GetSelection();
  if(MyGraph->ThreeDFlag)
    MyGraph->grtype = 5;
  else
    MyGraph->grtype = 0;

  tmpInput = inputs[6]->GetValue();
  if( stringToDouble(this, tmpInput, labels[3]->GetLabel() ))
    tmpInput.ToDouble(&(MyGraph->xlo));
  
  tmpInput = inputs[7]->GetValue();
  if( stringToDouble(this, tmpInput, labels[3]->GetLabel() ))
    tmpInput.ToDouble(&(MyGraph->ylo));

  tmpInput = inputs[9]->GetValue();
  if( stringToDouble(this, tmpInput, labels[3]->GetLabel() ))
    tmpInput.ToDouble(&(MyGraph->xhi));

  tmpInput = inputs[10]->GetValue();
  if( stringToDouble(this, tmpInput, labels[3]->GetLabel() ))
    tmpInput.ToDouble(&(MyGraph->yhi));

  if(MyGraph->ThreeDFlag)
  {
    tmpInput = inputs[0]->GetValue();
    if( stringToDouble(this, tmpInput, labels[3]->GetLabel() ))
      tmpInput.ToDouble(&(MyGraph->xmin));

    tmpInput = inputs[1]->GetValue();
    if( stringToDouble(this, tmpInput, labels[3]->GetLabel() ))
      tmpInput.ToDouble(&(MyGraph->ymin));

    tmpInput = inputs[3]->GetValue();
    if( stringToDouble(this, tmpInput, labels[3]->GetLabel() ))
      tmpInput.ToDouble(&(MyGraph->xmax));

    tmpInput = inputs[4]->GetValue();
    if( stringToDouble(this, tmpInput, labels[3]->GetLabel() ))
      tmpInput.ToDouble(&(MyGraph->ymax));
  }
  else
  {
    MyGraph->xmin = MyGraph->xlo;
    MyGraph->xmax = MyGraph->xhi;
    MyGraph->ymin = MyGraph->ylo;
    MyGraph->ymax = MyGraph->yhi;
  }

  tmpInput = inputs[2]->GetValue();
  if( stringToDouble(this, tmpInput, labels[3]->GetLabel() ))
    tmpInput.ToDouble(&(MyGraph->zmin));


  tmpInput = inputs[5]->GetValue();
  if( stringToDouble(this, tmpInput, labels[3]->GetLabel() ))
    tmpInput.ToDouble(&(MyGraph->zmax));


  tmpInput = inputs[8]->GetValue();
  if( stringToDouble(this, tmpInput, labels[3]->GetLabel() ))
    tmpInput.ToDouble(&(MyGraph->Phi));

  tmpInput = inputs[11]->GetValue();
  if( stringToDouble(this, tmpInput, labels[3]->GetLabel() ))
    tmpInput.ToDouble(&(MyGraph->Theta));


  tmpInput = inputs[12]->GetValue();
#ifdef wxUSE_UNICODE
  strcpy(MyGraph->xlabel, tmpInput.mb_str());
#else
  strcpy(MyGraph->xlabel, tmpInput.c_str());
#endif

  tmpInput = inputs[13]->GetValue();
#ifdef wxUSE_UNICODE
  strcpy(MyGraph->ylabel, tmpInput.mb_str());
#else
  strcpy(MyGraph->ylabel, tmpInput.c_str());
#endif

  tmpInput = inputs[14]->GetValue();
#ifdef wxUSE_UNICODE
  strcpy(MyGraph->zlabel, tmpInput.mb_str());
#else
  strcpy(MyGraph->zlabel, tmpInput.c_str());
#endif

  tmpInput = inputs[15]->GetValue();
  if( stringToInt(this, tmpInput, labels[3]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    MyGraph->xshft = (int) tmplong;
  }

  tmpInput = inputs[16]->GetValue();
  if( stringToInt(this, tmpInput, labels[3]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    MyGraph->yshft = (int) tmplong;
  }

  tmpInput = inputs[17]->GetValue();
  if( stringToInt(this, tmpInput, labels[3]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    MyGraph->zshft = (int) tmplong;
  }

  MyGraph->xv[0] = axisChoices[0]->GetSelection();
  MyGraph->yv[0] = axisChoices[1]->GetSelection();
  MyGraph->zv[0] = axisChoices[2]->GetSelection();

  check_windows();
  check_flags();
}

void ViewDlg::UpdateDisplay()
{
  wxString tmp;

  tmp.Printf(_T("%.16g"), MyGraph->xmin);
  inputs[0]->SetValue(tmp);

  tmp.Printf(_T("%.16g"), MyGraph->ymin);
  inputs[1]->SetValue(tmp);

  tmp.Printf(_T("%.16g"), MyGraph->zmin);
  inputs[2]->SetValue(tmp);

  tmp.Printf(_T("%.16g"), MyGraph->xmax);
  inputs[3]->SetValue(tmp);

  tmp.Printf(_T("%.16g"), MyGraph->ymax);
  inputs[4]->SetValue(tmp);

  tmp.Printf(_T("%.16g"), MyGraph->zmax);
  inputs[5]->SetValue(tmp);

  tmp.Printf(_T("%.16g"), MyGraph->xlo);
  inputs[6]->SetValue(tmp);

  tmp.Printf(_T("%.16g"), MyGraph->ylo);
  inputs[7]->SetValue(tmp);

  tmp.Printf(_T("%.16g"), MyGraph->Phi);
  inputs[8]->SetValue(tmp);

  tmp.Printf(_T("%.16g"), MyGraph->xhi);
  inputs[9]->SetValue(tmp);

  tmp.Printf(_T("%.16g"), MyGraph->yhi);
  inputs[10]->SetValue(tmp);

  tmp.Printf(_T("%.16g"), MyGraph->Theta);
  inputs[11]->SetValue(tmp);

  tmp.Printf(_T("%s"), MyGraph->xlabel);
  inputs[12]->SetValue(tmp);

  tmp.Printf(_T("%s"), MyGraph->ylabel);
  inputs[13]->SetValue(tmp);

  tmp.Printf(_T("%s"), MyGraph->zlabel);
  inputs[14]->SetValue(tmp);

  tmp.Printf(_T("%i"), MyGraph->xshft);
  inputs[15]->SetValue(tmp);

  tmp.Printf(_T("%i"), MyGraph->yshft);
  inputs[16]->SetValue(tmp);

  tmp.Printf(_T("%i"), MyGraph->zshft);
  inputs[17]->SetValue(tmp);

  axisChoices[0]->SetSelection(MyGraph->xv[0]);
  axisChoices[1]->SetSelection(MyGraph->yv[0]);
  axisChoices[2]->SetSelection(MyGraph->zv[0]);

  radioBox->SetSelection(MyGraph->ThreeDFlag);
}

bool ViewDlg::GetFitFlag()
{
  return fitCheck->IsChecked();
}

/* Integrate Parameters Dialog */

IntParsDlg::IntParsDlg(wxWindow *p)
  : wxDialog(p, -1, _T("Integration Parameters"), wxDefaultPosition)
{
  parent = p;
  int i;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  wxString methodChoices[15] =
{_T("Discrete"),
 _T("Euler"),
 _T("Mod. Euler"),
 _T("Runge-Kutta"),
 _T("Adams"),
 _T("Gear"),
 _T("Volterra"),
 _T("BackEul"),
 _T("Qualst.RK4"),
 _T("Stiff"),
 _T("CVode"),
 _T("DoPri(5)"),
 _T("DoPri(8)3"),
 _T("Rosen(2)2"),
 _T("Symplectic")};

  wxString labelStrings[11] =
{_T("Rel. Toler."),
 _T("Abs. Toler."),
 _T("Min Step"),
 _T("Max Step"),
 _T("Bounds"),
 _T("Total"),
 _T("Transient"),
 _T("Delta T"),
 _T("Nout"),
 _T("TStart"),
 _T("Max Delay")};

  labels = new wxStaticText * [11];
  for(i=0; i<11; i++)
  {
    labels[i] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);
  }

  radioBox = new wxRadioBox(panel, -1, _T("Method"),
    wxDefaultPosition, wxDefaultSize, 15, methodChoices, 1, wxRA_SPECIFY_COLS);

  inputs = new wxTextCtrl * [11];
  for(i=0; i<11; i++)
  {
    inputs[i] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
      wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  }
  
  tolBox = new wxStaticBox(panel, -1, _T("Tolerances"));
  timeBox = new wxStaticBox(panel, -1, _T("Time stuff"));

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  UpdateDisplay();

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  vertSizer = new wxBoxSizer(wxVERTICAL);
  horizSizer = new wxBoxSizer(wxHORIZONTAL);
  
  wxBoxSizer *buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer1 = new wxGridSizer(2);
  gridSizer2 = new wxGridSizer(2);

  tolBoxSizer = new wxStaticBoxSizer(tolBox, wxVERTICAL);
  timeBoxSizer = new wxStaticBoxSizer(timeBox, wxVERTICAL);

  for(i=0; i<5; i++)
  {
    gridSizer1->Add(labels[i]);
    gridSizer1->Add(inputs[i]);
  }
  tolBoxSizer->Add(gridSizer1);

  for(i=5; i<11; i++)
  {
    gridSizer2->Add(labels[i]);
    gridSizer2->Add(inputs[i]);
  }
  timeBoxSizer->Add(gridSizer2);

  vertSizer->Add(tolBoxSizer);
  vertSizer->Add(timeBoxSizer);

  horizSizer->Add(radioBox);
  horizSizer->Add(vertSizer);

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  mainSizer->Add(panelSizer);

  panelSizer->Add(horizSizer);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void IntParsDlg::UpdateValues()
{
  wxString tmpInput;
  long tmplong;

  METHOD = radioBox->GetSelection();

  tmpInput = inputs[0]->GetValue();
  if( stringToDouble(this, tmpInput, labels[0]->GetLabel() ))
    tmpInput.ToDouble(&TOLER);

  tmpInput = inputs[1]->GetValue();
  if( stringToDouble(this, tmpInput, labels[0]->GetLabel() ))
    tmpInput.ToDouble(&ATOLER);

  tmpInput = inputs[2]->GetValue();
  if( stringToDouble(this, tmpInput, labels[0]->GetLabel() ))
    tmpInput.ToDouble(&HMIN);

  tmpInput = inputs[3]->GetValue();
  if( stringToDouble(this, tmpInput, labels[0]->GetLabel() ))
    tmpInput.ToDouble(&HMAX);

  tmpInput = inputs[4]->GetValue();
  if( stringToDouble(this, tmpInput, labels[0]->GetLabel() ))
    tmpInput.ToDouble(&BOUND);

  tmpInput = inputs[5]->GetValue();
  if( stringToDouble(this, tmpInput, labels[0]->GetLabel() ))
    tmpInput.ToDouble(&TEND);

  tmpInput = inputs[6]->GetValue();
  if( stringToDouble(this, tmpInput, labels[0]->GetLabel() ))
    tmpInput.ToDouble(&TRANS);

  tmpInput = inputs[7]->GetValue();
  if( stringToDouble(this, tmpInput, labels[0]->GetLabel() ))
    tmpInput.ToDouble(&DELTA_T);

  tmpInput = inputs[8]->GetValue();
  if( stringToInt(this, tmpInput, labels[8]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    NJMP = (int) tmplong;
  }

  tmpInput = inputs[9]->GetValue();
  if( stringToDouble(this, tmpInput, labels[0]->GetLabel() ))
    tmpInput.ToDouble(&T0);

  tmpInput = inputs[10]->GetValue();
  if( stringToDouble(this, tmpInput, labels[0]->GetLabel() ))
    tmpInput.ToDouble(&DELAY);

}

void IntParsDlg::UpdateDisplay()
{
  wxString tmp;
  radioBox->SetSelection(METHOD);

  tmp.Printf(_T("%g"), TOLER);
  inputs[0]->SetValue(tmp);

  tmp.Printf(_T("%g"), ATOLER);
  inputs[1]->SetValue(tmp);

  tmp.Printf(_T("%g"), HMIN);
  inputs[2]->SetValue(tmp);

  tmp.Printf(_T("%g"), HMAX);
  inputs[3]->SetValue(tmp);

  tmp.Printf(_T("%g"), BOUND);
  inputs[4]->SetValue(tmp);

  tmp.Printf(_T("%g"), TEND);
  inputs[5]->SetValue(tmp);

  tmp.Printf(_T("%g"), TRANS);
  inputs[6]->SetValue(tmp);

  tmp.Printf(_T("%g"), DELTA_T);
  inputs[7]->SetValue(tmp);

  tmp.Printf(_T("%i"), NJMP);
  inputs[8]->SetValue(tmp);

  tmp.Printf(_T("%g"), T0);
  inputs[9]->SetValue(tmp);

  tmp.Printf(_T("%g"), DELAY);
  inputs[10]->SetValue(tmp);
}

/* Controlling Parameters Dialog */

ControlParamsDlg::ControlParamsDlg(wxWindow *p)
  : wxDialog(p, -1, _T("Control Parameters"), wxDefaultPosition)
{
  parent = p;
  int i;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  wxString labelStrings[9] =
{_T("Iter"),
 _T("Eps"),
 _T("Tol"),
 _T("Iter"),
 _T("Eps"),
 _T("Dh"),
 _T("Alpha Max"),
 _T("Omega Max"),
 _T("Grid")};

  labels = new wxStaticText * [9];
  inputs = new wxTextCtrl * [9];
  for(i=0; i<9; i++)
  {
    labels[i] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);

    inputs[i] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
      wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  }

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  bvpBox = new wxStaticBox(panel, -1, _T("BVP Controls"));
  newtonBox = new wxStaticBox(panel, -1, _T("Newton Controls"));
  delayBox = new wxStaticBox(panel, -1, _T("Delay Controls"));
  
  UpdateDisplay();

  // Sizers 
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  horizSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer0 = new wxGridSizer(2);
  gridSizer1 = new wxGridSizer(2);
  gridSizer2 = new wxGridSizer(2);

  bvpBoxSizer = new wxStaticBoxSizer(bvpBox, wxVERTICAL);
  newtonBoxSizer = new wxStaticBoxSizer(newtonBox, wxVERTICAL);
  delayBoxSizer = new wxStaticBoxSizer(delayBox, wxVERTICAL);

  for(i=0; i<3; i++)
  {
    gridSizer0->Add(labels[i], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
    gridSizer0->Add(inputs[i]);
  }
  bvpBoxSizer->Add(gridSizer0);

  for(i=3; i<6; i++)
  {
    gridSizer1->Add(labels[i], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
    gridSizer1->Add(inputs[i]);
  }
  newtonBoxSizer->Add(gridSizer1);

  for(i=6; i<9; i++)
  {
    gridSizer2->Add(labels[i], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
    gridSizer2->Add(inputs[i]);
  }
  delayBoxSizer->Add(gridSizer2);

  horizSizer->Add(bvpBoxSizer);
  horizSizer->Add(newtonBoxSizer);
  horizSizer->Add(delayBoxSizer);

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  panelSizer->Add(horizSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void ControlParamsDlg::UpdateValues()
{
  wxString tmpInput;
  long tmplong;

  tmpInput = inputs[0]->GetValue();
  if( stringToInt(this, tmpInput, labels[0]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    BVP_MAXIT = (int) tmplong;
  }

  tmpInput = inputs[1]->GetValue();
  if( stringToDouble(this, tmpInput, labels[1]->GetLabel() ))
    tmpInput.ToDouble(&BVP_EPS);

  tmpInput = inputs[2]->GetValue();
  if( stringToDouble(this, tmpInput, labels[2]->GetLabel() ))
    tmpInput.ToDouble(&BVP_TOL);

  tmpInput = inputs[3]->GetValue();
  if( stringToInt(this, tmpInput, labels[3]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    EVEC_ITER = (int) tmplong;
  }

  tmpInput = inputs[4]->GetValue();
  if( stringToDouble(this, tmpInput, labels[4]->GetLabel() ))
    tmpInput.ToDouble(&NEWT_ERR);

  tmpInput = inputs[5]->GetValue();
  if( stringToDouble(this, tmpInput, labels[5]->GetLabel() ))
    tmpInput.ToDouble(&EVEC_ERR);

  tmpInput = inputs[6]->GetValue();
  if( stringToDouble(this, tmpInput, labels[6]->GetLabel() ))
    tmpInput.ToDouble(&AlphaMax);

  tmpInput = inputs[7]->GetValue();
  if( stringToDouble(this, tmpInput, labels[7]->GetLabel() ))
    tmpInput.ToDouble(&OmegaMax);

  tmpInput = inputs[8]->GetValue();
  if( stringToInt(this, tmpInput, labels[8]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    DelayGrid = (int) tmplong;
  }
}

void ControlParamsDlg::UpdateDisplay()
{
  wxString tmp;

  tmp.Printf(_T("%i"), BVP_MAXIT);
  inputs[0]->SetValue(tmp);

  tmp.Printf(_T("%g"), BVP_EPS);
  inputs[1]->SetValue(tmp);

  tmp.Printf(_T("%g"), BVP_TOL);
  inputs[2]->SetValue(tmp);

  tmp.Printf(_T("%i"), EVEC_ITER);
  inputs[3]->SetValue(tmp);

  tmp.Printf(_T("%g"), NEWT_ERR);
  inputs[4]->SetValue(tmp);

  tmp.Printf(_T("%g"), EVEC_ERR);
  inputs[5]->SetValue(tmp);

  tmp.Printf(_T("%g"), AlphaMax);
  inputs[6]->SetValue(tmp);

  tmp.Printf(_T("%g"), OmegaMax);
  inputs[7]->SetValue(tmp);

  tmp.Printf(_T("%i"), DelayGrid);
  inputs[8]->SetValue(tmp);
}

/* Phaseplane Parameters Dialog */

PhaseParamsDlg::PhaseParamsDlg(wxWindow *p)
  : wxDialog(p, -1, _T("Phaseplane Parameters"), wxDefaultPosition)
{
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  int i;

  wxString labelStrings[3] =
{_T("Nullcline Grid"),
 _T("Dir. Fld. Grid"),
 _T("Flow Grid")};
  
  labels = new wxStaticText * [3];
  inputs = new wxTextCtrl * [3];
  for(i=0; i<3; i++)
  {
    labels[i] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);

    inputs[i] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
      wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  }

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  UpdateDisplay();

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(2);  

  for(i=0; i<3; i++)
  {
    gridSizer->Add(labels[i], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(inputs[i]);
  }

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void PhaseParamsDlg::UpdateValues()
{
  wxString tmpInput;
  long tmplong;

  tmpInput = inputs[0]->GetValue();
  if( stringToInt(this, tmpInput, labels[0]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    NMESH = (int) tmplong;
  }

  tmpInput = inputs[1]->GetValue();
  if( stringToInt(this, tmpInput, labels[1]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    DF_GRID = (int) tmplong;
  }

  tmpInput = inputs[2]->GetValue();
  if( stringToInt(this, tmpInput, labels[2]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    FLOW_GRID = (int) tmplong;
  }
}

void PhaseParamsDlg::UpdateDisplay()
{
  wxString tmp;

  tmp.Printf(_T("%i"), NMESH);
  inputs[0]->SetValue(tmp);

  tmp.Printf(_T("%i"), DF_GRID);
  inputs[1]->SetValue(tmp);

  tmp.Printf(_T("%i"), FLOW_GRID);
  inputs[2]->SetValue(tmp);
}


/* Add Edit Curves Dialog */

BEGIN_EVENT_TABLE(AddEditCurvesDlg, wxDialog)
  EVT_CHOICE(AEC_COLOR0, AddEditCurvesDlg::OnColor)
  EVT_CHOICE(AEC_COLOR1, AddEditCurvesDlg::OnColor)
  EVT_CHOICE(AEC_COLOR2, AddEditCurvesDlg::OnColor)
  EVT_CHOICE(AEC_COLOR3, AddEditCurvesDlg::OnColor)
  EVT_CHOICE(AEC_COLOR4, AddEditCurvesDlg::OnColor)
  EVT_CHOICE(AEC_COLOR5, AddEditCurvesDlg::OnColor)
  EVT_CHOICE(AEC_COLOR6, AddEditCurvesDlg::OnColor)
  EVT_CHOICE(AEC_COLOR7, AddEditCurvesDlg::OnColor)
  EVT_CHOICE(AEC_COLOR8, AddEditCurvesDlg::OnColor)
  EVT_CHOICE(AEC_COLOR9, AddEditCurvesDlg::OnColor)
  EVT_BUTTON(AEC_ADD, AddEditCurvesDlg::OnAdd)
  EVT_BUTTON(AEC_DEL, AddEditCurvesDlg::OnDel)
END_EVENT_TABLE()

AddEditCurvesDlg::AddEditCurvesDlg(xppRunWindow *p)
  : wxDialog(p, -1, _T("Add Edit Curves"), wxDefaultPosition)
{
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  int i;

  wxString labelStrings[7] =
{_T("Curve"),
 _T("X-axis"),
 _T("Y-axis"),
 _T("Z-axis"),
 _T("Style"),
 _T("Color"),
 _T("Display Color")};
  
  labels = new wxStaticText * [6];
  for(i=0; i<7; i++)
  {
    labels[i] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);
  }

  addButton = new wxButton(panel, AEC_ADD, _T("Add Curve"));
  delButton = new wxButton(panel, AEC_DEL, _T("Delete Last"));
  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(7);  

  for(i=0; i<7; i++)
  {
    gridSizer->Add(labels[i], 0, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL);
  }

  UpdateDisplay();

  buttonSizer->Add(addButton, 0, wxALL, 3);
  buttonSizer->Add(delButton, 0, wxALL, 3);
  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void AddEditCurvesDlg::UpdateValues()
{
  int choice, i;
  int n=MyGraph->nvars;

  for(i=0; i<n; i++)
  {
    choice = axisChoices[3*i]->GetSelection();
    MyGraph->xv[i] = choice;
    
    choice = axisChoices[3*i+1]->GetSelection();
    MyGraph->yv[i] = choice;

    choice = axisChoices[3*i+2]->GetSelection();
    MyGraph->zv[i] = choice;

    choice = styleChoices[i]->GetSelection();
    if(choice == 0)
    {
      MyGraph->line[i] = 1;
    }
    else
    {
      MyGraph->line[i] = 0;
    }

    choice = colorChoices[i]->GetSelection();
    MyGraph->color[i] = choice;

  }
}

void AddEditCurvesDlg::UpdateDisplay()
{
  wxString tmp;
  int i;
  int n=MyGraph->nvars;

  wxString colorStrings[10] = 
{_T("0"), _T("1"), _T("2"), _T("3"), _T("4"), _T("5"), _T("6"), _T("7"), _T("8"), _T("9")};

  curveLabels = new wxStaticText * [MAXPERPLOT];
  axisChoices = new wxChoice * [3*MAXPERPLOT];
  colorChoices = new wxChoice * [MAXPERPLOT];
  colorDisplays = new wxWindow * [MAXPERPLOT];
  styleChoices = new wxChoice * [MAXPERPLOT];

  for(i=0; i<n; i++)
  {
    tmp.Printf(_T("%i"), i);
    curveLabels[i] = new wxStaticText(panel, -1, tmp, wxDefaultPosition);

    axisChoices[3*i] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
    axisChoices[3*i+1] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
    axisChoices[3*i+2] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);

    axisChoices[3*i]->Append(_T("Time"));
    axisChoices[3*i+1]->Append(_T("Time"));
    axisChoices[3*i+2]->Append(_T("Time"));

    populateUvarChoice(axisChoices[3*i]);
    populateUvarChoice(axisChoices[3*i+1]);
    populateUvarChoice(axisChoices[3*i+2]);

    axisChoices[3*i]->SetSelection(MyGraph->xv[i]);
    axisChoices[3*i+1]->SetSelection(MyGraph->yv[i]);
    axisChoices[3*i+2]->SetSelection(MyGraph->zv[i]);

    styleChoices[i] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
    styleChoices[i]->Append(_T("Line"));
    styleChoices[i]->Append(_T("Point"));

    if(MyGraph->line[i] > 0)
    {
      // This is the line style
      styleChoices[i]->SetSelection(0);
    }
    else if(MyGraph->line[i] == 0)
    {
      // This is the point style
      styleChoices[i]->SetSelection(1);
    }
    else
    {
      // This should be large beeds, < 0 styleChoices[i]->SetSelection(1);
    }

    colorChoices[i] = new wxChoice(panel, AEC_COLOR0 + i, wxDefaultPosition, wxDefaultSize, 10, colorStrings);
    colorChoices[i]->SetSelection(MyGraph->color[i]);

    colorDisplays[i] = new wxWindow(panel, -1);
    colorDisplays[i]->SetBackgroundColour(parent->GetNormalColour(MyGraph->color[i]));

    gridSizer->Add(curveLabels[i], 0, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL | wxALL, 3);
    gridSizer->Add(axisChoices[3*i], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(axisChoices[3*i+1], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(axisChoices[3*i+2], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(styleChoices[i], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(colorChoices[i], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(colorDisplays[i], 0, wxEXPAND | wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL | wxALL, 3);
  }
}

void AddEditCurvesDlg::OnColor(wxCommandEvent &event)
{
  int n = event.GetId() - AEC_COLOR0;
  int choice = colorChoices[n]->GetSelection();

  colorDisplays[n]->SetBackgroundColour(parent->GetNormalColour(choice));
  colorDisplays[n]->Refresh();
}

void AddEditCurvesDlg::OnAdd(wxCommandEvent &event)
{
  if(MyGraph->nvars >= MAXPERPLOT)
  {
    wxString msg;
    msg.Printf(_T("Can't add any more curves!"));

    wxMessageBox(msg, _T("Error"), wxOK, this);
  }
  else
  {
    int i = MyGraph->nvars;
    wxString tmp;

  wxString colorStrings[10] = 
{_T("0"), _T("1"), _T("2"), _T("3"), _T("4"), _T("5"), _T("6"), _T("7"), _T("8"), _T("9")};

    tmp.Printf(_T("%i"), i);
    curveLabels[i] = new wxStaticText(panel, -1, tmp, wxDefaultPosition);

    axisChoices[3*i] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
    axisChoices[3*i+1] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
    axisChoices[3*i+2] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);

    axisChoices[3*i]->Append(_T("Time"));
    axisChoices[3*i+1]->Append(_T("Time"));
    axisChoices[3*i+2]->Append(_T("Time"));

    populateUvarChoice(axisChoices[3*i]);
    populateUvarChoice(axisChoices[3*i+1]);
    populateUvarChoice(axisChoices[3*i+2]);

    axisChoices[3*i]->SetSelection(MyGraph->xv[i]);
    axisChoices[3*i+1]->SetSelection(MyGraph->yv[i]);
    axisChoices[3*i+2]->SetSelection(MyGraph->zv[i]);

    styleChoices[i] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
    styleChoices[i]->Append(_T("Line"));
    styleChoices[i]->Append(_T("Point"));

    if(MyGraph->line[i] > 0)
    {
      // This is the line style
      styleChoices[i]->SetSelection(0);
    }
    else if(MyGraph->line[i] == 0)
    {
      // This is the point style
      styleChoices[i]->SetSelection(1);
    }
    else
    {
      // This should be large beeds, < 0
      styleChoices[i]->SetSelection(1);
    }

    colorChoices[i] = new wxChoice(panel, AEC_COLOR0 + i, wxDefaultPosition, wxDefaultSize, 10, colorStrings);
    colorChoices[i]->SetSelection(MyGraph->color[i]);

    colorDisplays[i] = new wxWindow(panel, -1);
    colorDisplays[i]->SetBackgroundColour(parent->GetNormalColour(MyGraph->color[i]));

    gridSizer->Add(curveLabels[i], 0, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL | wxALL, 3);
    gridSizer->Add(axisChoices[3*i], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(axisChoices[3*i+1], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(axisChoices[3*i+2], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(styleChoices[i], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(colorChoices[i], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(colorDisplays[i], 0, wxEXPAND | wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL | wxALL, 3);

    MyGraph->nvars = MyGraph->nvars + 1;

    mainSizer->Fit(this);
  }
}

void AddEditCurvesDlg::OnDel(wxCommandEvent &event)
{
  if(MyGraph->nvars == 1)
  {
    wxString msg;
    msg.Printf(_T("Can't delete curve 0!"));

    wxMessageBox(msg, _T("Error"), wxOK, this);
  }
  else
  {
    int i = MyGraph->nvars = MyGraph->nvars-1;
    curveLabels[i]->Destroy();
    axisChoices[3*i]->Destroy();
    axisChoices[3*i+1]->Destroy();
    axisChoices[3*i+2]->Destroy();
    colorChoices[i]->Destroy();
    colorDisplays[i]->Destroy();
    styleChoices[i]->Destroy();
    mainSizer->Fit(this);
  }
}

/* Monte Carlo Equilibrium Dialog */

MonteCarloEqDlg::MonteCarloEqDlg(wxWindow *p)
  : wxDialog(p, -1, _T("Monte Carlo Equilibrium"), wxDefaultPosition)
{
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  int i;

  wxString labelStrings[3] =
{_T("Number of Guesses"),
 _T("Tolerance"),
 _T("Append")};
  
  labels = new wxStaticText * [3];
  inputs = new wxTextCtrl * [2];
  for(i=0; i<2; i++)
  {
    labels[i] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);

    inputs[i] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
      wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  }

  appendCheck = new wxCheckBox(panel, -1, labelStrings[2]);

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(2);  

  for(i=0; i<2; i++)
  {
    gridSizer->Add(labels[i], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(inputs[i]);
  }

  UpdateDisplay();

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  panelSizer->Add(appendCheck);
  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void MonteCarloEqDlg::UpdateValues()
{
  wxString tmpInput;
  long tmplong;
  int i;

  tmpInput = inputs[0]->GetValue();
  if( stringToInt(this, tmpInput, labels[0]->GetLabel() ))
  {
    tmpInput.ToLong(&tmplong);
    fixptguess.n = (int) tmplong;
  }

  tmpInput = inputs[1]->GetValue();
  if( stringToDouble(this, tmpInput, labels[1]->GetLabel() ))
    tmpInput.ToDouble(&fixptguess.tol);

  for(i=0; i<NEQ; i++)
  {
    tmpInput = uvarInputs[2*i]->GetValue();
    if( stringToDouble(this, tmpInput, uvarLabels[2*i]->GetLabel() ))
      tmpInput.ToDouble(&fixptguess.xlo[i]);

    tmpInput = uvarInputs[2*i+1]->GetValue();
    if( stringToDouble(this, tmpInput, uvarLabels[2*i+1]->GetLabel() ))
      tmpInput.ToDouble(&fixptguess.xhi[i]);
  }
}

void MonteCarloEqDlg::UpdateDisplay()
{
  wxString tmp;
  int i;

  appendCheck->SetValue(false);

  tmp.Printf(_T("%i"), fixptguess.n);
  inputs[0]->SetValue(tmp);

  tmp.Printf(_T("%g"), fixptguess.tol);
  inputs[1]->SetValue(tmp);

  uvarLabels = new wxStaticText * [2*NEQ];
  uvarInputs = new wxTextCtrl * [2*NEQ];

  for(i=0; i<NEQ; i++)
  {
    tmp.Printf(_T("%s lo"), uvar_names[i]);
    uvarLabels[2*i] = new wxStaticText(panel, -1, tmp, wxDefaultPosition);

    tmp.Printf(_T("%s hi"), uvar_names[i]);
    uvarLabels[2*i+1] = new wxStaticText(panel, -1, tmp, wxDefaultPosition);

    uvarInputs[2*i] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
      wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));

    uvarInputs[2*i+1] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition,
      wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));

    gridSizer->Add(uvarLabels[2*i], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(uvarInputs[2*i]);

    gridSizer->Add(uvarLabels[2*i+1], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(uvarInputs[2*i+1]);

    tmp.Printf(_T("%g"), fixptguess.xlo[i]);
    uvarInputs[2*i]->SetValue(tmp);

    tmp.Printf(_T("%g"), fixptguess.xhi[i]);
    uvarInputs[2*i+1]->SetValue(tmp);
  }
}

bool MonteCarloEqDlg::GetAppendFlag()
{
  return appendCheck->GetValue();
}

/* Add Data Col Dialog */

AddDataColDlg::AddDataColDlg(wxWindow *p)
  : wxDialog(p, -1, _T("Add Column"), wxDefaultPosition)
{
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  int i;

  wxString labelStrings[2] =
{_T("Name"),
 _T("Formula")};
  
  labels = new wxStaticText * [2];
  inputs = new wxTextCtrl * [2];
  for(i=0; i<2; i++)
  {
    labels[i] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);

    inputs[i] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition, wxDefaultSize);//, 0, wxTextValidator(wxFILTER_NUMERIC));

    inputs[i]->SetMaxLength(12);
  }

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(2);  

  for(i=0; i<2; i++)
  {
    gridSizer->Add(labels[i], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(inputs[i]);
  }

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

wxString AddDataColDlg::GetName()
{
  return inputs[0]->GetValue();
}

wxString AddDataColDlg::GetFormula()
{
  return inputs[1]->GetValue();
}

/* Find Data Value Dialog */

FindDataValueDlg::FindDataValueDlg(wxWindow *p)
  : wxDialog(p, -1, _T("Find Value"), wxDefaultPosition)
{
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  int i;

  wxString labelStrings[2] =
{_T("Variable"),
 _T("Value")};
  
  labels = new wxStaticText * [2];
  for(i=0; i<2; i++)
  {
    labels[i] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);

  }

  varBox = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);

  varBox->Append(_T("Time"));
  populateUvarChoice(varBox);

  input = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(2);  

  gridSizer->Add(labels[0], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(varBox);

  gridSizer->Add(labels[1], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(input);

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

int FindDataValueDlg::GetVarIndex()
{
  return varBox->GetSelection();
}

double FindDataValueDlg::GetValue()
{
  wxString tmpInput;
  double tmpDouble;

  tmpInput = input->GetValue();
  if( stringToDouble(this, tmpInput, labels[1]->GetLabel() ))
    tmpInput.ToDouble(&tmpDouble);

  return tmpDouble;
}

/* Export Data Table Dialog */

BEGIN_EVENT_TABLE(DataTableDlg, wxDialog)
  EVT_BUTTON( DT_BROWSE, DataTableDlg::OnBrowse)
END_EVENT_TABLE()

DataTableDlg::DataTableDlg(wxWindow *p)
  : wxDialog(p, -1, _T("Write Tabular"), wxDefaultPosition)
{
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  int i;

  wxString labelStrings[4] =
{_T("Variable"),
 _T("Xhi"),
 _T("Xlo")
 _T("File")};
  
  labels = new wxStaticText * [4];
  inputs = new wxTextCtrl * [3];

  varBox = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);

  varBox->Append(_T("Time"));
  populateUvarChoice(varBox);

  for(i=0; i<2; i++)
  {
    labels[i] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);

    inputs[i] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
  }

  labels[2] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);

  labels[3] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);

  inputs[2] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition, wxDefaultSize);

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));
  browseButton = new wxButton(panel, DT_BROWSE, _T("Browse..."));

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(2);  

  gridSizer->Add(labels[0], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(varBox);

  for(i=0; i<3; i++)
  {
    gridSizer->Add(labels[i+1], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(inputs[i]);
  }

  gridSizer->Add(browseButton, 0, wxALL, 3);

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

int DataTableDlg::GetVar()
{
  return varBox->GetSelection();
}

double DataTableDlg::GetXlo()
{
  wxString tmpInput;
  double tmpDouble;

  tmpInput = inputs[0]->GetValue();
  if( stringToDouble(this, tmpInput, labels[1]->GetLabel() ))
    tmpInput.ToDouble(&tmpDouble);

  return tmpDouble;
}

double DataTableDlg::GetXhi()
{
  wxString tmpInput;
  double tmpDouble;

  tmpInput = inputs[1]->GetValue();
  if( stringToDouble(this, tmpInput, labels[1]->GetLabel() ))
    tmpInput.ToDouble(&tmpDouble);

  return tmpDouble;

}

wxString DataTableDlg::GetFilename()
{
  return inputs[2]->GetValue();
}

void DataTableDlg::OnBrowse(wxCommandEvent &event)
{
  wxFileDialog dlg(this,_T("Save data to..."),_T(""),_T(""),_T("TAB (*.tab)|*.tab"), wxSAVE | wxOVERWRITE_PROMPT);                                           
  if (dlg.ShowModal() == wxID_OK)
  {
    inputs[2]->SetValue(dlg.GetPath());
  }
}

/* delete data col dialog */

DeleteColDlg::DeleteColDlg(wxWindow *p)
  : wxDialog(p, -1, _T("Delete Column"), wxDefaultPosition)
{
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  varBox = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);

  populateUvarChoice(varBox);

  label = new wxStaticText(panel, -1, _T("Variable"), wxDefaultPosition);

  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(2);  

  gridSizer->Add(label, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  gridSizer->Add(varBox);

  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

wxString DeleteColDlg::GetVar()
{
  return varBox->GetStringSelection();
}

