/*
xppDialogs_wx.h

Defines all dialogs the xppRunWindow uses

*/

#ifndef __XPPDIALOGSH__
#define __XPPDIALOGSH__

#include "Ccode/xpplim.h"

extern "C" int NUPAR;
extern "C" int NMarkov;
extern "C" int FIX_VAR;
extern "C" int METHOD; /* chosen method */
extern "C" int NEQ; /* number of total equations */
extern "C" int NODE; /* number of DEs */
extern "C" int *my_ode[];
extern "C" int EqType[MAXODE]; /* equation type */
extern "C" char uvar_names[MAXODE][12]; /* dep. variable names */
extern "C" char upar_names[MAXPAR][11]; /* parameter names */
extern "C" char *ode_names[MAXODE]; /* actual equation */
extern "C" int get_param_index(char *s);
extern "C" int add_expr(char *expr, int *command, int *length);
extern "C" double last_ic[MAXODE];
extern "C" int NVAR;
//extern "C" char *save_eqn[5000]; /* the input file */
extern "C" void check_windows(); /* graph bounds checking */
extern "C" void check_flags(); /* graph flag checking */
extern "C" double range_item();
extern "C" double range_item2();
extern "C" double TOLER; /*tolerance*/
extern "C" double ATOLER; /*absolute tolerance*/
extern "C" double HMIN; /*min step*/
extern "C" double HMAX; /*max step*/
extern "C" double BOUND; /*bounds*/
extern "C" double TEND; /*total time*/
extern "C" double FOREVER; /*1 if total time<0*/
extern "C" double TRANS; /*transient time*/
extern "C" double DELTA_T; /*guess...*/
extern "C" int NJMP; /*nout*/
extern "C" double T0; /*start time*/
extern "C" double DELAY; /*max delay*/

extern "C" int NMESH; /* num grids for nullclines */
extern "C" int DF_GRID; /* num grids for dir field */
extern "C" int FLOW_GRID; /* num grids for flow field */

extern "C" int BVP_MAXIT; /* bvp max iterations */
extern "C" double BVP_EPS; /* bvp epsilon */
extern "C" double BVP_TOL; /* bvp tolerance */
extern "C" int EVEC_ITER; /* newton iterations */
extern "C" double NEWT_ERR; /* newton tolerance */
extern "C" double EVEC_ERR; /* eigenvector tolerance */
extern "C" double AlphaMax;
extern "C" double OmegaMax;
extern "C" int DelayGrid;

class xppRunWindow;

void populateUvarChoice(wxChoice *choiceBox);
void populateParamChoice(wxChoice *choiceBox);
bool stringToDouble(wxWindow *parent, wxString input, wxString label);
bool stringToInt(wxWindow *parent, wxString input, wxString label);

class IntRangeDlg : public wxDialog
{
  public:
    IntRangeDlg(wxWindow *p);
    void UpdateValues();
    void UpdateDisplay();

  private:
    wxWindow *parent;
    wxPanel *panel;

    wxStaticText **labels;
    wxTextCtrl **inputs;
    wxCheckBox **checks;
    wxChoice *paramBox;


    wxButton *okButton;
    wxButton *cancelButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxGridSizer *gridSizer;
    wxBoxSizer *buttonSizer;
};

class Int2RangeDlg : public wxDialog
{
  public:
    Int2RangeDlg(wxWindow *p);
    void UpdateValues();
    void UpdateDisplay();

  private:
    wxWindow *parent;
    wxPanel *panel;

    wxStaticText **labels;
    wxTextCtrl **inputs;
    wxCheckBox **checks;
    wxChoice **paramBoxes;
    wxRadioBox *radioBox;

    wxButton *okButton;
    wxButton *cancelButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxGridSizer *gridSizer;
    wxBoxSizer *buttonSizer;
};

class BVPRangeDlg : public wxDialog
{
  public:
    BVPRangeDlg(wxWindow *p);
    void UpdateValues();
    void UpdateDisplay();

  private:
    wxWindow *parent;
    wxPanel *panel;

    wxStaticText **labels;
    wxTextCtrl **inputs;
    wxRadioBox *radioBox;
    wxChoice *paramBox;
    wxCheckBox *check;

    wxButton *okButton;
    wxButton *cancelButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxGridSizer *gridSizer;
    wxBoxSizer *buttonSizer;
};

class FixedPtRangeDlg : public wxDialog
{
  public:
    FixedPtRangeDlg(wxWindow *p);
    void UpdateValues();
    void UpdateDisplay();

  private:
    wxWindow *parent;
    wxPanel *panel;

    wxStaticText **labels;
    wxTextCtrl **inputs;
    wxCheckBox *check;
    wxChoice *paramBox;
    wxChoice *stabilityBox;

    wxButton *okButton;
    wxButton *cancelButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxGridSizer *gridSizer;
    wxBoxSizer *buttonSizer;
};

class ViewDlg : public wxDialog
{
  public:
    ViewDlg(xppRunWindow *p);
    void OnRadio(wxCommandEvent &WXUNUSED(event));
    void Disable3D();
    void Enable3D();
    void UpdateValues();
    void UpdateDisplay();
    bool GetFitFlag();

  private:
    xppRunWindow *parent;
    wxPanel *panel;

    wxStaticText **labels;
    wxTextCtrl **inputs;
    wxChoice **axisChoices;
    wxRadioBox *radioBox;
    wxCheckBox *fitCheck;

    wxButton *okButton;
    wxButton *applyButton;
    wxButton *applyGoButton;
    wxButton *defaultButton;
    wxButton *cancelButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxBoxSizer *topHorizSizer;
    wxGridSizer *gridSizer;
    wxBoxSizer *buttonSizer;

  DECLARE_EVENT_TABLE()
};

enum
{
  VIEW_RADIO = 1400,
};

class IntParsDlg : public wxDialog
{
  public:
    IntParsDlg(wxWindow *p);
    void UpdateValues();
    void UpdateDisplay();

  private:
    wxWindow *parent;
    wxPanel *panel;

    wxStaticText **labels;
    wxTextCtrl **inputs;
    wxRadioBox *radioBox;
    wxStaticBox *tolBox;
    wxStaticBox *timeBox;

    wxButton *okButton;
    wxButton *cancelButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxBoxSizer *vertSizer;
    wxBoxSizer *horizSizer;
    wxGridSizer *gridSizer1;
    wxGridSizer *gridSizer2;
    wxBoxSizer *buttonSizer;
    wxStaticBoxSizer *tolBoxSizer;
    wxStaticBoxSizer *timeBoxSizer;
};

class ControlParamsDlg : public wxDialog
{
  public:
    ControlParamsDlg(wxWindow *p);
    void UpdateValues();
    void UpdateDisplay();

  private:
    wxWindow *parent;
    wxPanel *panel;

    wxStaticText **labels;
    wxTextCtrl **inputs;

    wxButton *okButton;
    wxButton *cancelButton;

    wxStaticBox *bvpBox;
    wxStaticBox *newtonBox;
    wxStaticBox *delayBox;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxBoxSizer *horizSizer;
    wxGridSizer *gridSizer0;
    wxGridSizer *gridSizer1;
    wxGridSizer *gridSizer2;
    wxStaticBoxSizer *bvpBoxSizer;
    wxStaticBoxSizer *newtonBoxSizer;
    wxStaticBoxSizer *delayBoxSizer;
    wxBoxSizer *buttonSizer;
};

class PhaseParamsDlg : public wxDialog
{
  public:
    PhaseParamsDlg(wxWindow *p);
    void UpdateValues();
    void UpdateDisplay();

  private:
    wxWindow *parent;
    wxPanel *panel;

    wxStaticText **labels;
    wxTextCtrl **inputs;

    wxButton *okButton;
    wxButton *cancelButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxGridSizer *gridSizer;
    wxBoxSizer *buttonSizer;
};

class AddEditCurvesDlg : public wxDialog
{
  public:
    AddEditCurvesDlg(xppRunWindow *p);
    void UpdateValues();
    void UpdateDisplay();
    void OnColor(wxCommandEvent &event);
    void OnAdd(wxCommandEvent &event);
    void OnDel(wxCommandEvent &event);

  private:
    xppRunWindow *parent;
    wxPanel *panel;

    wxStaticText **labels;
    wxStaticText **curveLabels;
    wxChoice **axisChoices;
    wxChoice **colorChoices;
    wxWindow **colorDisplays;
    wxChoice **styleChoices;

    wxButton *addButton;
    wxButton *delButton;
    wxButton *okButton;
    wxButton *cancelButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxGridSizer *gridSizer;
    wxBoxSizer *buttonSizer;

  DECLARE_EVENT_TABLE()
};

enum
{
  AEC_COLOR0 = 1500,
  AEC_COLOR1,
  AEC_COLOR2,
  AEC_COLOR3,
  AEC_COLOR4,
  AEC_COLOR5,
  AEC_COLOR6,
  AEC_COLOR7,
  AEC_COLOR8,
  AEC_COLOR9,
  AEC_ADD,
  AEC_DEL
};

class MonteCarloEqDlg : public wxDialog
{
  public:
    MonteCarloEqDlg(wxWindow *p);
    void UpdateValues();
    void UpdateDisplay();
    bool GetAppendFlag();

  private:
    wxWindow *parent;
    wxPanel *panel;

    wxStaticText **labels;
    wxStaticText **uvarLabels;
    wxTextCtrl **inputs;
    wxTextCtrl **uvarInputs;
    wxCheckBox *appendCheck;

    wxButton *okButton;
    wxButton *cancelButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxGridSizer *gridSizer;
    wxBoxSizer *buttonSizer;
};

class AddDataColDlg : public wxDialog
{
  public:
    AddDataColDlg(wxWindow *p);
    wxString GetName();
    wxString GetFormula();

  private:
    wxWindow *parent;
    wxPanel *panel;

    wxStaticText **labels;
    wxTextCtrl **inputs;

    wxButton *okButton;
    wxButton *cancelButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxGridSizer *gridSizer;
    wxBoxSizer *buttonSizer;
};

class FindDataValueDlg : public wxDialog
{
  public:
    FindDataValueDlg(wxWindow *p);
    int GetVarIndex();
    double GetValue();

  private:
    wxWindow *parent;
    wxPanel *panel;

    wxStaticText **labels;
    wxChoice *varBox;
    wxTextCtrl *input;

    wxButton *okButton;
    wxButton *cancelButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxGridSizer *gridSizer;
    wxBoxSizer *buttonSizer;
};

class DataTableDlg : public wxDialog
{
  public:
    DataTableDlg(wxWindow *p);
    int GetVar();
    double GetXlo();
    double GetXhi();
    wxString GetFilename();

  private:
    void OnBrowse(wxCommandEvent &event);

    wxWindow *parent;
    wxPanel *panel;

    wxStaticText **labels;
    wxTextCtrl **inputs;
    wxChoice *varBox;

    wxButton *okButton;
    wxButton *cancelButton;
    wxButton *browseButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxGridSizer *gridSizer;
    wxBoxSizer *buttonSizer;

  DECLARE_EVENT_TABLE();
};

enum
{
  DT_BROWSE = 1600
};

class DeleteColDlg : public wxDialog
{
  public:
    DeleteColDlg(wxWindow *p);
    wxString GetVar();

  private:
    wxWindow *parent;
    wxPanel *panel;

    wxStaticText *label;
    wxChoice *varBox;

    wxButton *okButton;
    wxButton *cancelButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxGridSizer *gridSizer;
    wxBoxSizer *buttonSizer;
};

#endif

