/*
xppEditRHSWin_wx.cpp

The RHS editor
*/
#include <wx/wxprec.h>

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif

#include <math.h>

#include "xppRunWindow_wx.h"

#include "xppEditRHSWin_wx.h"

BEGIN_EVENT_TABLE(xppEditRHSWin, wxDialog)
  EVT_BUTTON (RHS_OK, xppEditRHSWin::OnOk )
  EVT_BUTTON (RHS_APPLY, xppEditRHSWin::OnApply )
  EVT_BUTTON (RHS_APPLY_GO, xppEditRHSWin::OnApplyGo )
  EVT_BUTTON (RHS_CANCEL, xppEditRHSWin::OnCancel )
  EVT_CLOSE( xppEditRHSWin::OnCloseWindow )
END_EVENT_TABLE()

xppEditRHSWin::xppEditRHSWin(xppRunWindow *p)
  : wxDialog(p, -1, _T("RHS Editor"), wxDefaultPosition)
{
  int i;
  int n = NEQ;
  wxString tmp;
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);
  labels = new wxStaticText * [n];
  inputs = new wxTextCtrl * [n];

  applyButton = new wxButton(panel, RHS_APPLY, _T("Apply"));
  applyGoButton = new wxButton(panel, RHS_APPLY_GO, _T("Go"));
  okButton = new wxButton(panel, RHS_OK, _T("OK"));
  cancelButton = new wxButton(panel, RHS_CANCEL, _T("Cancel"));

  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  // make the grid approximately square
  gridSizer = new wxGridSizer(2 * (int)sqrt(n));

  for(i=0; i<n; i++)
  {
    if(i<NODE && METHOD>0)
      tmp.Printf(_T("d%s/dT = "), uvar_names[i]);
    if(i<NODE && METHOD==0)
      tmp.Printf(_T("%s(n+1) = "), uvar_names[i]);
    if(i<NODE && EqType[i]==1)
      tmp.Printf(_T("%s(T) = "), uvar_names[i]);
    if(i>=NODE)
      tmp.Printf(_T("%s = "), uvar_names[i]);

    labels[i] = new wxStaticText(panel, -1, tmp);

    tmp.Printf(_T("%s"), ode_names[i]);
    inputs[i] = new wxTextCtrl(panel, -1, tmp, wxDefaultPosition,
      wxDefaultSize, 0, wxTextValidator(wxFILTER_NONE));

    gridSizer->Add(labels[i], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(inputs[i], 0, wxEXPAND);
  }

  buttonSizer->Add(applyButton, 0, wxALL, 3);
  buttonSizer->Add(applyGoButton, 0, wxALL, 3);
  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void xppEditRHSWin::OnOk(wxCommandEvent &WXUNUSED(event))
{
  Hide();
  UpdateValues();
}

void xppEditRHSWin::OnApply(wxCommandEvent &WXUNUSED(event))
{
  UpdateValues();
}

void xppEditRHSWin::OnApplyGo(wxCommandEvent &WXUNUSED(event))
{
  UpdateValues();
  parent->Go();
}

void xppEditRHSWin::OnCancel(wxCommandEvent &WXUNUSED(event))
{
  Hide();
  /* Reset unapplied changes */
  UpdateDisplay();
}

void xppEditRHSWin::OnCloseWindow(wxCloseEvent &WXUNUSED(event))
{
  Hide();
  /* Reset unapplied changes */
  UpdateDisplay();
}

void xppEditRHSWin::UpdateValues()
{
  int i, j, i0, err;
  int len;
  char eqn[86];
  wxString msg;
  int **command;

  command = (int **)malloc(NEQ*sizeof(int*));

  for(i=0; i<NEQ; i++)
    command[i] = (int *)malloc(200*sizeof(int));

  for(i=0; i<NEQ; i++)
  {
#ifdef wxUSE_UNICODE
    strcpy(eqn, inputs[i]->GetValue().mb_str()); 
    strcpy(eqn, inputs[i]->GetValue().mb_str()); 
#else
    strcpy(eqn, inputs[i]->GetValue().c_str()); 
    strcpy(eqn, inputs[i]->GetValue().c_str()); 
#endif
    if(i<NODE || (i>=(NODE + NMarkov)) )
    {
      err=add_expr(eqn, command[i], &len);
      if(err==1)
      {
        msg = _T("Bad RHS:") + labels[i]->GetLabel() + inputs[i]->GetValue();
        wxMessageDialog dlg(parent, msg, _T("Error"), wxOK, wxDefaultPosition);
        dlg.ShowModal();
      }
      else 
      {
        free(ode_names[i]);
        ode_names[i]=(char *)malloc(inputs[i]->GetValue().Length()+5);
        strcpy(ode_names[i],eqn);
        i0=i;
        if(i>=NODE)
          i0 = i0 + FIX_VAR - NMarkov;
        for(j=0; j<len; j++)
          my_ode[i0][j] = command[i][j];
      }
    }
  }

  for(i=0; i<NEQ; i++)
  {
    free(command[i]);
  }
  free(command);
}

void xppEditRHSWin::UpdateDisplay()
{
  int i;
  int n = NEQ;
  wxString tmp;

  for(i=0; i<n; i++)
  {
    tmp.Printf(_T("%s"), ode_names[i]);
    inputs[i]->SetValue(tmp);
  }
}

