/*
xppInitDataWin_wx.cpp

Window that shows all initial data

*/
#include <wx/wxprec.h>

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif

#include <math.h>

#include "xppRunWindow_wx.h"

#include "xppInitDataWin_wx.h"

BEGIN_EVENT_TABLE(xppInitDataWin, wxDialog)
  EVT_BUTTON (INITD_OK, xppInitDataWin::OnOk )
  EVT_BUTTON (INITD_APPLY, xppInitDataWin::OnApply )
  EVT_BUTTON (INITD_APPLY_GO, xppInitDataWin::OnApplyGo )
  EVT_BUTTON (INITD_CANCEL, xppInitDataWin::OnCancel )
  EVT_CLOSE( xppInitDataWin::OnCloseWindow )
END_EVENT_TABLE()

xppInitDataWin::xppInitDataWin(xppRunWindow *p)
  : wxDialog(p, -1, _T("Initial Data"), wxDefaultPosition)
{
  int i;
  int n = NODE + NMarkov;
  wxString tmp;
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);
  labels = new wxStaticText * [n];
  inputs = new wxTextCtrl * [n];

  applyButton = new wxButton(panel, INITD_APPLY, _T("Apply"));
   applyGoButton = new wxButton(panel, INITD_APPLY_GO, _T("Go"));
  okButton = new wxButton(panel, INITD_OK, _T("OK"));
  cancelButton = new wxButton(panel, INITD_CANCEL, _T("Cancel"));

  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  // make the grid approximately square
  gridSizer = new wxGridSizer(2 * (int)sqrt(n));

  for(i=0; i<n; i++)
  {
    tmp.Printf(_T("%s(0)="), uvar_names[i]);
    labels[i] = new wxStaticText(panel, -1, tmp);

    tmp.Printf(_T("%.16g"), last_ic[i]);
    inputs[i] = new wxTextCtrl(panel, -1, tmp, wxDefaultPosition,
      wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));

    gridSizer->Add(labels[i], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(inputs[i]);
  }

  buttonSizer->Add(applyButton, 0, wxALL, 3);
  buttonSizer->Add(applyGoButton, 0, wxALL, 3);
  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void xppInitDataWin::OnOk(wxCommandEvent &WXUNUSED(event))
{
  Hide();
  UpdateValues();
}

void xppInitDataWin::OnApply(wxCommandEvent &WXUNUSED(event))
{
  UpdateValues();
}

void xppInitDataWin::OnApplyGo(wxCommandEvent &WXUNUSED(event))
{
  UpdateValues();
  parent->Go();
}

void xppInitDataWin::OnCancel(wxCommandEvent &WXUNUSED(event))
{
  Hide();
  /* Reset unapplied changes */
  UpdateDisplay();
}

void xppInitDataWin::OnCloseWindow(wxCloseEvent &WXUNUSED(event))
{
  Hide();
  /* Reset unapplied changes */
  UpdateDisplay();
}

void xppInitDataWin::UpdateValues()
{
  int i;
  double testValue;
  int n = NODE + NMarkov;
  wxString tmp, msg;
  bool convertFlag;

  for(i=0; i<n; i++)
  {
    tmp = inputs[i]->GetValue();
    convertFlag = tmp.ToDouble(&testValue);

    if(convertFlag)
    {
      last_ic[i] = testValue;
    }
    else
    {
      msg = _T("Couldn't convert ") + labels[i]->GetLabel() + _T("(0)=") + tmp + _T(" to a double.");
      wxMessageDialog dlg(parent, msg, _T("Error"), wxOK, wxDefaultPosition);
      dlg.ShowModal();
    }
  }
}

void xppInitDataWin::UpdateDisplay()
{
  int i;
  int n = NODE + NMarkov;
  wxString tmp;

  for(i=0; i<n; i++)
  {
    tmp.Printf(_T("%.16g"), last_ic[i]);
    inputs[i]->SetValue(tmp);
  }
}

