/*
xppMainWindow_wx.cpp

xppMainWindow contains a wxPanel which contains the wxNotebook that
holds the code editor and potentially other views of ODE input.
The sizes of the windows and controls are managed by sizers that
are private members of the xppMainWindow class.

*/

#include <stdio.h>
#include <wx/wxprec.h>

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif

#include <wx/notebook.h>
#include <wx/filedlg.h>
#include <wx/fdrepdlg.h>
#include <wx/accel.h>
#include <wx/fontdlg.h>

#include "interfaceCPP_wx.h"

#include "xppaut.h"
#include "xppLogWindow_wx.h"
#include "xppCodePanel_wx.h"
#include "xppRunWindow_wx.h"
#include "Ccode/defines.h"

#include "xppMainWindow_wx.h"

#define XPPVERSION 0.5

extern xppLogWindow *appLog;
extern xppaut *xppautPtr;

BEGIN_EVENT_TABLE(xppMainWindow, wxFrame)
  EVT_MENU (wxID_NEW,       xppMainWindow::OnNew)
  EVT_MENU (wxID_OPEN,      xppMainWindow::OnOpen)
  EVT_MENU (wxID_SAVE,      xppMainWindow::OnSave)
  EVT_MENU (wxID_SAVEAS,    xppMainWindow::OnSaveAs)
  EVT_MENU (wxID_EXIT,      xppMainWindow::OnExit)
  EVT_MENU (wxID_SELECTALL, xppMainWindow::OnSelectAll)
  EVT_MENU (wxID_ABOUT,     xppMainWindow::OnAbout)
  EVT_CLOSE(                xppMainWindow::OnWindowClose)
  EVT_MENU (XPP_RUN_PLOT,   xppMainWindow::OnRunPlot)
  EVT_MENU (XPP_RUN_BATCH,  xppMainWindow::OnRunBatch)
  EVT_MENU (XPP_RUN_PS,     xppMainWindow::OnRunPS)
  EVT_MENU (XPP_LOAD,       xppMainWindow::OnLoad)
  EVT_MENU (XPP_SHOW_LOG,   xppMainWindow::OnShowLog)
  EVT_MENU (XPP_SET_FONT,   xppMainWindow::OnSetFont)
#if wxUSE_FINDREPLDLG
  EVT_MENU (XPP_FIND,    xppMainWindow::OnFind)
  EVT_MENU (XPP_FIND_REPLACE, xppMainWindow::OnFindReplace)
#endif
END_EVENT_TABLE()

xppMainWindow::xppMainWindow()
  : wxFrame(NULL, -1, _T(""))
{
#ifdef __WXMSW__
  SetIcon(wxICON(xppaut));
#endif
  haveOpenedFile = false;
  haveSavedFile = false;
  // Fix up the program version and set the inital title
  versionString.Printf(_T("%g"), XPPVERSION);
  codeFilename.Printf(_T("%s"), _T("untitled.ode"));
  codeFullFilename.Printf(_T("%s"), _T("untitled.ode"));
  SetWindowTitle(codeFilename);

  // Keyboard Shortcuts
  accelEntries[0].Set(wxACCEL_CTRL, (int) 'A', wxID_SELECTALL);
  accelTable = new wxAcceleratorTable(1, accelEntries);
  SetAcceleratorTable(*accelTable);

#if wxUSE_MENUS
  // how to add a submenu - mainEdit is a wxMenu
  // mainFile->Append(SOME_ID, "Edit", mainEdit);

  // File menu 
  wxMenu *mainFile = new wxMenu();
  mainFile->Append(wxID_NEW, _T("New\tCtrl-N"), _T("New ODE File"));
  mainFile->Append(wxID_OPEN, _T("Open...\tCtrl-O"), _T("Open ODE File"));
  mainFile->AppendSeparator();
  mainFile->Append(wxID_SAVE, _T("Save\tCtrl-S"), _T("Save ODE File"));
  mainFile->Append(wxID_SAVEAS, _T("Save as..."), _T("Save ODE File with a new filename"));
  mainFile->AppendSeparator();
  mainFile->Append(XPP_SET_FONT, _T("Font..."), _T("Set the display font for the editor"));
  mainFile->AppendSeparator();
  mainFile->Append(wxID_EXIT, _T("Exit\tCtrl-Q"), _T("Exit XPPAUT"));

/*
  // Edit menu
  wxMenu *mainEdit = new wxMenu();
  mainEdit->Append(wxID_UNDO, _T("Undo"));
  mainEdit->Append(wxID_REDO, _T("Redo"));
  mainEdit->AppendSeparator();
  mainEdit->Append(wxID_CUT, _T("Cut\tCtrl-X"), _T("Cut selected text to clipboard"));
  mainEdit->Append(wxID_COPY, _T("Copy\tCtrl-C"), _T("Copy selected text to clipboard"));
  mainEdit->Append(wxID_PASTE, _T("Paste\tCtrl-V"), _T("Paste clipboard into ODE file"));
  mainEdit->Append(wxID_SELECTALL, _T("Select All\tCtrl-A"), _T("Select All"));
  mainEdit->AppendSeparator();
#if wxUSE_FINDREPLDLG
  mainEdit->Append(XPP_FIND, _T("Find...\tCtrl-F"), _T("Find text in ODE file"));
  mainEdit->Append(XPP_FIND_REPLACE, _T("Find and Replace...\tCtrl-F"), _T("Find and Replace text in ODE file"));
#endif
  mainEdit->Append(XPP_SET_FONT, _T("Font..."), _T("Set the display font for the editor"));
*/

  // Run menu
  wxMenu* mainRun = new wxMenu();
  mainRun->Append(XPP_LOAD, _T("Load"), _T("Load the equations for solving"));
  mainRun->Append(XPP_RUN_PLOT, _T("Go\tCtrl-G"), _T("Run ODE and plot results"));
  mainRun->Append(XPP_RUN_BATCH, _T("Batch to file"), _T("Batch integrate ODE and store to file"));
//  mainRun->Append(XPP_RUN_PS, "Run to PS", "Solve the ODE and output a Postcript graph");

  // Help menu
  wxMenu *mainHelp = new wxMenu();
  mainHelp->Append(XPP_SHOW_LOG, _T("Show Log"), _T("Show Log"));
//  mainHelp->Append(wxID_HELP, _T("Online Help..."), _T("The not yet there online help system"));
  mainHelp->Append(wxID_ABOUT, _T("About..."), _T("What this is all about"));

  // Create the main menu bar
  wxMenuBar *mainMenuBar = new wxMenuBar();
  mainMenuBar->Append(mainFile, _T("&File"));
//  mainMenuBar->Append(mainEdit, _T("&Edit"));
  mainMenuBar->Append(mainRun, _T("&Run"));
  mainMenuBar->Append(mainHelp, _T("&Help"));

  SetMenuBar(mainMenuBar);
#endif

#if wxUSE_STATUSBAR
  CreateStatusBar();
  SetStatusText(_T("Welcome to XPPAUT"));
#endif

  mainPanel = (wxPanel *) NULL;
  mainCode = (xppCodePanel *) NULL;

  mainPanel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  mainCode = new xppCodePanel(mainPanel);

  // Sizer stuff
  mainFrameSizer = new wxBoxSizer(wxVERTICAL);
  mainPanelSizer = new wxBoxSizer(wxVERTICAL);

  mainFrameSizer->SetMinSize(wxSize(640,480));
  mainFrameSizer->Add(mainPanelSizer, 1, wxEXPAND);

  mainPanelSizer->Add(mainCode, 1, wxEXPAND);
  mainPanelSizer->Layout();

  mainPanel->SetSizer(mainFrameSizer);
  mainPanel->SetAutoLayout(TRUE);

  mainFrameSizer->Fit(this);

  // Initial font sizing stuff for the first run
  wxPaintDC dc(this);
  wxFont *myFont = new wxFont(-1, wxMODERN, wxNORMAL, wxNORMAL);
  dc.SetFont(*myFont);
  setBigFontParams (&dc);
  setSmallFontParams (&dc);
}

void xppMainWindow::SetWindowTitle(wxString filename)
{
  newTitle.Printf(_T("XPPAUT %s >> %s"), versionString.c_str(), filename.c_str());
  SetTitle(newTitle);
}

void xppMainWindow::OnNew(wxCommandEvent &event)
{

}

void xppMainWindow::OnOpen(wxCommandEvent &event)
{
  wxFileDialog dialog(this,_T("Open ODE file..."),_T(""),_T(""),_T("ODE (*.ode)|*.ode|All files (*.*)|*.*"), wxOPEN|wxFILE_MUST_EXIST);
  if (dialog.ShowModal() == wxID_OK)
  {
    codeFilename = dialog.GetFilename();
    codeFullFilename = dialog.GetPath();
    mainCode->LoadFile(codeFullFilename);  

    SetWindowTitle(codeFilename);
    haveOpenedFile = true;
  }
}

void xppMainWindow::OnSave(wxCommandEvent &event)
{
  mainCode->SaveFile(codeFullFilename);
  haveSavedFile = true;
}

void xppMainWindow::OnSaveAs(wxCommandEvent &event)
{
  wxFileDialog dialog(this,_T("Save As..."),_T(""),_T(""),_T("ODE (*.ode)|*.ode"), wxSAVE | wxOVERWRITE_PROMPT); 
  if (dialog.ShowModal() == wxID_OK)
  {
    codeFilename = dialog.GetFilename();
    codeFullFilename = dialog.GetPath();
    mainCode->SaveFile(dialog.GetPath());
    haveSavedFile = true;
  }

  SetWindowTitle(codeFilename);
}

void xppMainWindow::OnSelectAll(wxCommandEvent &event)
{
  mainCode->SelectAll();
}

#if wxUSE_FINDREPLDLG
void xppMainWindow::OnFind(wxCommandEvent &event)
{
  mainCode->FindText();
}

void xppMainWindow::OnFindReplace(wxCommandEvent &event)
{
  mainCode->FindReplaceText();
}
#endif

void xppMainWindow::OnExit(wxCommandEvent &event)
{
  CheckAndExit(event);
}

void xppMainWindow::OnWindowClose(wxCloseEvent &event)
{
  CheckAndExit(event);
}

void xppMainWindow::CheckAndExit(wxEvent &event)
{
  bool modified = mainCode->IsModified();

  if(modified)
  {
    wxMessageDialog dialog(this, _T("ODE file is Modified\nQuit anyway?"),_T("Whoa there"), wxYES_NO|wxICON_EXCLAMATION);

    if(dialog.ShowModal() == wxID_YES)
    {
      appLog->Destroy();
      Destroy();
    }
  }
  else
  {
    appLog->Destroy();
    Destroy();
  }
}

void xppMainWindow::OnShowLog(wxCommandEvent &event)
{
  appLog->Show();
}

void xppMainWindow::OnRunPlot(wxCommandEvent &event)
{
  bool runTheModel;

  if(mainCode->IsModified())
  {
    /* they changed something */
    wxMessageDialog *dlg = new wxMessageDialog(this, _T("ODE file is Modified\nSave and run?"), _T("Whoa there"), wxYES_NO|wxICON_EXCLAMATION);

    if(dlg->ShowModal() == wxID_YES)
    {
      /* save and run */
      if(haveSavedFile)
      {
        /* they already picked a name */
        mainCode->SaveFile(codeFullFilename);
        runTheModel = true;
      }
      else
      {
        /* Do a 'save as' deal */
        wxFileDialog dialog(this,_T("Save As..."),_T(""),_T(""),_T("ODE (*.ode)|*.ode"), wxSAVE | wxOVERWRITE_PROMPT); 
        if (dialog.ShowModal() == wxID_OK)
        {
          codeFilename = dialog.GetFilename();
          codeFullFilename = dialog.GetPath();
          mainCode->SaveFile(dialog.GetPath());
          haveSavedFile = true;
        }
        SetWindowTitle(codeFilename);
        runTheModel = true;
      }
    }
    else
    {
      /* canceled the save dialog */
      runTheModel = false;
    }
  }
  else
  {
    /* they haven't changed anything */
    if(haveOpenedFile
    || haveSavedFile)
    {
      /* they opened a file
         or saved their last modifications */
      runTheModel = true;
    }
    else
    {
      /* haven't changed anything,
         didn't open anything.
         I'm not doing anything */
      runTheModel = false;
    }
  }

  if(runTheModel)
  {
    int err_flag;
    char *inputFile;

    inputFile = (char *) malloc (sizeof (char)*(codeFullFilename.Length()));
    strcpy(inputFile, codeFullFilename.mb_str());

    err_flag = onGo(inputFile, NULL, RUNPLOT);
    if(err_flag == ODE_ERROR)
    {
      appLog->Show(true);
    }
    else
    {
      xppautPtr->runWindow = new xppRunWindow(this, codeFullFilename, newTitle, GetPosition());

      Hide();
      xppautPtr->runWindow->Show(TRUE);
    }
  }
}

void xppMainWindow::OnRunBatch(wxCommandEvent &event)
{
  bool runTheModel;

  if(mainCode->IsModified())
  {
    wxMessageDialog *dlg = new wxMessageDialog(this, _T("ODE file is Modified\nSave and run?"), _T("Whoa there"), wxYES_NO);

    if(dlg->ShowModal() == wxID_YES)
    {
      /* save and run */
      if(haveSavedFile)
      {
        /* they already picked a name */
        mainCode->SaveFile(codeFullFilename);
        runTheModel = true;
      }
      else
      {
        /* Do a 'save as' deal */
        wxFileDialog dialog(this,_T("Save As..."),_T(""),_T(""),_T("ODE (*.ode)|*.ode"), wxSAVE | wxOVERWRITE_PROMPT); 
        if (dialog.ShowModal() == wxID_OK)
        {
          codeFilename = dialog.GetFilename();
          codeFullFilename = dialog.GetPath();
          mainCode->SaveFile(dialog.GetPath());
          haveSavedFile = true;
        }
        SetWindowTitle(codeFilename);
        runTheModel = true;
      }
    }
    else
    {
      runTheModel = false;
    }
  }
  else
  {
    /* they haven't changed anything */
    if(haveOpenedFile
    || haveSavedFile)
    {
      /* they opened a file 
         or saved their last modifications */
      runTheModel = true;
    }
    else
    {
      /* haven't changed anything,
         didn't open anything.
         I'm not doing anything */
      runTheModel = false;
    }
  }
  if(runTheModel)
  {
    char *inputFile;
    char *outputFile = NULL;

  // need a dialog here to get the output file.  Right now the output is output.ps

    inputFile = (char *) malloc (sizeof (char)*(codeFullFilename.Length()));
    //outputFile = (char *) malloc (sizeof (char)*( OUTPUT HERE ));
    strcpy(inputFile, codeFullFilename.mb_str());
    onGo(inputFile, outputFile, RUNBATCH);
  }
}

void xppMainWindow::OnRunPS(wxCommandEvent &event)
{
  bool runTheModel;

  if(mainCode->IsModified())
  {
    wxMessageDialog *dlg = new wxMessageDialog(this, _T("ODE file is Modified\nSave and run?"), _T("Whoa there"), wxYES_NO);

    if(dlg->ShowModal() == wxID_YES)
    {
      /* save and run */
      if(haveSavedFile)
      {
        /* they already picked a name */
        mainCode->SaveFile(codeFullFilename);
        runTheModel = true;
      }
      else
      {
        /* Do a 'save as' deal */
        wxFileDialog dialog(this,_T("Save As..."),_T(""),_T(""),_T("ODE (*.ode)|*.ode"), wxSAVE | wxOVERWRITE_PROMPT); 
        if (dialog.ShowModal() == wxID_OK)
        {
          codeFilename = dialog.GetFilename();
          codeFullFilename = dialog.GetPath();
          mainCode->SaveFile(dialog.GetPath());
          haveSavedFile = true;
        }
        SetWindowTitle(codeFilename);
        runTheModel = true;
      }
    }
    else
    {
      runTheModel = false;
    }
  }
  else
  {
    /* they haven't changed anything */
    if(haveOpenedFile
    || haveSavedFile)
    {
      /* they opened a file
         or saved their last modifications */
      runTheModel = true;
    }
    else
    {
      /* haven't changed anything,
         didn't open anything.
         I'm not doing anything */
      runTheModel = false;
    }
  }
  if(runTheModel)
  {
    char *inputFile;
    char *outputFile = NULL;

  // need a dialog here to get the output file.  Right now the output is output.ps

    inputFile = (char *) malloc (sizeof (char)*(codeFullFilename.Length()));
    //outputFile = (char *) malloc (sizeof (char)*( OUTPUT HERE ));
    strcpy(inputFile, codeFullFilename.mb_str());
    onGo(inputFile, outputFile, RUNPS);
  }
}

void xppMainWindow::OnLoad(wxCommandEvent &event)
{
  bool runTheModel;

  if(mainCode->IsModified())
  {
    wxMessageDialog *dlg = new wxMessageDialog(this, _T("ODE file is Modified\nSave and run?"), _T("Whoa there"), wxYES_NO);

    if(dlg->ShowModal() == wxID_YES)
    {
      mainCode->SaveFile(codeFullFilename);
      runTheModel = true;
    }
    else
    {
      runTheModel = false;
    }
  }
  else
  {
    /* they haven't changed anything */
    if(haveOpenedFile)
    {
      /* they opened a file */
      runTheModel = true;
    }
    else
    {
      /* haven't changed anything,
         didn't open anything.
         I'm not doing anything */
      runTheModel = false;
    }
  }

  if(runTheModel)
  {
    int err_flag;
    char *inputFile;

    inputFile = (char *) malloc (sizeof (char)*(codeFullFilename.Length()));
    strcpy(inputFile, codeFullFilename.mb_str());

    err_flag = onGo(inputFile, NULL, RUNLOAD);
    if(err_flag == ODE_ERROR)
    {
      appLog->Show(true);
    }
    else
    {
      xppautPtr->runWindow = new xppRunWindow(this, codeFullFilename, newTitle, GetPosition());

      Hide();
      xppautPtr->runWindow->Show(TRUE);
    }
  }
}

void xppMainWindow::OnAbout(wxCommandEvent &event)
{
  wxString msg;
  msg.Printf(_T("XPPAUT, a wxWidgets port.\n"));

  wxMessageBox(msg, _T("About XPPAUT"), wxOK, this);
}

void xppMainWindow::OnSetFont(wxCommandEvent &event)
{
  wxTextAttr myTextAttr = mainCode->GetTextCtrl()->GetDefaultStyle();

  wxFontData *fontData = new wxFontData();
  fontData->SetInitialFont(myTextAttr.GetFont());

  wxFontDialog *dlg = new wxFontDialog(this, fontData);

  if(dlg->ShowModal() == wxID_OK)
  {
    wxTextAttr newTextAttr(myTextAttr.GetTextColour(),myTextAttr.GetBackgroundColour(), dlg->GetFontData().GetChosenFont());

    mainCode->GetTextCtrl()->SetStyle(0, mainCode->GetTextCtrl()->GetLastPosition(), newTextAttr); 
/*
    redrawCurrentGraph(currentCanvas);
    s->Refresh();
*/
  }
}
