/*
xppParamsWin_wx.cpp

Window that shows all initial data

*/
#include <wx/wxprec.h>

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif

#include <math.h>

#include "xppRunWindow_wx.h"

#include "xppParamsWin_wx.h"

BEGIN_EVENT_TABLE(xppParamsWin, wxDialog)
  EVT_BUTTON (PARAM_OK, xppParamsWin::OnOk )
  EVT_BUTTON (PARAM_APPLY, xppParamsWin::OnApply )
  EVT_BUTTON (PARAM_APPLY_GO, xppParamsWin::OnApplyGo )
  EVT_BUTTON (PARAM_DEFAULT, xppParamsWin::OnDefault )
  EVT_BUTTON (PARAM_CANCEL, xppParamsWin::OnCancel )
  EVT_CLOSE( xppParamsWin::OnCloseWindow )
END_EVENT_TABLE()

xppParamsWin::xppParamsWin(xppRunWindow *p)
  : wxDialog(p, -1, _T("Parameters"), wxDefaultPosition)
{
  int i;
  int n = NUPAR;
  wxString tmp;
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);
  labels = new wxStaticText * [n];
  inputs = new wxTextCtrl * [n];

  applyButton = new wxButton(panel, PARAM_APPLY, _T("Apply"));
   applyGoButton = new wxButton(panel, PARAM_APPLY_GO, _T("Go"));
  okButton = new wxButton(panel, PARAM_OK, _T("OK"));
  defaultButton = new wxButton(panel, PARAM_DEFAULT, _T("Default"));
  cancelButton = new wxButton(panel, PARAM_CANCEL, _T("Cancel"));

  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  // make the grid approximately square
  gridSizer = new wxGridSizer(2 * (int)sqrt(n));

  for(i=0; i<n; i++)
  {
    tmp.Printf(_T("%s="), upar_names[i]);
    labels[i] = new wxStaticText(panel, -1, tmp);

    tmp.Printf(_T("%.16g"), default_val[i]);
    inputs[i] = new wxTextCtrl(panel, -1, tmp, wxDefaultPosition,
      wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));

    gridSizer->Add(labels[i], 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(inputs[i]);
  }

  buttonSizer->Add(applyButton, 0, wxALL, 3);
  buttonSizer->Add(applyGoButton, 0, wxALL, 3);
  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(defaultButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void xppParamsWin::OnOk(wxCommandEvent &WXUNUSED(event))
{
  Hide();
  UpdateValues();
}

void xppParamsWin::OnApply(wxCommandEvent &WXUNUSED(event))
{
  UpdateValues();
}

void xppParamsWin::OnApplyGo(wxCommandEvent &WXUNUSED(event))
{
  UpdateValues();
  parent->Go();
}

void xppParamsWin::OnDefault(wxCommandEvent &WXUNUSED(event))
{
  int i;
  int n = NUPAR;
  wxString tmp;
  double value;
  char name[11];

  for(i=0; i<n; i++)
  {
    value = default_val[i];
#ifdef wxUSE_UNICODE
    sprintf(name, labels[i]->GetLabel().mb_str());
#else
    sprintf(name, labels[i]->GetLabel().c_str());
#endif
    set_val(name, value);

    /* as long as we're here, update the display */
    tmp.Printf(_T("%.16g"), value);
    inputs[i]->SetValue(tmp);
  }
}

void xppParamsWin::OnCancel(wxCommandEvent &WXUNUSED(event))
{
  Hide();
  /* Reset unapplied changes */
  UpdateDisplay();
}

void xppParamsWin::OnCloseWindow(wxCloseEvent &WXUNUSED(event))
{
  Hide();
  /* Reset unapplied changes */
  UpdateDisplay();
}

void xppParamsWin::UpdateValues()
{
  int i;
  double testValue;
  int n = NUPAR;
  wxString tmpInput, tmpLabel, msg;
  bool convertFlag;
  int assignFlag;

  for(i=0; i<n; i++)
  {
    tmpInput = inputs[i]->GetValue();
    tmpLabel = labels[i]->GetLabel();

    convertFlag = tmpInput.ToDouble(&testValue);

    if(convertFlag)
    {
      assignFlag = set_val(upar_names[i], testValue);

      if(assignFlag == 0)
      {
        msg = _T("Couldn't assign ") + tmpLabel + tmpInput + _T(" to a double.");
        wxMessageDialog dlg(parent, msg, _T("Error"), wxOK, wxDefaultPosition);
        dlg.ShowModal();
      }
    }
    else
    {
      msg = _T("Couldn't convert ") + tmpLabel + _T("=") + tmpInput + _T(" to a double.");
      wxMessageDialog dlg(parent, msg, _T("Error"), wxOK, wxDefaultPosition);
      dlg.ShowModal();
    }
  }
}

void xppParamsWin::UpdateDisplay()
{
  int i;
  int n = NUPAR;
  wxString tmp;
  double value;
//  char name[11];

  for(i=0; i<n; i++)
  {
    //sprintf(name, labels[i]->GetLabel().c_str());
    get_val(upar_names[i], &value);

    tmp.Printf(_T("%g"), value);
    inputs[i]->SetValue(tmp);
  }
}


