/*
xppPlotCanvas_wx.cpp

Creates a canvas for plotting
*/

#include <wx/wxprec.h>

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif
#include <wx/listimpl.cpp>
#include <wx/notebook.h>

#include "Ccode/struct.h"
#include "interfaceCPP_wx.h"
#include "xppRunWindow_wx.h"
#include "xppPlotCanvas_wx.h"

// complete the class declaration for the plot list class
WX_DEFINE_LIST(plotList);

IMPLEMENT_DYNAMIC_CLASS(xppPlotCanvas, wxPanel)

BEGIN_EVENT_TABLE(xppPlotCanvas, wxPanel)
  EVT_PAINT(  xppPlotCanvas::OnPaint)
  EVT_MOUSE_EVENTS( xppPlotCanvas::OnMouse)
  EVT_SIZE( xppPlotCanvas::OnSize)
END_EVENT_TABLE()

xppPlotCanvas::xppPlotCanvas(wxWindow *parent, xppRunWindow *runPtr, int id)
  : wxPanel(parent, -1)
{
  runWindow = runPtr;
  index = id;

  bigFont = new wxFont(-1, wxMODERN, wxNORMAL, wxNORMAL);
  smallFont = new wxFont(-1, wxMODERN, wxNORMAL, wxNORMAL);

  gettingUserClick = FALSE;

  // DC stuff
  plotDC = new wxClientDC();


  graphPtr = &graph[index];
  setCurrentGraph (this);
}

void xppPlotCanvas::Retitle()
{
  re_title ();
}

wxString xppPlotCanvas::getTitle()
{
  wxString tmp;
  tmp.Printf(_T("%s"), graphPtr->gr_title);
  return tmp;
}

wxClientDC* xppPlotCanvas::GetDC()
{
  return plotDC;
}

void xppPlotCanvas::DrawPlot(wxDC &dc)
{
  /* Set up initial colors and styles */
  wxPen paintPen;
  wxBrush paintBrush(runWindow->GetNormalColour(11), wxSOLID);

  paintPen.SetWidth(graphPtr->plottedLines[0].width);
  if(graphPtr->plottedLines[0].style == -1)
    paintPen.SetStyle(wxDOT);
  else
    paintPen.SetStyle(wxSOLID);

  paintPen.SetColour(runWindow->GetNormalColour(graphPtr->plottedLines[0].color));

  dc.SetFont(*smallFont);
  dc.SetTextBackground(runWindow->GetNormalColour(11));
  dc.SetTextForeground(runWindow->GetNormalColour(0));

  dc.SetPen(paintPen);
  dc.SetBackground(paintBrush);
  setSmallFontParams(&dc);

  /* Draw everything */
  dc.Clear();

  int i, n;

  n = graphPtr->numPoints;
  for(i=0; i<n; i++)
  {
    paintPen.SetColour(runWindow->GetNormalColour(graphPtr->plottedPoints[i].color));
    dc.SetPen(paintPen);

    dc.DrawPoint
    (
      graphPtr->plottedPoints[i].x,
      graphPtr->plottedPoints[i].y
    );
  }

  n = graphPtr->numLines;
  for(i=0; i<n; i++)
  {
    paintPen.SetWidth(graphPtr->plottedLines[i].width);

    if(graphPtr->plottedLines[i].style == -1)
      paintPen.SetStyle(wxDOT);
    else
    {
      paintPen.SetStyle(wxSOLID);
      paintPen.SetCap(wxCAP_BUTT);
      paintPen.SetJoin(wxJOIN_BEVEL);
    }

    paintPen.SetColour(runWindow->GetNormalColour(graphPtr->plottedLines[i].color));
    paintPen.SetWidth(1);

    dc.SetPen(paintPen);

    dc.DrawLine
    (
      graphPtr->plottedLines[i].x1,
      graphPtr->plottedLines[i].y1,
      graphPtr->plottedLines[i].x2,
      graphPtr->plottedLines[i].y2
    );
  }

  n = graphPtr->numFArcs;
  for(i=0; i<n; i++)
  {
    paintPen.SetColour(runWindow->GetNormalColour(graphPtr->plottedFArcs[i].color));
    dc.SetPen(paintPen);

    paintBrush.SetColour(runWindow->GetNormalColour(graphPtr->plottedFArcs[i].color));
    dc.SetBrush(paintBrush);

    dc.DrawCircle(
      graphPtr->plottedFArcs[i].x,
      graphPtr->plottedFArcs[i].y,
      graphPtr->plottedFArcs[i].r
    );
  }
  paintBrush.SetColour(runWindow->GetNormalColour(11));

  n = graphPtr->numRects;
  for(i=0; i<n; i++)
  {
    paintPen.SetColour(runWindow->GetNormalColour(graphPtr->plottedRects[i].color));
    dc.SetPen(paintPen);

    paintBrush.SetColour(runWindow->GetNormalColour(graphPtr->plottedRects[i].color));
    dc.SetBrush(paintBrush);

    dc.DrawRectangle(
      graphPtr->plottedRects[i].x,
      graphPtr->plottedRects[i].y,
      graphPtr->plottedRects[i].w,
      graphPtr->plottedRects[i].h
    );
  }
  paintBrush.SetColour(runWindow->GetNormalColour(11));

  dc.SetPen(*wxWHITE_PEN);
  n = graphPtr->numTexts;
  wxString tmp;
  for(i=0; i<n; i++)
  {
    tmp.Printf(_T("%s"), graphPtr->plottedTexts[i].string),
    dc.DrawText
    (
      tmp,
      graphPtr->plottedTexts[i].x,
      graphPtr->plottedTexts[i].y
    );
  }
}

void xppPlotCanvas::DrawBitmap(wxBitmap *bitmap)
{
  wxMemoryDC memoryDC;
  memoryDC.SelectObject(*bitmap);

  memoryDC.SetBackground(*wxWHITE_BRUSH);
  memoryDC.Clear();

  DrawPlot(memoryDC);
}

void xppPlotCanvas::OnPaint(wxPaintEvent &event)
{
  wxPaintDC paintDC(this);

  DrawPlot(paintDC);
}

void xppPlotCanvas::OnSize(wxSizeEvent &event)
{
  resizedGraph(this);

  /* The xppPlotCanvas windows can resize in any order
     making MyGraph impossible to track, so we reset it to
     the currently selected tab */

  runWindow->SetMyGraph();
}

void xppPlotCanvas::OnMouse(wxMouseEvent &event)
{
  wxString msg;
  float x, y;

  if(graphPtr->ThreeDFlag)
  {
    if(event.LeftDown())
    {
      dragStart = event.GetPosition();
      theta = graphPtr->Theta;
      phi = graphPtr->Phi;
    }
    else if(event.LeftUp())
    {
      redraw_the_graph();
      Refresh();
    }
    else if(event.Dragging())
    {
      dragChange = dragStart - event.GetPosition();
      graphPtr->Theta = theta + (double) dragChange.x;
      graphPtr->Phi = phi + (double) dragChange.y;
      
      ClearBackground();
      redraw_cube(graphPtr->Theta, graphPtr->Phi);
      redraw_the_graph();
      Refresh();

      msg.Printf(_T("Theta=%f  |  Phi=%f"), graphPtr->Theta, graphPtr->Phi);
    }
  }
  else
  {
    if(event.LeftDown())
    {
      if(gettingUserClick)
      {
        clickedX = event.m_x;
        clickedY = event.m_y;
        gettingUserClick = FALSE;
        runWindow->GotMouseClick();
      }
      else if(gettingUserBox)
      {
        dragStart = event.GetPosition();
      }
    }
    else if(event.LeftUp())
    {
      if(gettingUserBox)
      {
        boxX1 = dragStart.x;
        boxY1 = dragStart.y;
        boxX2 = event.m_x;
        boxY2 = event.m_y;
        runWindow->GotMouseClick();
      }
    }
    else if(event.Dragging())
    {
      if(gettingUserBox)
      {
        wxClientDC dc(this);
        DrawPlot(dc);
        wxPen boxPen;
        boxPen.SetColour(runWindow->GetNormalColour(0));
        dc.SetPen(boxPen);

        wxPoint topLeft = dragStart;
        wxPoint topRight = wxPoint(event.m_x, dragStart.y);
        wxPoint bottomLeft = wxPoint(dragStart.x, event.m_y);
        wxPoint bottomRight = event.GetPosition();

        dc.DrawLine(topLeft, topRight);
        dc.DrawLine(topLeft, bottomLeft);
        dc.DrawLine(topRight, bottomRight);
        dc.DrawLine(bottomLeft, bottomRight);
      }
    }

    else
    {
      scale_to_real(event.m_x, event.m_y, &x, &y);
      msg.Printf(_T("(%f, %f)"), x , y);
    }
  }

  runWindow->SetStatusText(msg);
}

void xppPlotCanvas::ZoomFit()
{

}

int xppPlotCanvas::GetIndex()
{
  return index;
}

wxFont* xppPlotCanvas::GetBigFont()
{
  return bigFont;
}

wxFont* xppPlotCanvas::GetSmallFont()
{
  return smallFont;
}

void xppPlotCanvas::SetBigFont(wxFont font)
{
  delete bigFont;
  bigFont = new wxFont(font.GetPointSize(), font.GetFamily(), font.GetStyle(),
                font.GetWeight(), font.GetUnderlined(), font.GetFaceName());
}

void xppPlotCanvas::SetSmallFont(wxFont font)
{
  delete smallFont;
  smallFont = new wxFont(font.GetPointSize(), font.GetFamily(), font.GetStyle(),
                  font.GetWeight(), font.GetUnderlined(), font.GetFaceName());
}

void xppPlotCanvas::UserClicked(int *x, int *y)
{
    *x = clickedX;
    *y = clickedY;
}

void xppPlotCanvas::UserBox(int *x1, int *y1, int *x2, int *y2)
{
  *x1 = boxX1;
  *y1 = boxY1;
  *x2 = boxX2;
  *y2 = boxY2;
}

void xppPlotCanvas::SetUserClickMode(int mode)
{
  if(mode == 0)
  {
    gettingUserClick = true;
    gettingUserBox = false;
  }
  else if(mode == 1)
  {
    gettingUserClick = false;
    gettingUserBox = true;
  }
}

void xppPlotCanvas::UnsetUserClickMode()
{
  gettingUserClick = false;
  gettingUserBox = false;
}
