/*
xppRunWindow_wx.cpp

Creates a new tabbed window containing the plots panel
*/
#include <wx/wxprec.h>

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif

#include <wx/bitmap.h>
#include <wx/image.h>
#include <wx/fontdlg.h>
#include <wx/notebook.h>
#include <wx/listimpl.cpp>
#include <wx/accel.h>

#include "Ccode/menudrive.h"
#include "Ccode/menus.h"
#include "Ccode/autlim.h"
#include "interfaceCPP_wx.h"
#include "xppLogWindow_wx.h"
#include "xppAutoWindow_wx.h"
#include "xppPlotCanvas_wx.h"
#include "xppDataWindow_wx.h"
#include "xppMainWindow_wx.h"
#include "xppInitDataWin_wx.h"
#include "xppParamsWin_wx.h"
#include "xppBndryWin_wx.h"
#include "xppEditRHSWin_wx.h"
#include "xppSlidersWin_wx.h"
#include "xppColorDialog_wx.h"
#include "xppDialogs_wx.h"

#include "xppRunWindow_wx.h"

extern xppLogWindow *appLog;
extern "C" double MyData[MAXODE];

BEGIN_EVENT_TABLE(xppRunWindow, wxFrame)
  /* Export submenu */
  EVT_MENU (RUN_EXPORT_IMG,  xppRunWindow::OnExportImg  )
  EVT_MENU (RUN_EXPORT_CPS,  xppRunWindow::OnExportCPS  )
  EVT_MENU (RUN_EXPORT_BWPS, xppRunWindow::OnExportBWPS )

  /* File menu */
  EVT_MENU (RUN_CLONE, xppRunWindow::OnClone)
  EVT_MENU (RUN_WRITE_SET, xppRunWindow::OnWriteSet)
  EVT_MENU (RUN_READ_SET, xppRunWindow::OnReadSet)
  EVT_MENU (RUN_AUTO, xppRunWindow::OnRunAuto)
  EVT_MENU (RUN_CLOSE,      xppRunWindow::OnClose     )
  EVT_MENU (RUN_EXIT,       xppRunWindow::OnExit      )

  /* Edit menu */
  EVT_MENU (RUN_EDIT_RHS, xppRunWindow::OnEditRHS )
  EVT_MENU (RUN_COPY, xppRunWindow::OnCopy )
  EVT_MENU (RUN_SMALL_FONT, xppRunWindow::OnSmallFont )
  EVT_MENU (RUN_EDIT_COLOR, xppRunWindow::OnEditColor )

  /* Shoot submenu */
  EVT_MENU (RUN_SHOOT_B1, xppRunWindow::OnShootB1 )
  EVT_MENU (RUN_SHOOT_B2, xppRunWindow::OnShootB2 )
  EVT_MENU (RUN_SHOOT_B3, xppRunWindow::OnShootB3 )
  EVT_MENU (RUN_SHOOT_B4, xppRunWindow::OnShootB4 )

  /* BVP submenu */
  EVT_MENU (RUN_BVP_SHOW, xppRunWindow::OnBVPShow )
  EVT_MENU (RUN_BVP_NO_SHOW, xppRunWindow::OnBVPNoShow )
  EVT_MENU (RUN_BVP_RANGE, xppRunWindow::OnBVPRange )

  /* Equilibria submenu */
  EVT_MENU (RUN_EQ_GO, xppRunWindow::OnEqGo )
  EVT_MENU (RUN_EQ_MOUSE, xppRunWindow::OnEqMouse )
  EVT_MENU (RUN_EQ_RANGE, xppRunWindow::OnEqRange )
  EVT_MENU (RUN_EQ_MONTECARLO, xppRunWindow::OnEqMonteCarlo )

  /* Run menu */
  EVT_MENU (RUN_GO, xppRunWindow::OnGo )
  EVT_MENU (RUN_LAST, xppRunWindow::OnLast )
  EVT_MENU (RUN_MOUSE, xppRunWindow::OnMouse )
  EVT_MENU (RUN_MICE, xppRunWindow::OnMice )
  EVT_MENU (RUN_MORE_TIME,    xppRunWindow::OnContinue )
  EVT_MENU (RUN_INT_RANGE, xppRunWindow::OnIntRange )
  EVT_MENU (RUN_INT_2RANGE, xppRunWindow::OnInt2Range )

  /* Graphics menu */
  EVT_MENU (RUN_VIEW, xppRunWindow::OnView )
  EVT_MENU (RUN_FIT, xppRunWindow::OnFit )
  EVT_MENU (RUN_ZOOM_IN, xppRunWindow::OnZoomIn )
  EVT_MENU (RUN_ZOOM_OUT, xppRunWindow::OnZoomOut )
  EVT_MENU (RUN_NEW_WIN,   xppRunWindow::OnNewPlot )
  EVT_MENU (RUN_KEEP_CURVE, xppRunWindow::OnKeepCurve )
  EVT_MENU (RUN_EDIT_CURVES, xppRunWindow::OnEditCurves )
  EVT_MENU (RUN_REDRAW, xppRunWindow::OnRestore )
  EVT_TOOL (RUN_ERASE,       xppRunWindow::OnErase )
  EVT_MENU (RUN_TOONS, xppRunWindow::OnToons )
  EVT_MENU (RUN_ARRAY_PLOT, xppRunWindow::OnArrayPlot )

  /* Stochastics submenu */
  EVT_MENU (RUN_SC_NEW, xppRunWindow::OnSCNew )
  EVT_MENU (RUN_SC_MANY, xppRunWindow::OnSCMany )
  EVT_MENU (RUN_SC_MEAN, xppRunWindow::OnSCMean )
  EVT_MENU (RUN_SC_VAR, xppRunWindow::OnSCVar )
  EVT_MENU (RUN_SC_HIST, xppRunWindow::OnSCHist )
  EVT_MENU (RUN_SC_FOURIER, xppRunWindow::OnSCFourier )
  EVT_MENU (RUN_SC_DATA_BACK, xppRunWindow::OnSCDataBack )

  /* Averaging submenu */
  EVT_MENU (RUN_AVG_NEW, xppRunWindow::OnAvgNew )
  EVT_MENU (RUN_AVG_CREATE_HF, xppRunWindow::OnAvgCreateHf )
  EVT_MENU (RUN_AVG_ADJOINT, xppRunWindow::OnAvgAdjoint )
  EVT_MENU (RUN_AVG_HF, xppRunWindow::OnAvgHf )
  EVT_MENU (RUN_AVG_ORBIT, xppRunWindow::OnAvgOrbit )

  /* Numerics menu */
  EVT_MENU (RUN_INT_PARS, xppRunWindow::OnIntPars )
  EVT_MENU (RUN_CONTROL_PARS, xppRunWindow::OnControlPars )
  EVT_MENU (RUN_COLORIZE, xppRunWindow::OnColorize )
  EVT_MENU (RUN_POINCARE_MAP, xppRunWindow::OnPoincareMap )
  EVT_MENU (RUN_RUELLE_PLOT, xppRunWindow::OnRuellePlot )
  EVT_MENU (RUN_TORUS, xppRunWindow::OnTorus )
  EVT_MENU (RUN_TRANSPOSE, xppRunWindow::OnTranspose )

  /* Phaseplane menu */
  EVT_MENU (RUN_NULLCLINES, xppRunWindow::OnNullclines )
  EVT_MENU (RUN_NO_NULLCLINES, xppRunWindow::OnNoNullclines )
  EVT_MENU (RUN_DIRECT_FLD, xppRunWindow::OnDirectFld )
  EVT_MENU (RUN_SCALED_DIRECT_FLD, xppRunWindow::OnScaledDirectFld )
  EVT_MENU (RUN_FLOW, xppRunWindow::OnFlow )
  EVT_MENU (RUN_REDRAW_NCLINE, xppRunWindow::OnRedrawNcline )
  EVT_MENU (RUN_COLOR_PHASE, xppRunWindow::OnColorPhase )
  EVT_MENU (RUN_NO_FLD, xppRunWindow::OnNoFld )
  EVT_MENU (RUN_GRID_PARAMS, xppRunWindow::OnGridParams )

  /* Window menu */
  EVT_MENU (RUN_DATA_WIN,        xppRunWindow::OnRunData   )
  EVT_MENU (RUN_ICS_WIN,         xppRunWindow::OnICsWin    )
  EVT_MENU (RUN_PARAMS_WIN,      xppRunWindow::OnParamsWin )
  EVT_MENU (RUN_BNDRY_CONDS_WIN, xppRunWindow::OnBndryWin  )
  EVT_MENU (RUN_SLIDERS_WIN, xppRunWindow::OnSlidersWin    )
  EVT_MENU (RUN_LOG_WIN,         xppRunWindow::OnShowLog   )

  /* Help menu */
  EVT_MENU (RUN_HELP,           xppRunWindow::OnHelp )
  EVT_MENU (wxID_ABOUT,         xppRunWindow::OnAbout )

  EVT_CLOSE(                    xppRunWindow::OnCloseWindow )
  EVT_NOTEBOOK_PAGE_CHANGED(RUN_NOTEBOOK, xppRunWindow::OnNotebook )
  EVT_NOTEBOOK_PAGE_CHANGING(RUN_NOTEBOOK, xppRunWindow::OnNotebookCheck )
  EVT_MENU(RUN_MOUSE_ESCAPE, xppRunWindow::OnEscapeKey )
END_EVENT_TABLE()

xppRunWindow::xppRunWindow(xppMainWindow *w, const wxString &filename, const wxString &runTitle, wxPoint pos)
  : wxFrame(NULL, 3823, runTitle, pos)
{
  mainWindow = w;
  inputFile = filename;
  runPanel = (wxPanel *) NULL;
  runNotebook = (wxNotebook *) NULL;
  runAutoWin = (xppAutoWindow *) NULL;
  runDataWin = (xppDataWindow *) NULL;
  runInitDataWin = (xppInitDataWin *) NULL;
  runParamsWin = (xppParamsWin *) NULL;
  runBndryWin = (xppBndryWin *) NULL;
  runEditRHSWin = (xppEditRHSWin *) NULL;
  runSlidersWin = (xppSlidersWin *) NULL;
  runPlotList = new plotList(wxKEY_INTEGER);

  delayedMouseCommand = -1;
  autoIsRunning = false;

  wxAcceleratorEntry entries[1];
  entries[0].Set(wxACCEL_NORMAL, WXK_ESCAPE, RUN_MOUSE_ESCAPE);
  wxAcceleratorTable accelTable(1, entries);

  SetAcceleratorTable(accelTable);

  normalColours[0] = wxColour(*wxBLACK);
  normalColours[1] = wxColour(*wxRED);
  normalColours[2] = wxColour(239, 101, 0);
  normalColours[3] = wxColour(255, 166, 0);
  normalColours[4] = wxColour(255, 203, 0);
  normalColours[5] = wxColour(255, 255, 0);
  normalColours[6] = wxColour(198, 235, 74);
  normalColours[7] = wxColour(*wxGREEN);
  normalColours[8] = wxColour(0, 255, 255);
  normalColours[9] = wxColour(*wxBLUE);
  normalColours[10] = wxColour(156, 32, 239);
  normalColours[11] = wxColour(*wxWHITE);

#if wxUSE_MENUS
  // Export submenu
  wxMenu *runExport = new wxMenu();
  runExport->Append(RUN_EXPORT_IMG, _T("Image..."), _T("Save graph to an image file"));
  runExport->Append(RUN_EXPORT_CPS, _T("Color PostScript..."), _T("Save graph to color postscript"));
  runExport->Append(RUN_EXPORT_BWPS, _T("BW PostScript..."), _T("Save graph to black and white postscript"));

  // File menu
  wxMenu *runFile = new wxMenu();
  runFile->Append(RUN_CLONE, _T("Save model..."), _T("Save current values to cloned ODE file"));
  runFile->AppendSeparator();
  runFile->Append(RUN_WRITE_SET, _T("Write set..."), _T("Save settings to a set file"));
  runFile->Append(RUN_READ_SET, _T("Read set..."), _T("Read settings from a set file"));
  runFile->AppendSeparator();
  runFile->Append(RUN_EXPORT, _T("Export"), runExport, _T(""));
//  runFile->Append(RUN_PRINT, _T("Print...(NW)"), _T("Print graph"));
//  runFile->Append(RUN_CLOSE, _T("Close\tCtrl-W"), _T("Close the run window"));
//  runFile->AppendSeparator();
  runFile->AppendSeparator();
  runFile->Append(RUN_EXIT, _T("Exit\tCtrl-Q"), _T("Quit XPPAUT"));

  // Edit menu
  wxMenu *runEdit = new wxMenu();
  runEdit->Append(RUN_DATA_WIN, _T("Data..."), _T("View and edit run data"));
  runEdit->AppendSeparator();
  runEdit->Append(RUN_EDIT_RHS, _T("Right Hand Sides..."), _T("View and edit equation right hand sides"));
  runEdit->Append(RUN_ICS_WIN, _T("Initial Conditions..."), _T("View and edit initial conditions"));
  runEdit->Append(RUN_PARAMS_WIN, _T("Parameters..."), _T("View and edit parameters"));
  runEdit->Append(RUN_BNDRY_CONDS_WIN, _T("Boundary Conditions..."), _T("View boundary conditions"));
  runEdit->AppendSeparator();
  runEdit->Append(RUN_SLIDERS_WIN, _T("Sliders..."), _T("Change model parameters with sliders"));
  runEdit->AppendSeparator();
  runEdit->Append(RUN_SMALL_FONT, _T("Font..."), _T("Change the plotting font"));
  runEdit->AppendSeparator();
  runEdit->Append(RUN_EDIT_COLOR, _T("Colormap..."), _T("Change the colormap"));
  
  // Shoot submenu
  wxMenu *runShoot = new wxMenu();
  runShoot->Append(RUN_SHOOT_B1, _T("Branch 1"), _T(""));
  runShoot->Append(RUN_SHOOT_B2, _T("Branch 2"), _T(""));
  runShoot->Append(RUN_SHOOT_B3, _T("Branch 3"), _T(""));
  runShoot->Append(RUN_SHOOT_B4, _T("Branch 4"), _T(""));

  // BVP submenu
  wxMenu *runBVP = new wxMenu();
  runBVP->Append(RUN_BVP_SHOW, _T("Show"), _T("Show each step of iteration"));
  runBVP->Append(RUN_BVP_NO_SHOW, _T("No Show"), _T("Don't show any but final step"));
  runBVP->Append(RUN_BVP_RANGE, _T("Range..."), _T("Solve BVP over range of parameters"));

  // Equilibria submenu
  wxMenu *runEq = new wxMenu();
  runEq->Append(RUN_EQ_GO, _T("Equilib Go"), _T(""));
  runEq->Append(RUN_EQ_MOUSE, _T("Mouse"), _T("ESC to quit. Use mouse to guess fixed point"));
  runEq->Append(RUN_EQ_RANGE, _T("Range..."), _T("Find fixed points over range of parameter"));
  runEq->Append(RUN_EQ_MONTECARLO, _T("Monte Carlo..."), _T("Monte carlo search for fixed points"));

  // Run menu
  wxMenu *runRun = new wxMenu();
  runRun->Append(RUN_GO, _T("Go\tCtrl-G"), _T("Run model with current settings"));
  runRun->Append(RUN_LAST, _T("Last\tCtrl-L"), _T("Pick up from last step of previous solution"));
  runRun->Append(RUN_MORE_TIME, _T("More Time...\tCtrl-T"), _T("Continue integration for more time"));
  runRun->AppendSeparator();
  runRun->Append(RUN_MOUSE, _T("Mouse"), _T("ESC to quit. Run model with mouse ICs (on phaseplane)"));
  runRun->Append(RUN_MICE, _T("Mice"), _T("ESC to quit. Run model with many mouse ICs (on phaseplane)"));
  runRun->AppendSeparator();
  runRun->Append(RUN_INT_RANGE, _T("Range Integrate..."), _T("Integrate over a parameter or IC range"));
  runRun->Append(RUN_INT_2RANGE, _T("2-Param Range..."), _T("Integrate over two parameter or IC ranges"));
  runRun->AppendSeparator();
  runRun->Append(RUN_SHOOT, _T("Shoot (NW)"), runShoot, _T(""));
  runRun->AppendSeparator();
  runRun->Append(RUN_BVP, _T("BVP"), runBVP, _T(""));
  runRun->AppendSeparator();
  runRun->Append(RUN_EQ, _T("Equilibria"), runEq, _T(""));
  runRun->AppendSeparator();
  runRun->Append(RUN_AUTO, _T("AUTO"), _T("Run AUTO, the bifurcation and continuation package"));

  // Graphics menu
  wxMenu *runGraphics = new wxMenu();
  runGraphics->Append(RUN_VIEW, _T("View...\tCtrl-V"), _T("Change the view window"));
  runGraphics->AppendSeparator();
  runGraphics->Append(RUN_FIT, _T("Fit"), _T("Fit view window to graph"));
  runGraphics->Append(RUN_ZOOM_IN, _T("Zoom in"), _T("ESC to quit. Zoom in with mouse"));
  runGraphics->Append(RUN_ZOOM_OUT, _T("Zoom out"), _T("ESC to quit. Zoom out with mouse"));
  runGraphics->AppendSeparator();
  runGraphics->Append(RUN_NEW_WIN, _T("New Window"), _T("Create new tab"));
  runGraphics->Append(RUN_KEEP_CURVE, _T("Keep curve... (NW)"), _T("Save a plotted curve for redrawing"));
  runGraphics->Append(RUN_EDIT_CURVES, _T("Add/edit curves..."), _T("Add to or edit saved curves"));
  runGraphics->AppendSeparator();
  runGraphics->Append(RUN_REDRAW, _T("Redraw\tCtrl-R"), _T("Redraw current graph"));
  runGraphics->Append(RUN_ERASE, _T("Erase\tCtrl-E"), _T("Erase graphs"));
  runGraphics->AppendSeparator();
  runGraphics->Append(RUN_TOONS, _T("Toons... (NW)"), _T(""));
  runGraphics->AppendSeparator();
  runGraphics->Append(RUN_ARRAY_PLOT, _T("ArrayPlot... (NW)"), _T(""));

  // Stochastics submenu
  wxMenu *runSC = new wxMenu();
  runSC->Append(RUN_SC_NEW, _T("New Seed"), _T(""));
  runSC->Append(RUN_SC_MANY, _T("Compute Many"), _T(""));
  runSC->Append(RUN_SC_MEAN, _T("Mean"), _T(""));
  runSC->Append(RUN_SC_VAR, _T("Variance"), _T(""));
  runSC->Append(RUN_SC_HIST, _T("Histogram"), _T(""));
  runSC->Append(RUN_SC_FOURIER, _T("Fourier"), _T(""));
  runSC->Append(RUN_SC_DATA_BACK, _T("Data back"), _T(""));

  // Averaging submenu
  wxMenu *runAvg = new wxMenu();
  runAvg->Append(RUN_AVG_NEW, _T("New Adjoint"), _T(""));
  runAvg->Append(RUN_AVG_CREATE_HF, _T("Create H-function"), _T(""));
  runAvg->Append(RUN_AVG_ADJOINT, _T("Adjoint"), _T(""));
  runAvg->Append(RUN_AVG_HF, _T("H-function"), _T(""));
  runAvg->Append(RUN_AVG_ORBIT, _T("Orbit"), _T(""));
  
  // Numerics menu
  wxMenu *runNumerics = new wxMenu();
  runNumerics->Append(RUN_INT_PARS, _T("Int. Pars...\tCtrl-I"), _T("Edit integration parameters"));
  runNumerics->Append(RUN_CONTROL_PARS, _T("Control pars..."), _T("Edit other parameters"));
  runNumerics->AppendSeparator();
  runNumerics->Append(RUN_COLORIZE, _T("Colorize... (NW)"), _T(""));
  runNumerics->Append(RUN_POINCARE_MAP, _T("Poincare map... (NW)"), _T(""));
  runNumerics->Append(RUN_RUELLE_PLOT, _T("Ruelle plot... (NW)"), _T(""));
  runNumerics->AppendSeparator();
  runNumerics->Append(RUN_SC, _T("Stochastics (NW)"), runSC, _T(""));
  runNumerics->AppendSeparator();
  runNumerics->Append(RUN_TORUS, _T("Torus... (NW)"), _T(""));
  runNumerics->AppendSeparator();
//  runNumerics->Append(RUN_TRANSPOSE, _T("Transpose... (NW)"), _T(""));
//  runNumerics->AppendSeparator();
  runNumerics->Append(RUN_AVG, _T("Averaging (NW)"), runAvg,_T(""));

  // Phaseplane menu
  wxMenu *runPhase = new wxMenu();
  runPhase->Append(RUN_GRID_PARAMS, _T("Grid parameters..."), _T("Change field plotting parameters"));
  runPhase->AppendSeparator();
  runPhase->Append(RUN_NULLCLINES, _T("Nullclines"), _T("Plot nullclines"));
  runPhase->Append(RUN_NO_NULLCLINES, _T("No Nullclines"), _T("Turn off nullcline plotting"));
  runPhase->AppendSeparator();
  runPhase->Append(RUN_DIRECT_FLD, _T("Direct Fld"), _T("Plot direction field"));
  runPhase->Append(RUN_SCALED_DIRECT_FLD, _T("Scaled Direct Fld"), _T("Plot scaled direction field"));
  runPhase->Append(RUN_FLOW, _T("Flow"), _T("Plot flow field. Does not redraw when resizing."));
//  runPhase->Append(RUN_REDRAW_NCLINE, _T("Redraw ncline"), _T(""));
//  runPhase->Append(RUN_COLOR_PHASE, _T("Color Phaseplane"), _T("Color the phaseplane on a grid"));
  runPhase->Append(RUN_NO_FLD, _T("No Fields"), _T("Turn off field plotting"));

  // Help menu
  wxMenu *runHelp = new wxMenu();
  runHelp->Append(RUN_LOG_WIN, _T("Log"), _T("Show the program log"));
//  runHelp->Append(RUN_HELP, _T("Help (NW)"), _T("Online help"));
  runHelp->AppendSeparator();
  runHelp->Append(wxID_ABOUT, _T("About..."), _T("About XPPAUT"));

  // Create the run window menu bar
  wxMenuBar *runMenuBar = new wxMenuBar();
  runMenuBar->Append(runFile, _T("&File"));
  runMenuBar->Append(runEdit, _T("&Edit"));
  runMenuBar->Append(runRun, _T("&Run"));
  runMenuBar->Append(runGraphics, _T("&Graphics"));
  runMenuBar->Append(runNumerics, _T("&Numerics"));
  runMenuBar->Append(runPhase, _T("&Phaseplane"));
  runMenuBar->Append(runHelp, _T("&Help"));

  SetMenuBar(runMenuBar);
#endif // wxUSE_MENUS

#if wxUSE_STATUSBAR
  CreateStatusBar();
  SetStatusText(_T("Run Window"));
#endif //wxUSE_STATUSBAR

  runPanel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  runNotebook = new wxNotebook(runPanel, RUN_NOTEBOOK);

  // the wxPlotPanel for the ever present graph[0]
  // The plot is a canvas (treat as a control) and needs to be put in a panel
  wxPanel *myPanel = new wxPanel(runNotebook);
  xppPlotCanvas *myPlot = new xppPlotCanvas(myPanel, this, 0);
  wxBoxSizer *mySizer = new wxBoxSizer(wxVERTICAL);

  runPlotList->Append((long)0, myPlot);
  currentCanvas = myPlot;

  mySizer->Add(myPlot, 1, wxEXPAND);
  myPanel->SetSizer(mySizer);

  // Various ever present (but not always shown) Windows
  runDataWin = new xppDataWindow(this);
  runInitDataWin = new xppInitDataWin(this);
  runParamsWin = new xppParamsWin(this);
  runBndryWin = new xppBndryWin(this);
  runEditRHSWin = new xppEditRHSWin(this);
  runSlidersWin = new xppSlidersWin(this);

  runNotebook->AddPage(myPanel, myPlot->getTitle() );

  // Sizer stuff
  runFrameSizer = new wxBoxSizer(wxVERTICAL);
  runPanelSizer = new wxBoxSizer(wxVERTICAL);

  runFrameSizer->SetMinSize(wxSize(640,480));
  runFrameSizer->Add(runPanelSizer, 1, wxEXPAND);

  runNotebookSizer = new wxNotebookSizer(runNotebook);

  runPanelSizer->Add(runNotebookSizer, 1, wxEXPAND | wxALL);
  runPanelSizer->Layout();

  runPanel->SetSizer(runFrameSizer);
  runPanel->SetAutoLayout(TRUE);

  runFrameSizer->Fit(this);
}

/* Export submenu */

void xppRunWindow::OnExportImg(wxCommandEvent &event)
{
/*
File types supported:
Dialog filter index
-------------------
        0            wxBITMAP_TYPE_BMP
        1            wxBITMAP_TYPE_JPEG
        2            wxBITMAP_TYPE_PNG
        3            wxBITMAP_TYPE_PCX (tries 8-bit, 24-bit otherwise).
        4            wxBITMAP_TYPE_PNM (as raw RGB always).
        5            wxBITMAP_TYPE_XPM

In wxImage but not supported:
wxBITMAP_TYPE_ICO 	Windows Icon File - graphs are too big
wxBITMAP_TYPE_CUR 	Windows cursor file - graphs are too big
wxBITMAP_TYPE_TIFF 	UNISYS legal issues
*/

  wxFileDialog dialog(this,_T("Export Graph As..."),_T(""),_T(""),_T("Bitmap (*.bmp)|*.bmp|JPEG (*.jpg)|*.jpg|Portable Network Graphic (*.png)|*.png|PCX (*.pcx)|*.pcx|PNM (raw RGB *.pnm)|*.pnm)|XPM Bitmap (*.xpm)|*.xpm"), wxSAVE | wxOVERWRITE_PROMPT); 

  if (dialog.ShowModal() == wxID_OK)
  {
    int type = wxBITMAP_TYPE_BMP;
    int width, height;
    wxString filename = dialog.GetFilename();
    wxString directory = dialog.GetDirectory();
    wxString fullFilename = dialog.GetPath();
    wxString extension;

    currentCanvas->GetClientSize(&width, &height);

    wxBitmap exportBitmap(width, height);

    currentCanvas->DrawBitmap(&exportBitmap);

    wxImage exportImage;
    exportImage = exportBitmap.ConvertToImage();

    switch(dialog.GetFilterIndex())
    {
      case 0:
        extension = _T(".bmp");
        type = wxBITMAP_TYPE_BMP;
        break;
      case 1:
        extension = _T(".jpg");
        type = wxBITMAP_TYPE_JPEG;
        break;
      case 2:
        extension = _T(".png");
        type = wxBITMAP_TYPE_PNG;
        break;
      case 3:
        extension = _T(".pcx");
        type = wxBITMAP_TYPE_PCX;
        break;
      case 4:
        extension = _T(".pnm");
        type = wxBITMAP_TYPE_PNM;
        break;
      case 5:
        extension = _T(".xpm");
        type = wxBITMAP_TYPE_XPM;
        break;
    }

    /* did they give the right extension? */
    if(filename.Right(4).IsSameAs(extension, false))
    {
      /* if so, save with their filename */
      exportImage.SaveFile(fullFilename, type);
    }
    else
    {
      /* if not, append the extension */
      exportImage.SaveFile(directory + filename + extension, type);
    }
  }

}

void xppRunWindow::OnExportCPS(wxCommandEvent &event)
{
  wxFileDialog dialog(this,_T("Color Postscript..."),_T(""),_T(""),_T("postscript (*.ps)|*.ps|All files (*.*)|*.*"), wxSAVE|wxOVERWRITE_PROMPT);

  if (dialog.ShowModal() == wxID_OK)
  {
    char filename[256];
#ifdef wxUSE_UNICODE
    sprintf(filename, dialog.GetPath().mb_str());
#else
    sprintf(filename, dialog.GetPath().c_str());
#endif
    if(ps_init(filename, 1))
    {
      ps_restore();
    }
  }
}

void xppRunWindow::OnExportBWPS(wxCommandEvent &event)
{
  wxFileDialog dialog(this,_T("BW Postscript..."),_T(""),_T(""),_T("postscript (*.ps)|*.ps|All files (*.*)|*.*"), wxSAVE|wxOVERWRITE_PROMPT);

  if (dialog.ShowModal() == wxID_OK)
  {
    char filename[256];
#ifdef wxUSE_UNICODE
    sprintf(filename, dialog.GetPath().mb_str());
#else
    sprintf(filename, dialog.GetPath().c_str());
#endif
    if(ps_init(filename, 0))
    {
      ps_restore();
    }
  }
}

/* File menu */

void xppRunWindow::OnClone(wxCommandEvent &event)
{
  wxFileDialog dialog(this,_T("Clone ODE"),_T(""),_T(""),_T("ODE (*.ode)|*.ode|All files (*.*)|*.*"), wxSAVE|wxOVERWRITE_PROMPT);

  if (dialog.ShowModal() == wxID_OK)
  {
    char *filename;
    filename = (char *) malloc (sizeof (char)*(dialog.GetPath().Length()));
#ifdef wxUSE_UNICODE
    strcpy(filename, dialog.GetPath().mb_str());
#else
    strcpy(filename, dialog.GetPath().c_str());
#endif
    clone_ode(filename);
  }
}

void xppRunWindow::OnWriteSet(wxCommandEvent &event)
{
  wxFileDialog dialog(this,_T("Save Set file..."),_T(""),_T(""),_T("set (*.set)|*.set|All files (*.*)|*.*"), wxSAVE|wxOVERWRITE_PROMPT);

  if (dialog.ShowModal() == wxID_OK)
  {
    char filename[256];
#ifdef wxUSE_UNICODE
    sprintf(filename, dialog.GetPath().mb_str());
#else
    sprintf(filename, dialog.GetPath().c_str());
#endif
    do_lunch(0, filename);
  }
}

void xppRunWindow::OnReadSet(wxCommandEvent &event)
{
  wxFileDialog dialog(this,_T("Open Set file..."),_T(""),_T(""),_T("set (*.set)|*.set|All files (*.*)|*.*"), wxOPEN|wxFILE_MUST_EXIST);

  if (dialog.ShowModal() == wxID_OK)
  {
    char filename[256];
#ifdef wxUSE_UNICODE
    sprintf(filename, dialog.GetPath().mb_str());
#else
    sprintf(filename, dialog.GetPath().c_str());
#endif
    do_lunch(1, filename);

    /* update the tab title - parameters could change */
    runNotebook->SetPageText(runNotebook->GetSelection(), currentCanvas->getTitle());
    runFrameSizer->RecalcSizes();
    Refresh();
    currentCanvas->Refresh();
  }
}

void xppRunWindow::OnRunAuto(wxCommandEvent &event)
{
  if(autoIsRunning)
  {
    runAutoWin->Raise();
  }
  else
  {
    if(NODE > NAUTO)
    {
      wxString tmp;
      tmp.Printf(_T("AUTO restricted to less than %d variables"), NAUTO);
      wxMessageDialog dialog(this, tmp, _T("Error"), wxOK);
      dialog.ShowModal();
    }
    else
    {
      runAutoWin = new xppAutoWindow(this, GetPosition());
      runAutoWin->Show();
      autoIsRunning = true;
    }
  }
}

void xppRunWindow::AutoShutDown()
{
  autoIsRunning = false;
}

void xppRunWindow::OnClose(wxCommandEvent &event)
{
  wxMessageDialog dialog(this, _T("Are you sure you want to quit?"),_T("Close"), wxYES_NO);

  if(dialog.ShowModal() == wxID_YES)
  {
    Destroy();
    appLog->Destroy();
    mainWindow->Destroy();
  }
}

void xppRunWindow::OnCloseWindow(wxCloseEvent &event)
{
  wxMessageDialog dialog(this, _T("Are you sure you want to quit?"),_T("Close"), wxYES_NO);

  if(dialog.ShowModal() == wxID_YES)
  {
    Destroy();
    appLog->Destroy();
    mainWindow->Destroy();
  }
}

void xppRunWindow::OnExit(wxCommandEvent &event)
{
  wxMessageDialog dialog(this, _T("Are you sure you want to quit?"),_T("Exit"), wxYES_NO);

  if(dialog.ShowModal() == wxID_YES)
  {
    Destroy();
    appLog->Destroy();
    mainWindow->Destroy();
  }
}

/* Edit menu */

void xppRunWindow::OnEditRHS(wxCommandEvent &event)
{
  if(runEditRHSWin->IsShown())
    runEditRHSWin->Hide();
  else
    runEditRHSWin->Show(true);
}

void xppRunWindow::OnCopy(wxCommandEvent &event)
{

}

void xppRunWindow::OnSmallFont(wxCommandEvent &event)
{
  wxFontData *smallData = new wxFontData();
  smallData->SetInitialFont(*(currentCanvas->GetSmallFont()));

  wxFontDialog *dlg = new wxFontDialog(this, smallData);

  if(dlg->ShowModal() == wxID_OK)
  {
    currentCanvas->SetSmallFont(dlg->GetFontData().GetChosenFont()); 
    redrawCurrentGraph(currentCanvas);
    currentCanvas->Refresh();
  }
}

void xppRunWindow::OnEditColor(wxCommandEvent &event)
{
  xppColorDialog dlg(this);

  if(dlg.ShowModal() == wxID_OK)
  {
    dlg.SetColorMap();
  }
}

/* Shoot submenu */

void xppRunWindow::OnShootB1(wxCommandEvent &event)
{

}

void xppRunWindow::OnShootB2(wxCommandEvent &event)
{

}

void xppRunWindow::OnShootB3(wxCommandEvent &event)
{

}

void xppRunWindow::OnShootB4(wxCommandEvent &event)
{

}

/* BVP submenu */

void xppRunWindow::OnBVPShow(wxCommandEvent &event)
{
  find_bvp_com(2);
  currentCanvas->Refresh();
}

void xppRunWindow::OnBVPNoShow(wxCommandEvent &event)
{
  find_bvp_com(1);
  currentCanvas->Refresh();
}

void xppRunWindow::OnBVPRange(wxCommandEvent &event)
{
  BVPRangeDlg dlg(this);

  if(dlg.ShowModal() == wxID_OK)
  {
    dlg.UpdateValues();
    find_bvp_com(0);
    redrawCurrentGraph(currentCanvas);
    currentCanvas->Refresh();
  }
}

/* Equilibria submenu */

void xppRunWindow::OnEqGo(wxCommandEvent &event)
{
  find_equilib_com(0);
  currentCanvas->Refresh();
}

void xppRunWindow::OnEqMouse(wxCommandEvent &event)
{
  int iv, jv;

  iv = MyGraph->xv[0] - 1;
  jv = MyGraph->yv[0] - 1;

  if (iv <  0 
  ||  iv >= NODE
  ||  jv <  0
  ||  jv >= NODE
  ||  jv == iv
  ||  MyGraph->grtype >= 5)
  {
    wxString msg;
    msg.Printf(_T("Not in useable 2D plane..."));

    wxMessageBox(msg, _T("Error"), wxOK, this);
  }
  else
  {
    delayedMouseCommand = 51;
    GetMouseClick();
  }
}

void xppRunWindow::OnEqRange(wxCommandEvent &event)
{
  FixedPtRangeDlg dlg(this);

  if(dlg.ShowModal() == wxID_OK)
  {
    dlg.UpdateValues();
    find_equilib_com(2);
    redraw_graph_noerase();
    currentCanvas->Refresh();
  }
}

void xppRunWindow::OnEqMonteCarlo(wxCommandEvent &event)
{
  MonteCarloEqDlg dlg(this);

  if(dlg.ShowModal() == wxID_OK)
  {
    int append;

    dlg.UpdateValues();
    if(dlg.GetAppendFlag())
      append = 1;
    else
      append = 0;

    do_monte_carlo_search(append, 1);
    
    redrawCurrentGraph(currentCanvas);
    currentCanvas->Refresh();
  }
}

/* Run menu */

// splitting this event into two functions
// so I can call it elsewhere
void xppRunWindow::Go()
{
  do_init_data(M_IG);
  redrawCurrentGraph(currentCanvas);
  currentCanvas->Refresh();

  /* update the tab title-parameters could change */
  runNotebook->SetPageText(runNotebook->GetSelection(), currentCanvas->getTitle());
  runFrameSizer->RecalcSizes();
  Refresh();
}

void xppRunWindow::OnGo(wxCommandEvent &event)
{
  Go();
}

void xppRunWindow::OnLast(wxCommandEvent &event)
{
  run_the_commands(2);
  currentCanvas->Refresh();
}

void xppRunWindow::OnMouse(wxCommandEvent &event)
{
  delayedMouseCommand = 5;
  GetMouseClick();
}

void xppRunWindow::OnMice(wxCommandEvent &event)
{
  delayedMouseCommand = 6;
  GetMouseClick();
}

void xppRunWindow::OnContinue(wxCommandEvent &event)
{
  cont_integ();
  currentCanvas->Refresh();
}

void xppRunWindow::OnIntRange(wxCommandEvent &event)
{
  IntRangeDlg dlg(this);

  if(dlg.ShowModal() == wxID_OK)
  {
    double *x = &MyData[0];
    dlg.UpdateValues();
    do_range(x,0);
    currentCanvas->Refresh();
  }
}

void xppRunWindow::OnInt2Range(wxCommandEvent &event)
{
  Int2RangeDlg dlg(this);
  if(dlg.ShowModal() == wxID_OK)
  {
    double *x = &MyData[0];
    dlg.UpdateValues();
    do_range(x,1);
    currentCanvas->Refresh();
  }
}

/* Graphics Menu */

void xppRunWindow::OnView(wxCommandEvent &event)
{
  ViewDlg dlg(this);
  if(dlg.ShowModal() == wxID_OK)
  {
    dlg.UpdateValues();
    Go();
    if(dlg.GetFitFlag())
    {
      fit_window();
    }
    /* redraw nullclines with newly computed grid */
    if(MyGraph->Nullrestore)
    {
      new_clines_com(3);
      new_clines_com(0);
      new_clines_com(2);
    }
    redrawCurrentGraph(currentCanvas);
    currentCanvas->Refresh();
  }
}

void xppRunWindow::OnFit(wxCommandEvent &event)
{
  fit_window();
  currentCanvas->Refresh();
}

void xppRunWindow::OnZoomIn(wxCommandEvent &event)
{
  delayedMouseCommand = 71;
  GetMouseBox();
}

void xppRunWindow::OnZoomOut(wxCommandEvent &event)
{
  delayedMouseCommand = 72;
  GetMouseBox();
}

void xppRunWindow::OnNewPlot(wxCommandEvent &event)
{
  create_a_pop();

  wxPanel *myPanel = new wxPanel(runNotebook);
  xppPlotCanvas *myPlot = new xppPlotCanvas(myPanel, this, current_pop);
  wxBoxSizer *mySizer = new wxBoxSizer(wxVERTICAL);

  runPlotList->Append(current_pop, myPlot);
  currentCanvas = myPlot;

  mySizer->Add(myPlot, 1, wxEXPAND);
  myPanel->SetSizer(mySizer);

  runNotebook->AddPage(myPanel, myPlot->getTitle());
  runNotebook->AdvanceSelection();
}

void xppRunWindow::OnKeepCurve(wxCommandEvent &event)
{

}

void xppRunWindow::OnEditCurves(wxCommandEvent &event)
{
  AddEditCurvesDlg dlg(this);
  if(dlg.ShowModal() == wxID_OK)
  {
    dlg.UpdateValues();
    Go();
    currentCanvas->Refresh();
  }
}

void xppRunWindow::OnRestore(wxCommandEvent &event)
{
  run_the_commands(M_R);
  currentCanvas->Refresh();
}


void xppRunWindow::OnErase(wxCommandEvent &event)
{
  run_the_commands(M_EE);
  currentCanvas->Refresh();
}

void xppRunWindow::OnToons(wxCommandEvent &event)
{

}

void xppRunWindow::OnArrayPlot(wxCommandEvent &event)
{

}

/* Stochastics submenu */

void xppRunWindow::OnSCNew(wxCommandEvent &event)
{

}

void xppRunWindow::OnSCMany(wxCommandEvent &event)
{

}

void xppRunWindow::OnSCMean(wxCommandEvent &event)
{

}

void xppRunWindow::OnSCVar(wxCommandEvent &event)
{

}

void xppRunWindow::OnSCHist(wxCommandEvent &event)
{

}

void xppRunWindow::OnSCFourier(wxCommandEvent &event)
{

}

void xppRunWindow::OnSCDataBack(wxCommandEvent &event)
{

}

/* Averaging submenu */

void xppRunWindow::OnAvgNew(wxCommandEvent &event)
{

}

void xppRunWindow::OnAvgCreateHf(wxCommandEvent &event)
{

}

void xppRunWindow::OnAvgAdjoint(wxCommandEvent &event)
{

}

void xppRunWindow::OnAvgHf(wxCommandEvent &event)
{

}

void xppRunWindow::OnAvgOrbit(wxCommandEvent &event)
{

}

/* Numerics menu */

void xppRunWindow::OnIntPars(wxCommandEvent &event)
{
  IntParsDlg dlg(this);

  if(dlg.ShowModal() == wxID_OK)
  {
    dlg.UpdateValues();
  } 
}

void xppRunWindow::OnControlPars(wxCommandEvent &event)
{
  ControlParamsDlg dlg(this);

  if(dlg.ShowModal() == wxID_OK)
  {
    dlg.UpdateValues();
  }
}

void xppRunWindow::OnColorize(wxCommandEvent &event)
{

}

void xppRunWindow::OnPoincareMap(wxCommandEvent &event)
{

}

void xppRunWindow::OnRuellePlot(wxCommandEvent &event)
{

}

void xppRunWindow::OnTorus(wxCommandEvent &event)
{

}

void xppRunWindow::OnTranspose(wxCommandEvent &event)
{

}

/* Phaseplane menu */
void xppRunWindow::OnNullclines(wxCommandEvent &event)
{
  new_clines_com(0); /* draw nullclines */
  new_clines_com(2); /* turn on auto redraw */
  currentCanvas->Refresh();
}

void xppRunWindow::OnNoNullclines(wxCommandEvent &event)
{
  new_clines_com(3);
  redrawCurrentGraph(currentCanvas);
  currentCanvas->Refresh();
}

void xppRunWindow::OnDirectFld(wxCommandEvent &event)
{
  direct_field_com(0);
  currentCanvas->Refresh();
}

void xppRunWindow::OnScaledDirectFld(wxCommandEvent &event)
{
  direct_field_com(4);
  currentCanvas->Refresh();
}

void xppRunWindow::OnFlow(wxCommandEvent &event)
{
  direct_field_com(1);
  currentCanvas->Refresh();
}

void xppRunWindow::OnRedrawNcline(wxCommandEvent &event)
{

}

void xppRunWindow::OnColorPhase(wxCommandEvent &event)
{
  direct_field_com(3);
  currentCanvas->Refresh();
}

void xppRunWindow::OnNoFld(wxCommandEvent &event)
{
  direct_field_com(2);
  redrawCurrentGraph(currentCanvas);
  currentCanvas->Refresh();
}

void xppRunWindow::OnGridParams(wxCommandEvent &event)
{
  PhaseParamsDlg dlg(this);

  if(dlg.ShowModal() == wxID_OK)
  {
    dlg.UpdateValues();
  }
}

/* More edit menu */

void xppRunWindow::OnRunData(wxCommandEvent &event)
{
  if(runDataWin->IsShown())
    runDataWin->Hide();
  else
    runDataWin->Show(true);
}

void xppRunWindow::OnICsWin(wxCommandEvent &event)
{
  if(runInitDataWin->IsShown())
    runInitDataWin->Hide();
  else
    runInitDataWin->Show(true);
}

void xppRunWindow::OnParamsWin(wxCommandEvent &event)
{
  if(runParamsWin->IsShown())
    runParamsWin->Hide();
  else
    runParamsWin->Show(true);
}

void xppRunWindow::OnBndryWin(wxCommandEvent &event)
{
  if(runBndryWin->IsShown())
    runBndryWin->Hide();
  else
    runBndryWin->Show(true);
}

void xppRunWindow::OnSlidersWin(wxCommandEvent &event)
{
  if(runSlidersWin->IsShown())
    runBndryWin->Hide();
  else
    runSlidersWin->Show(true);
}

void xppRunWindow::OnShowLog(wxCommandEvent &event)
{
  appLog->Show(true);
}

/* Help menu */

void xppRunWindow::OnHelp(wxCommandEvent &event)
{

}

void xppRunWindow::OnAbout(wxCommandEvent &event)
{
  wxString msg;
  msg.Printf(_T("XPPAUT, a wxWidgets port.\n"));

  wxMessageBox(msg, _T("About XPPAUT"), wxOK, this);
}

/* misc */

void xppRunWindow::SetMyGraph ()
{
  setCurrentGraph(currentCanvas);
}

void xppRunWindow::OnNotebook(wxNotebookEvent &event)
{
  int i = event.GetSelection();
  if(i != -1)
  {
    currentCanvas = (xppPlotCanvas*)(runNotebook->GetPage(i)->GetChildren().GetFirst()->GetData());
    if(currentCanvas != NULL)
      setCurrentGraph(currentCanvas);
  }
}

void xppRunWindow::OnNotebookCheck(wxNotebookEvent &event)
{
  if(delayedMouseCommand != -1)
  {
    event.Veto();
    return;
  }
  else
  {
    event.Skip();
  }
}
void xppRunWindow::GetMouseClick()
{
  wxMenuBar *menubar = GetMenuBar();
  int i;
  int numMenus = menubar->GetMenuCount();

  /* disable the menus */
  for(i=0; i<numMenus; i++)
  {
    menubar->EnableTop(i, FALSE);
  }

  /* Notebook page changing events will be vetoed */

  currentCanvas->SetUserClickMode(0);
}

void xppRunWindow::GetMouseBox()
{
  wxMenuBar *menubar = GetMenuBar();
  int i;
  int numMenus = menubar->GetMenuCount();

  /* disable the menus */
  for(i=0; i<numMenus; i++)
  {
    menubar->EnableTop(i, FALSE);
  }

  /* Notebook page changing events will be vetoed */

  currentCanvas->SetUserClickMode(1);
}

void xppRunWindow::GotMouseClick()
{
  wxMenuBar *menubar = GetMenuBar();
  int i;
  int numMenus = menubar->GetMenuCount();

  if(delayedMouseCommand == 5   /* Mouse IC */
  || delayedMouseCommand == 51  /* Mouse Equilib */
  || delayedMouseCommand == 71  /* Zoom in */
  || delayedMouseCommand == 72) /* Zoom out */
  {
    /* re-enable the menus */
    for(i=0; i<numMenus; i++)
    {
      menubar->EnableTop(i, TRUE);
    }

    run_the_commands(delayedMouseCommand);
    delayedMouseCommand = -1;
    currentCanvas->UnsetUserClickMode();
    currentCanvas->Refresh();
  }
  else if(delayedMouseCommand == 6) /* Mice ICs */
  {
    run_the_commands(5);
    currentCanvas->Refresh();
    currentCanvas->SetUserClickMode(0);
  }
}

void xppRunWindow::OnEscapeKey(wxCommandEvent &event)
{
  if(delayedMouseCommand != -1)
  {
    wxMenuBar *menubar = GetMenuBar();
    int i;
    int numMenus = menubar->GetMenuCount();

    /* re-enable the menus */
    for(i=0; i<numMenus; i++)
    {
      menubar->EnableTop(i, TRUE);
    }

    delayedMouseCommand = -1;
    currentCanvas->UnsetUserClickMode();
    currentCanvas->Refresh();
  }
}

wxColour xppRunWindow::GetNormalColour(int n)
{
  if(n<0 || n>12)
    return normalColours[0];
  else
    return normalColours[n];
}

void xppRunWindow::SetNormalColour(int n, wxColour c)
{
  if(n<0 || n>12)
    return;
  else
    normalColours[n] = c;

  currentCanvas->Refresh();
}

xppPlotCanvas* xppRunWindow::GetCurrentCanvas()
{
  return currentCanvas;
}

void xppRunWindow::UpdateAllEditWins()
{
  runInitDataWin->UpdateDisplay();
  runParamsWin->UpdateDisplay();
  runEditRHSWin->UpdateDisplay();
}
