/*
xppRunWindow.h

Defines the tabbed window which will hold all of the runs of a .ode file
*/

#ifndef __XPPRUNWINH__
#define __XPPRUNWINH__

#include <wx/notebook.h>
#include "Ccode/struct.h"

class plotList;
class xppPlotCanvas;
class xppMainWindow;
class xppAutoWindow;
class xppDataWindow;
class xppInitDataWin;
class xppParamsWin;
class xppBndryWin;
class xppEditRHSWin;
class xppSlidersWin;

class xppRunWindow : public wxFrame
{
  public:
    xppRunWindow(xppMainWindow *w, const wxString &file, const wxString &runTitle, wxPoint pos);

    /* Export submenu */
    void OnExportImg  (wxCommandEvent &event);
    void OnExportCPS  (wxCommandEvent &event);
    void OnExportBWPS (wxCommandEvent &event);

    /* File menu */
    void OnClone     (wxCommandEvent &event);
    void OnWriteSet  (wxCommandEvent &event);
    void OnReadSet   (wxCommandEvent &event);
    void OnRunAuto   (wxCommandEvent &event);
    void OnClose     (wxCommandEvent &event);
    void OnExit      (wxCommandEvent &event);

    /* Edit menu */
    void OnEditRHS    (wxCommandEvent &event);
    void OnRunData    (wxCommandEvent &event);
    void OnICsWin     (wxCommandEvent &event);
    void OnParamsWin  (wxCommandEvent &event);
    void OnBndryWin   (wxCommandEvent &event);
    void OnSlidersWin (wxCommandEvent &event);
    void OnCopy       (wxCommandEvent &event);
    void OnSmallFont  (wxCommandEvent &event);
    void OnEditColor  (wxCommandEvent &event);

    /* Shoot submenu */
    void OnShootB1 (wxCommandEvent &event);
    void OnShootB2 (wxCommandEvent &event);
    void OnShootB3 (wxCommandEvent &event);
    void OnShootB4 (wxCommandEvent &event);

    /* BVP submenu */
    void OnBVPShow   (wxCommandEvent &event);
    void OnBVPNoShow (wxCommandEvent &event);
    void OnBVPRange  (wxCommandEvent &event);

    /* Equilibria submenu */
    void OnEqGo         (wxCommandEvent &event);
    void OnEqMouse      (wxCommandEvent &event);
    void OnEqRange      (wxCommandEvent &event);
    void OnEqMonteCarlo (wxCommandEvent &event);

    /* Run menu */
    void OnGo        (wxCommandEvent &event);
    void OnLast      (wxCommandEvent &event);
    void OnMouse     (wxCommandEvent &event);
    void OnMice      (wxCommandEvent &event);
    void OnContinue  (wxCommandEvent &event);
    void OnIntRange  (wxCommandEvent &event);
    void OnInt2Range (wxCommandEvent &event);

    /* Graphics menu */
    void OnView       (wxCommandEvent &event);
    void OnFit        (wxCommandEvent &event);
    void OnZoomIn     (wxCommandEvent &event);
    void OnZoomOut    (wxCommandEvent &event);
    void OnNewPlot    (wxCommandEvent &event);
    void OnKeepCurve  (wxCommandEvent &event);
    void OnEditCurves (wxCommandEvent &event);
    void OnRestore    (wxCommandEvent &event);
    void OnErase      (wxCommandEvent &event);
    void OnToons      (wxCommandEvent &event);
    void OnArrayPlot  (wxCommandEvent &event);

    /* Stochastics submenu */
    void OnSCNew       (wxCommandEvent &event);
    void OnSCMany      (wxCommandEvent &event);
    void OnSCMean      (wxCommandEvent &event);
    void OnSCVar       (wxCommandEvent &event);
    void OnSCHist      (wxCommandEvent &event);
    void OnSCFourier   (wxCommandEvent &event);
    void OnSCDataBack  (wxCommandEvent &event);

    /* Averaging submenu */
    void OnAvgNew      (wxCommandEvent &event);
    void OnAvgCreateHf (wxCommandEvent &event);
    void OnAvgAdjoint  (wxCommandEvent &event);
    void OnAvgHf       (wxCommandEvent &event);
    void OnAvgOrbit    (wxCommandEvent &event);

    /* Numerics menu */
    void OnIntPars     (wxCommandEvent &event);
    void OnControlPars (wxCommandEvent &event);
    void OnColorize    (wxCommandEvent &event);
    void OnPoincareMap (wxCommandEvent &event);
    void OnRuellePlot  (wxCommandEvent &event);
    void OnTorus       (wxCommandEvent &event);
    void OnTranspose   (wxCommandEvent &event);

    /* Phaseplane menu */
    void OnNullclines      (wxCommandEvent &event);
    void OnNoNullclines    (wxCommandEvent &event);
    void OnDirectFld       (wxCommandEvent &event);
    void OnScaledDirectFld (wxCommandEvent &event);
    void OnFlow            (wxCommandEvent &event);
    void OnRedrawNcline    (wxCommandEvent &event);
    void OnColorPhase      (wxCommandEvent &event);
    void OnNoFld           (wxCommandEvent &event);
    void OnGridParams      (wxCommandEvent &event);

    /* Window menu */

    /* Help menu */
    void OnShowLog   (wxCommandEvent &event);
    void OnHelp      (wxCommandEvent &event);
    void OnAbout     (wxCommandEvent &event);

    /* Misc */
    void OnNotebook      (wxNotebookEvent &event);
    void OnNotebookCheck (wxNotebookEvent &event);
    void OnCloseWindow   (wxCloseEvent &event);
    void Go();
    void SetMyGraph();
    void GetMouseClick();
    void GotMouseClick();
    void GetMouseBox();
    void OnEscapeKey(wxCommandEvent &event);
    wxColour GetNormalColour(int n);
    void SetNormalColour(int n, wxColour c);
    xppPlotCanvas* GetCurrentCanvas();
    void UpdateAllEditWins();
    void AutoShutDown();

  private:
    xppMainWindow *mainWindow;
    wxString inputFile;

    wxColour normalColours[12];

    xppPlotCanvas *currentCanvas;

    xppAutoWindow *runAutoWin;

    // These windows are made once and then
    // hidden and shown
    xppDataWindow *runDataWin;
    xppInitDataWin *runInitDataWin;
    xppParamsWin *runParamsWin;
    xppBndryWin *runBndryWin;
    xppEditRHSWin *runEditRHSWin;
    xppSlidersWin *runSlidersWin;

    wxPanel *runPanel;
    wxNotebook *runNotebook;
    wxBoxSizer *runFrameSizer;
    wxBoxSizer *runPanelSizer;
    wxNotebookSizer *runNotebookSizer;

    plotList *runPlotList;

    int delayedMouseCommand;
    bool autoIsRunning;

  DECLARE_EVENT_TABLE()
};

enum
{
  /* Export submenu events */
  RUN_EXPORT = 8000,
  RUN_EXPORT_IMG,
  RUN_EXPORT_CPS,
  RUN_EXPORT_BWPS,
  /* File menu events */
  RUN_CLOSE,
  RUN_EXIT,
  RUN_CLONE,
  RUN_WRITE_SET,
  RUN_READ_SET,
  RUN_PRINT,
  RUN_AUTO,
  /* Edit menu events */
  RUN_EDIT_RHS,
  RUN_DATA_WIN,
  RUN_ICS_WIN,
  RUN_PARAMS_WIN,
  RUN_BNDRY_CONDS_WIN,
  RUN_SLIDERS_WIN,
  RUN_COPY,
  RUN_SMALL_FONT,
  RUN_EDIT_COLOR,
  /* Shoot submenu events */
  RUN_SHOOT,
  RUN_SHOOT_B1,
  RUN_SHOOT_B2,
  RUN_SHOOT_B3,
  RUN_SHOOT_B4,
  /* BVP submenu events */
  RUN_BVP,
  RUN_BVP_SHOW,
  RUN_BVP_NO_SHOW,
  RUN_BVP_RANGE,
  /* Equilibria submenu events */
  RUN_EQ,
  RUN_EQ_GO,
  RUN_EQ_MOUSE,
  RUN_EQ_RANGE,
  RUN_EQ_MONTECARLO,
  /* Run menu events */
  RUN_GO,
  RUN_LAST,
  RUN_MOUSE,
  RUN_MICE,
  RUN_NEW_ICS,
  RUN_MORE_TIME,
  RUN_INT_RANGE,
  RUN_INT_2RANGE,
  /* Graphics menu events */
  RUN_VIEW,
  RUN_FIT,
  RUN_ZOOM_IN,
  RUN_ZOOM_OUT,
  RUN_NEW_WIN,
  RUN_KEEP_CURVE,
  RUN_EDIT_CURVES,
  RUN_REDRAW,
  RUN_ERASE,
  RUN_TOONS,
  RUN_ARRAY_PLOT,
  /* Stochastics submenu events */
  RUN_SC,
  RUN_SC_NEW,
  RUN_SC_MANY,
  RUN_SC_MEAN,
  RUN_SC_VAR,
  RUN_SC_HIST,
  RUN_SC_FOURIER,
  RUN_SC_DATA_BACK,
  /* Averaging submenu events */
  RUN_AVG,
  RUN_AVG_NEW,
  RUN_AVG_CREATE_HF,
  RUN_AVG_ADJOINT,
  RUN_AVG_HF,
  RUN_AVG_ORBIT,
  /* Numerics menu events */
  RUN_INT_PARS,
  RUN_CONTROL_PARS,
  RUN_COLORIZE,
  RUN_POINCARE_MAP,
  RUN_RUELLE_PLOT,
  RUN_TORUS,
  RUN_TRANSPOSE,
  /* Phaseplane menu events */
  RUN_NULLCLINES,
  RUN_NO_NULLCLINES,
  RUN_DIRECT_FLD,
  RUN_SCALED_DIRECT_FLD,
  RUN_FLOW,
  RUN_NO_FLD,
  RUN_REDRAW_NCLINE,
  RUN_COLOR_PHASE,
  RUN_GRID_PARAMS,
  /* Help menu events */
  RUN_LOG_WIN,
  RUN_HELP,
  /* about is wxID_ABOUT */

  RUN_NOTEBOOK,
  RUN_MOUSE_ESCAPE,
};

extern "C" {
  int onGo(char *inputFile, char *outputFile, int outputFlag);
  void create_a_pop();
  void cont_integ();
  void xi_vs_t();
  void get_3d_par_com();
  void get_num_par(char ch);
  void fit_window();
  void do_lunch(int f, char *filename);
  void fit_window();
  void new_clines_com(int i);
  void direct_field_com(int c);
  int ps_init(char *filename, int color);
  void ps_restore();
  int do_range(double *x, int flag);
  void find_bvp_com(int com);
  void find_equilib_com(int com);
  void do_monte_carlo_search(int append, int stuffbrowse);
  void redraw_graph_noerase();
  void clone_ode(char *clone);
}


#endif
