/*

xppSliderWin_wx.cpp

Window where parameter varying sliders reside

*/
#include <wx/wxprec.h>

#ifdef __BORLANDC__
  #pragma hdrstop
#endif

#ifndef WX_PRECOMP
  #include <wx/wx.h>
#endif

//#include <stdio.h>
#include <math.h>
#include <wx/slider.h>

#include "xppDialogs_wx.h"
#include "xppRunWindow_wx.h"
#include "xppSlidersWin_wx.h"

BEGIN_EVENT_TABLE(xppSlidersWin, wxDialog)
  EVT_BUTTON(SLIDER_GO, xppSlidersWin::OnGo)
  EVT_BUTTON(SLIDER_ADD_EDIT, xppSlidersWin::OnAddEdit)
  EVT_BUTTON(SLIDER_CLOSE, xppSlidersWin::OnClose)
  EVT_SCROLL(  xppSlidersWin::OnScroll)
  EVT_CLOSE(  xppSlidersWin::OnCloseWindow)
END_EVENT_TABLE()

xppSlidersWin::xppSlidersWin(xppRunWindow *p)
  : wxDialog(p, -1, _T("Sliders"), wxDefaultPosition)
{
  parent = p;

  numSliders = 0;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  int n = NEQ + NUPAR;
  labels = new wxStaticText * [n];
  sliders = new wxSlider * [n];
  types = new int [n];
  params = new int [n];
  values = new double [n];
  mins = new double [n];
  maxes = new double [n];
  steps = new double [n];

  addEditButton = new wxButton(panel, SLIDER_ADD_EDIT, _T("Edit Sliders"));
  goButton = new wxButton(panel, SLIDER_GO, _T("Go"));
  closeButton = new wxButton(panel, SLIDER_CLOSE, _T("Close"));

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(2);

  buttonSizer->Add(addEditButton, 0, wxALL, 3);
  buttonSizer->Add(goButton, 0, wxALL, 3);
  buttonSizer->Add(closeButton, 0, wxALL, 3);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void xppSlidersWin::UpdateValues()
{
/*
  int choice, i;
  int n=MyGraph->nvars;

  for(i=0; i<n; i++)
  {
    choice = axisChoices[3*i]->GetSelection();
    MyGraph->xv[i] = choice;
    
    choice = axisChoices[3*i+1]->GetSelection();
    MyGraph->yv[i] = choice;

    choice = axisChoices[3*i+2]->GetSelection();
    MyGraph->zv[i] = choice;

    choice = styleChoices[i]->GetSelection();
    if(choice == 0)
    {
      MyGraph->line[i] = 1;
    }
    else
    {
      MyGraph->line[i] = 0;
    }

    choice = colorChoices[i]->GetSelection();
    MyGraph->color[i] = choice;

  }
*/
}

void xppSlidersWin::UpdateDisplay()
{
/*
  wxString tmp;
  int i;
  int n=MyGraph->nvars;

  wxString colorStrings[10] = 
{_T("0"), _T("1"), _T("2"), _T("3"), _T("4"), _T("5"), _T("6"), _T("7"), _T("8"), _T("9")};

  curveLabels = new wxStaticText * [MAXPERPLOT];
  axisChoices = new wxChoice * [3*MAXPERPLOT];
  colorChoices = new wxChoice * [MAXPERPLOT];
  colorDisplays = new wxWindow * [MAXPERPLOT];
  styleChoices = new wxChoice * [MAXPERPLOT];

  for(i=0; i<n; i++)
  {
    tmp.Printf("%i", i);
    curveLabels[i] = new wxStaticText(panel, -1, tmp, wxDefaultPosition);

    axisChoices[3*i] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
    axisChoices[3*i+1] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
    axisChoices[3*i+2] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);

    axisChoices[3*i]->Append(_T("Time"));
    axisChoices[3*i+1]->Append(_T("Time"));
    axisChoices[3*i+2]->Append(_T("Time"));

    populateUvarChoice(axisChoices[3*i]);
    populateUvarChoice(axisChoices[3*i+1]);
    populateUvarChoice(axisChoices[3*i+2]);

    axisChoices[3*i]->SetSelection(MyGraph->xv[i]);
    axisChoices[3*i+1]->SetSelection(MyGraph->yv[i]);
    axisChoices[3*i+2]->SetSelection(MyGraph->zv[i]);

    styleChoices[i] = new wxChoice(panel, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
    styleChoices[i]->Append(_T("Line"));
    styleChoices[i]->Append(_T("Point"));

    if(MyGraph->line[i] > 0)
    {
      // This is the line style
      styleChoices[i]->SetSelection(0);
    }
    else if(MyGraph->line[i] == 0)
    {
      // This is the point style
      styleChoices[i]->SetSelection(1);
    }
    else
    {
      // This should be large beeds, < 0 styleChoices[i]->SetSelection(1);
    }

    colorChoices[i] = new wxChoice(panel, AEC_COLOR0 + i, wxDefaultPosition, wxDefaultSize, 10, colorStrings);
    colorChoices[i]->SetSelection(MyGraph->color[i]);

    colorDisplays[i] = new wxWindow(panel, -1);
    colorDisplays[i]->SetBackgroundColour(parent->GetNormalColour(MyGraph->color[i]));

    gridSizer->Add(curveLabels[i], 0, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL | wxALL, 3);
    gridSizer->Add(axisChoices[3*i], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(axisChoices[3*i+1], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(axisChoices[3*i+2], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(styleChoices[i], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(colorChoices[i], 0, wxEXPAND | wxALL, 3);
    gridSizer->Add(colorDisplays[i], 0, wxEXPAND | wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL | wxALL, 3);
  }
*/
}

void xppSlidersWin::OnGo(wxCommandEvent &event)
{
  int i;
  int n = numSliders;

  for(i=0; i<n; i++)
  {
    if(types[i] == 1)
    {
      set_val(upar_names[params[i]], values[i]);
    }
    else
    {
      last_ic[params[i]] = values[i];
    }
  }
  parent->Go();
  parent->UpdateAllEditWins();
}

void xppSlidersWin::OnAddEdit(wxCommandEvent &event)
{
  AddEditSliderDlg dlg(this);

  bool done = false;
  wxString tmp;
  int i, n;
  int choice, value, min, max;

  while(!done)
  {
    if(dlg.ShowModal() == wxID_OK)
    {
      if(dlg.Check())
      {
        for(i=0; i<numSliders; i++)
        {
          labels[i]->Destroy();
          sliders[i]->Destroy();
        }

        n = numSliders = dlg.GetNewNumSliders();
    
        for(i=0; i<n; i++)
        {
          values[i] = dlg.GetSliderValue(i);
          mins[i] = dlg.GetSliderMin(i);
          maxes[i] = dlg.GetSliderMax(i);
          steps[i] = dlg.GetSliderStep(i);
          choice = dlg.GetSliderParam(i);

          if(choice >= NEQ)
          {
            tmp.Printf(_T("%s = %g"), upar_names[choice - NEQ], values[i]);
            types[i] = 1;
            params[i] = choice-NEQ;
          }
          else
          {
            tmp.Printf(_T("%s(0) = %g"), uvar_names[choice], values[i]);
            types[i] = 0;
            params[i] = choice;
          }

          labels[i] = new wxStaticText(panel, -1, tmp, wxDefaultPosition);
    
          max = (int) ceil(maxes[i] / steps[i]);
          min = (int) floor(mins[i] / steps[i]);
          value = (int) (values[i] / steps[i]);
    
          sliders[i] = new wxSlider(panel, SLIDER_SLIDE + i, value, min, max);

          gridSizer->Add(labels[i], 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
          gridSizer->Add(sliders[i], 0, wxEXPAND);
        }
        done = true;
        mainSizer->Fit(this);
      }
    }
    else
    {
      done = true;
    }
  }
}

void xppSlidersWin::OnClose(wxCommandEvent &event)
{
  Hide();
}

void xppSlidersWin::OnCloseWindow(wxCloseEvent &event)
{
  Hide();
}

void xppSlidersWin::OnScroll(wxScrollEvent &event)
{
  wxString tmp;

  int n = event.GetId() - SLIDER_SLIDE;
  int pos = event.GetPosition();

  values[n] = pos * steps[n];

  if(types[n] == 1)
  {
    tmp.Printf(_T("%s = %g"), upar_names[params[n]], values[n]);
    labels[n]->SetLabel(tmp);
  }
  else
  {
    tmp.Printf(_T("%s(0) = %g"), uvar_names[params[n]], values[n]);
    labels[n]->SetLabel(tmp);
  }
}

int xppSlidersWin::GetNumSliders()
{
  return numSliders;
}

int xppSlidersWin::GetSliderParam(int i)
{
  return params[i];
}

int xppSlidersWin::GetSliderType(int i)
{
  return types[i];
}

double xppSlidersWin::GetSliderValue(int i)
{
  return values[i];
}

double xppSlidersWin::GetSliderMin(int i)
{
  return mins[i];
}

double xppSlidersWin::GetSliderMax(int i)
{
  return maxes[i];
}

double xppSlidersWin::GetSliderStep(int i)
{
  return steps[i];
}

/* Add Edit Sliders Dialog */

BEGIN_EVENT_TABLE(AddEditSliderDlg, wxDialog)
  EVT_BUTTON(AES_ADD, AddEditSliderDlg::OnAdd)
  EVT_BUTTON(AES_DEL, AddEditSliderDlg::OnDel)
  EVT_COMMAND_RANGE(AES_CHOICE, AES_CHOICE_LAST, wxEVT_COMMAND_CHOICE_SELECTED, AddEditSliderDlg::OnChoice)
END_EVENT_TABLE()

AddEditSliderDlg::AddEditSliderDlg(xppSlidersWin *p)
  : wxDialog(p, -1, _T("Add Edit Sliders"), wxDefaultPosition)
{
  parent = p;

  panel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxCLIP_CHILDREN | wxNO_BORDER);

  numCurrentSliders = parent->GetNumSliders();
  int n = NEQ + NUPAR;
  int i;

  wxString labelStrings[5] =
{_T("Parameter"),
 _T("Value"),
 _T("Min"),
 _T("Max"),
 _T("Step")};
  
  labels = new wxStaticText * [5];
  inputs = new wxTextCtrl * [4*n];
  paramChoices = new wxChoice * [n];

  for(i=0; i<5; i++)
  {
    labels[i] = new wxStaticText(panel, -1, labelStrings[i], wxDefaultPosition);
  }

  addButton = new wxButton(panel, AES_ADD, _T("Add Slider"));
  delButton = new wxButton(panel, AES_DEL, _T("Delete Last"));
  okButton = new wxButton(panel, wxID_OK, _T("OK"));
  cancelButton = new wxButton(panel, wxID_CANCEL, _T("Cancel"));

  // Sizers
  mainSizer = new wxBoxSizer(wxVERTICAL);
  panelSizer = new wxBoxSizer(wxVERTICAL);
  buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  gridSizer = new wxGridSizer(5);

  for(i=0; i<5; i++)
  {
    gridSizer->Add(labels[i], 0, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL);
  }

  UpdateDisplay();

  buttonSizer->Add(addButton, 0, wxALL, 3);
  buttonSizer->Add(delButton, 0, wxALL, 3);
  buttonSizer->Add(okButton, 0, wxALL, 3);
  buttonSizer->Add(cancelButton, 0, wxALL, 3);

  panelSizer->Add(gridSizer, 1, wxEXPAND);
  panelSizer->Add(buttonSizer, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL, 10);
  panelSizer->Layout();

  mainSizer->Add(panelSizer, 1, wxEXPAND);

  panel->SetSizer(mainSizer);
  panel->SetAutoLayout(true);

  mainSizer->Fit(this);
  CenterOnParent();
}

void AddEditSliderDlg::UpdateDisplay()
{
  wxString tmp;
  int i;

  for(i=0; i<numCurrentSliders; i++)
  {
    paramChoices[i] = new wxChoice(panel, AES_CHOICE+i, wxDefaultPosition, wxDefaultSize, 0, NULL);

    populateUvarChoice(paramChoices[i]);
    populateParamChoice(paramChoices[i]);

    if(parent->GetSliderType(i) == 0)
    {
      paramChoices[i]->SetSelection(parent->GetSliderParam(i));
    }
    else
    {
      paramChoices[i]->SetSelection(parent->GetSliderParam(i) + NEQ);
    }

    inputs[4*i] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
    inputs[4*i+1] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
    inputs[4*i+2] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
    inputs[4*i+3] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));

    tmp.Printf(_T("%g"), parent->GetSliderValue(i));
    inputs[4*i]->SetValue(tmp);
    tmp.Printf(_T("%g"), parent->GetSliderMin(i));
    inputs[4*i+1]->SetValue(tmp);
    tmp.Printf(_T("%g"), parent->GetSliderMax(i));
    inputs[4*i+2]->SetValue(tmp);
    tmp.Printf(_T("%g"), parent->GetSliderStep(i));
    inputs[4*i+3]->SetValue(tmp);

    gridSizer->Add(paramChoices[i], 0, wxEXPAND | wxALL | wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL, 3);
    gridSizer->Add(inputs[4*i], 0, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(inputs[4*i+1], 0, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(inputs[4*i+2], 0, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(inputs[4*i+3], 0, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL);

    mainSizer->Fit(this);
  }
}

void AddEditSliderDlg::OnAdd(wxCommandEvent &event)
{
  int n = NEQ + NUPAR;
  if(numCurrentSliders >= n)
  {
    wxString msg;
    msg.Printf(_T("Can't add any sliders!"));

    wxMessageBox(msg, _T("Error"), wxOK, this);
  }
  else
  {
    int i = numCurrentSliders;
    wxString tmp;

    paramChoices[i] = new wxChoice(panel, AES_CHOICE+i, wxDefaultPosition, wxDefaultSize, 0, NULL);

    populateUvarChoice(paramChoices[i]);
    populateParamChoice(paramChoices[i]);

    //paramChoices[i]->SetSelection(0);

    inputs[4*i] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
    inputs[4*i+1] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
    inputs[4*i+2] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));
    inputs[4*i+3] = new wxTextCtrl(panel, -1, _T(""), wxDefaultPosition, wxDefaultSize, 0, wxTextValidator(wxFILTER_NUMERIC));


    gridSizer->Add(paramChoices[i], 0, wxEXPAND | wxALL | wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL, 3);
    gridSizer->Add(inputs[4*i], 0, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(inputs[4*i+1], 0, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(inputs[4*i+2], 0, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(inputs[4*i+3], 0, wxALIGN_CENTER | wxALIGN_CENTER_VERTICAL);

    numCurrentSliders++;

    mainSizer->Fit(this);
    mainSizer->Layout();
  }
}

void AddEditSliderDlg::OnDel(wxCommandEvent &event)
{
  if(numCurrentSliders <= 0)
  {
    wxString msg;
    msg.Printf(_T("No sliders to delete"));

    wxMessageBox(msg, _T("Error"), wxOK, this);
  }
  else
  {
    numCurrentSliders--;
    int i = numCurrentSliders;
    paramChoices[i]->Destroy();
    inputs[4*i]->Destroy();
    inputs[4*i+1]->Destroy();
    inputs[4*i+2]->Destroy();
    inputs[4*i+3]->Destroy();
    mainSizer->Fit(this);
  }
}

void AddEditSliderDlg::OnChoice(wxCommandEvent &event)
{
  wxString tmp;
  int i = event.GetId()-AES_CHOICE;
  int choice = paramChoices[i]->GetSelection();
  double value;

  if(choice >= NEQ)
  {
    get_val(upar_names[choice - NEQ], &value);
    tmp.Printf(_T("%g"), value);
  }
  else
  {
    value = last_ic[choice];
    tmp.Printf(_T("%g"), value);
  }

  inputs[4*i]->SetValue(tmp);
  tmp.Printf(_T("%g"), value-5);
  inputs[4*i+1]->SetValue(tmp);
  tmp.Printf(_T("%g"), value+5);
  inputs[4*i+2]->SetValue(tmp);

  value = 0.5;
  tmp.Printf(_T("%g"), value);
  inputs[4*i+3]->SetValue(tmp);
}

int AddEditSliderDlg::GetNewNumSliders()
{
  return numCurrentSliders;
}

int AddEditSliderDlg::GetSliderParam(int i)
{
  return paramChoices[i]->GetSelection();
}

double AddEditSliderDlg::GetSliderValue(int i)
{
  wxString tmpInput;
  double tmpDouble;

  tmpInput = inputs[4*i]->GetValue();
  if( stringToDouble(this, tmpInput, labels[1]->GetLabel() ))
    tmpInput.ToDouble(&tmpDouble);

  return tmpDouble;
}

double AddEditSliderDlg::GetSliderMin(int i)
{
  wxString tmpInput;
  double tmpDouble;

  tmpInput = inputs[4*i+1]->GetValue();
  if( stringToDouble(this, tmpInput, labels[2]->GetLabel() ))
    tmpInput.ToDouble(&tmpDouble);

  return tmpDouble;
}

double AddEditSliderDlg::GetSliderMax(int i)
{
  wxString tmpInput;
  double tmpDouble;

  tmpInput = inputs[4*i+2]->GetValue();
  if( stringToDouble(this, tmpInput, labels[3]->GetLabel() ))
    tmpInput.ToDouble(&tmpDouble);

  return tmpDouble;
}

double AddEditSliderDlg::GetSliderStep(int i)
{
  wxString tmpInput;
  double tmpDouble;

  tmpInput = inputs[4*i+3]->GetValue();
  if( stringToDouble(this, tmpInput, labels[4]->GetLabel() ))
    tmpInput.ToDouble(&tmpDouble);

  return tmpDouble;
}

bool AddEditSliderDlg::Check()
{
  wxString msg;
  int i, j;

  for(i=0; i<numCurrentSliders; i++)
  {
    for(j=i+1; j<numCurrentSliders; j++)
    {
      if(paramChoices[i]->GetSelection() == paramChoices[j]->GetSelection())
      {
        msg.Printf(_T("More than one slider defined for parameter %s"), paramChoices[i]->GetStringSelection().c_str());

        wxMessageBox(msg, _T("Error"), wxOK, this);

        return false;
      }
    }

    if(GetSliderMin(i) >= GetSliderMax(i))
    {
      msg.Printf(_T("Error assigning range for parameter %s"), paramChoices[i]->GetStringSelection().c_str());

      wxMessageBox(msg, _T("Error"), wxOK, this);

      return false;
    }

    if(GetSliderValue(i) < GetSliderMin(i)
    || GetSliderValue(i) > GetSliderMax(i))
    {
      msg.Printf(_T("Value is not in range of parameter %s"), paramChoices[i]->GetStringSelection().c_str());

      wxMessageBox(msg, _T("Error"), wxOK, this);

      return false;
    }

    if(GetSliderStep(i) <=0)
    {
      msg.Printf(_T("Step size for parameter %s must be positive"), paramChoices[i]->GetStringSelection().c_str());

      wxMessageBox(msg, _T("Error"), wxOK, this);

      return false;
    }

    if(GetSliderStep(i) >= (GetSliderMax(i) - GetSliderMin(i)))
    {
      msg.Printf(_T("Step size for parameter %s exceeds range size"), paramChoices[i]->GetStringSelection().c_str());

      wxMessageBox(msg, _T("Error"), wxOK, this);

      return false;
    }

  }
  
  return true;
}

