/*
xppSliderWin_wx.h

Defines the window that parameter varying sliders reside
*/

#ifndef __XPPSLIDERWIN__
#define __XPPSLIDERWIN__

#include "Ccode/xpplim.h"

extern "C" int NEQ;
extern "C" int NUPAR;
extern "C" char uvar_names[MAXODE][12]; /* dep. variable names */
extern "C" char upar_names[MAXPAR][11]; /* parameter names */
extern "C" int get_val(char *name, double *value);
extern "C" int set_val(char *name, double value);
extern "C" double last_ic[MAXODE]; /* initial condition values */

class xppSlidersWin : public wxDialog
{
  public:
    xppSlidersWin(xppRunWindow *p);
    void UpdateValues();
    void UpdateDisplay();
    void OnGo(wxCommandEvent &event);
    void OnAddEdit(wxCommandEvent &event);
    void OnClose(wxCommandEvent &event);
    void OnCloseWindow(wxCloseEvent &event);
    void OnScroll(wxScrollEvent &event);

    int GetNumSliders();
    int GetSliderParam(int i);
    int GetSliderType(int i);
    double GetSliderValue(int i);
    double GetSliderMin(int i);
    double GetSliderMax(int i);
    double GetSliderStep(int i);

  private:
    xppRunWindow *parent;
    wxPanel *panel;

    int numSliders;
    int *types;
    int *params;
    double *values;
    double *mins;
    double *maxes;
    double *steps;
    wxStaticText **labels;
//    wxTextCtrl **inputs;
//    wxChoice **paramChoices;
    wxSlider **sliders;

    wxButton *addEditButton;
    wxButton *goButton;
    wxButton *closeButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxGridSizer *gridSizer;
    wxBoxSizer *buttonSizer;

  DECLARE_EVENT_TABLE()
};

enum
{
  SLIDER_ADD_EDIT = 1700,
  SLIDER_GO,
  SLIDER_CLOSE,
  SLIDER_SLIDE /* must always be last */
};

class AddEditSliderDlg : public wxDialog
{
  public:
    AddEditSliderDlg(xppSlidersWin *p);
    void UpdateDisplay();

    void OnAdd(wxCommandEvent &event);
    void OnDel(wxCommandEvent &event);
    void OnChoice(wxCommandEvent &event);

    int GetNewNumSliders();
    int GetSliderParam(int i);
    double GetSliderValue(int i);
    double GetSliderMin(int i);
    double GetSliderMax(int i);
    double GetSliderStep(int i);
    bool Check();

  private:
    xppSlidersWin *parent;
    wxPanel *panel;

    int numCurrentSliders;
    wxStaticText **labels;
    wxTextCtrl **inputs;
    wxChoice **paramChoices;

    wxButton *addButton;
    wxButton *delButton;
    wxButton *okButton;
    wxButton *cancelButton;

    wxBoxSizer *mainSizer;
    wxBoxSizer *panelSizer;
    wxGridSizer *gridSizer;
    wxBoxSizer *buttonSizer;

  DECLARE_EVENT_TABLE()
};

enum
{
  AES_ADD = 1800,
  AES_DEL,
  AES_CHOICE /* must be last */
};

enum
{
  /* MAXODE + MAXPAR */
  AES_CHOICE_LAST = AES_CHOICE + 2350
};


#endif

